/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded;

import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.Nullable;
import org.rhq.core.clientapi.agent.PluginContainerException;
import org.rhq.core.domain.content.PackageType;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceCreationDataType;
import org.rhq.core.domain.resource.ResourceType;
import org.rhq.core.pc.PluginContainer;
import org.rhq.core.pc.inventory.InventoryEventListener;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Events;

import org.jboss.on.embedded.manager.ResourceManager;

/**
 * @author Ian Springer
 */
@Name("inventoryEventListener")
@Scope(ScopeType.APPLICATION)
public class EmbeddedInventoryEventListener implements InventoryEventListener
{
    private final Log log = LogFactory.getLog(this.getClass());

    private boolean navTreeInitialized;

    public void resourcesAdded(Set<Resource> resources)
    {
        for (Resource resource : resources)
        {
            if (this.navTreeInitialized)
                Events.instance().raiseEvent(ResourceManager.RESOURCE_CREATED_EVENT, resource.getResourceType(),
                        resource.getParentResource());
            ResourceType resourceType = resource.getResourceType();
            if (isContentBacked(resourceType))
            {
                PackageType creationPackageType = getCreationPackageType(resourceType);
                if (creationPackageType != null)
                {
                    try
                    {
                        PluginContainer.getInstance().getContentManager().executeResourcePackageDiscoveryImmediately(
                                resource.getId(), creationPackageType.getName());
                    }
                    catch (PluginContainerException e)
                    {
                        log.error("Failed to discover underlying " + creationPackageType.getName() + " package for "
                                + resourceType.getName() + " Resource.", e);
                    }
                }
            }
        }
    }

    public void resourcesRemoved(Set<Resource> resources)
    {
        if (this.navTreeInitialized)
            for (Resource resource : resources)
                Events.instance().raiseEvent(ResourceManager.RESOURCE_DELETED_EVENT, resource);
    }

    public void resourceActivated(Resource resource)
    {
        return;
    }

    public void resourceDeactivated(Resource resource) {
        return;
    }

    @Observer(ResourceManager.NAV_TREE_INITIALIZED)
    public void navTreeInitialized()
    {
        this.navTreeInitialized = true;
    }

    private static boolean isContentBacked(ResourceType resourceType)
    {
        return resourceType.isCreatable() && resourceType.getCreationDataType() == ResourceCreationDataType.CONTENT;
    }

    @Nullable
    private static PackageType getCreationPackageType(ResourceType resourceType)
    {
        Set<PackageType> packageTypes = resourceType.getPackageTypes();
        for (PackageType packageType : packageTypes)
        {
            if (packageType.isCreationData())
                return packageType;
        }
        return null;
    }
}
