/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * A filter which lazily initializes another filter.
 * <p/>
 * <p>The filter requires an initalization parameter with the name {@link org.jboss.on.embedded.LazyStartupFilter}
 * <p/>
 * <p>The value is the name of the filter to lazily load. All other parameters are passed to the lazily loaded filter.
 * <p/>
 * <pre><code>
 *  &lt;filter>
 *      &lt;filter-name>Seam Filter&lt;/filter-name&gt;
 *      &lt;!-- Lazily start up the Seam Filter --&gt;
 *      &lt;filter-class&gt;org.jboss.on.embedded.LazyStartupFilter&lt;/filter-class&gt;
 * <p/>
 *      &lt;init-param&gt;
 *       &lt;param-name&gt;org.jboss.on.embedded.LazyStartupFilter&lt;/param-name&gt;
 *       &lt;param-value&gt;org.jboss.seam.servlet.SeamFilter&lt;/param-value&gt;
 *      &lt;/init-param&gt;
 *  &lt;/filter&gt;
 *  &lt;filter-mapping&gt;
 *      &lt;filter-name&gt;Seam Filter&lt;/filter-name&gt;
 *      &lt;url-pattern&gt;/*&lt;/url-pattern&gt;
 *  &lt;/filter-mapping&gt;
 *  </code></pre>
 *
 * @author Jason T. Greene
 */
public class LazyStartupFilter implements Filter {
    private final Log log = LogFactory.getLog(getClass());

    private volatile Filter filter;
    private volatile boolean initialized = false;
    private volatile FilterConfig config;

    public void destroy() {
        if (!initialized) {
            filter = null;
            config = null;
            return;
        }

        try {
            init();
        } catch (ServletException e) {
            log.error("Filter through an exception during init: ", e);
        }

        filter.destroy();
    }

    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException,
        ServletException {
        init();

        filter.doFilter(request, response, chain);
    }

    private void init() throws ServletException {
        if (!initialized) {
            String parameter = getClass().getName();
            String filterName = config.getInitParameter(parameter);
            if (filterName == null)
                throw new IllegalArgumentException("Filter was not specified using the " + parameter + " parameter");

            log.debug("Loading filter" + filterName);
            try {
                filter = (Filter) Class.forName(filterName).newInstance();
            } catch (Exception e) {
                log.error("Could not instantiated filter: " + filterName, e);
            }

            filter.init(config);
            log.debug("Filter activated: " + filter);
            initialized = true;
        }
    }

    public void init(FilterConfig config) throws ServletException {
        this.config = config;
    }

}
