/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded;

import java.util.ArrayList;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.locks.ReentrantReadWriteLock;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextAttributeEvent;
import javax.servlet.ServletContextAttributeListener;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.servlet.ServletRequestAttributeEvent;
import javax.servlet.ServletRequestAttributeListener;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;
import javax.servlet.http.HttpSessionAttributeListener;
import javax.servlet.http.HttpSessionBindingEvent;
import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * LazyStartupListener is a special listener that can lazily start any number of hard-coded Servlet listeners.
 * <p/>
 * <p>This works by delaying instantiation and context initialization events until any post-init event occurs (session
 * creation, servlet request, etc).
 * <p/>
 * <p>This filter expects a context parameter with the name: {@link org.jboss.on.embedded.LazyStartupListener}
 * <p/>
 * <p>The value is a comma separated list of listeners to lazily load:
 * <p/>
 * <pre><code>
 *  &lt;!-- Seam (lazy loaded)--&gt;
 *  &lt;context-param&gt;
 *      <param-name>org.jboss.on.embedded.LazyStartupListener&lt;/param-name&gt;
 *      <param-value>org.jboss.seam.servlet.SeamListener&lt;/param-value&gt;
 *  &lt;/context-param&gt;
 *  &lt;listener&gt;
 *      &lt;listener-class>org.jboss.on.embedded.LazyStartupListener&lt;/listener-class&gt;
 *  &lt;/listener&gt;
 * </code></pre>
 *
 * @author Jason T. Greene
 */
public class LazyStartupListener implements HttpSessionListener, ServletRequestListener, ServletContextListener, HttpSessionAttributeListener,
        ServletRequestAttributeListener, ServletContextAttributeListener
{
    private final Log log = LogFactory.getLog(getClass());
    private final ConcurrentLinkedQueue<ContextEvent> contextEvents = new ConcurrentLinkedQueue<ContextEvent>();

    /*
    * Technically the array list fields don't need to be volatile, since initialized
    * is read before they are read and written after they are read.
    */
    private volatile ArrayList<HttpSessionListener> sessionListeners;
    private volatile ArrayList<HttpSessionAttributeListener> sessionAttributeListeners;
    private volatile ArrayList<ServletRequestListener> requestListeners;
    private volatile ArrayList<ServletRequestAttributeListener> requestAttributeListeners;
    private volatile ArrayList<ServletContextListener> contextListeners;
    private volatile ArrayList<ServletContextAttributeListener> contextAttributeListeners;

    private volatile boolean initialized = false;
    private volatile ServletContext context;
    private volatile String[] names;


    /**
     * This lock guards event replay. Any thread which adds to the context event queue MUST obtain a read lock. The
     * thread which replays the queue MUST obtain a write lock.
     */
    private final ReentrantReadWriteLock replayLock = new ReentrantReadWriteLock();

    private enum ContextEventType
    {
        ADD, REMOVE, REPLACE, INIT
    }

    ;

    private final class ContextEvent
    {
        final ContextEventType type;
        final ServletContextEvent event;

        private ContextEvent(ContextEventType type, ServletContextEvent event)
        {
            this.type = type;
            this.event = event;
        }
    }

    private void setupListeners()
    {
        final ArrayList<HttpSessionListener> sessionListeners = new ArrayList<HttpSessionListener>();
        final ArrayList<HttpSessionAttributeListener> sessionAttributeListeners = new ArrayList<HttpSessionAttributeListener>();
        final ArrayList<ServletRequestListener> requestListeners = new ArrayList<ServletRequestListener>();
        final ArrayList<ServletRequestAttributeListener> requestAttributeListeners = new ArrayList<ServletRequestAttributeListener>();
        final ArrayList<ServletContextListener> contextListeners = new ArrayList<ServletContextListener>();
        final ArrayList<ServletContextAttributeListener> contextAttributeListeners = new ArrayList<ServletContextAttributeListener>();

        Object[] objs = new Object[names.length];
        for (int i = 0; i < names.length; i++)
        {
            try
            {
                objs[i] = Class.forName(names[i]).newInstance();
            }
            catch (Exception e)
            {
                log.error("Could not find/load listener: " + names[i], e);
            }

        }

        for (Object obj : objs)
        {
            if (obj instanceof HttpSessionListener)
                sessionListeners.add((HttpSessionListener)obj);
            if (obj instanceof HttpSessionAttributeListener)
                sessionAttributeListeners.add((HttpSessionAttributeListener)obj);
            if (obj instanceof ServletRequestListener)
                requestListeners.add((ServletRequestListener)obj);
            if (obj instanceof ServletRequestAttributeListener)
                requestAttributeListeners.add((ServletRequestAttributeListener)obj);
            if (obj instanceof ServletContextListener)
                contextListeners.add((ServletContextListener)obj);
            if (obj instanceof ServletContextAttributeListener)
                contextAttributeListeners.add((ServletContextAttributeListener)obj);

            log.debug("Activated listener: " + obj);
        }

        this.sessionListeners = sessionListeners;
        this.sessionAttributeListeners = sessionAttributeListeners;
        this.requestListeners = requestListeners;
        this.requestAttributeListeners = requestAttributeListeners;
        this.contextListeners = contextListeners;
        this.contextAttributeListeners = contextAttributeListeners;
    }

    private void initialize()
    {
        if (initialized)
            return;

        log.debug("Activating listeners");
        setupListeners();

        try
        {
            replayLock.writeLock().lock();

            ConcurrentLinkedQueue<ContextEvent> events = contextEvents;
            ContextEvent event = events.poll();
            while (event != null)
            {
                switch (event.type)
                {
                    case INIT:
                        for (ServletContextListener listener : contextListeners)
                            listener.contextInitialized(event.event);
                        break;
                    case ADD:
                        for (ServletContextAttributeListener listener : contextAttributeListeners)
                            listener.attributeAdded((ServletContextAttributeEvent)event.event);
                        break;
                    case REMOVE:
                        for (ServletContextAttributeListener listener : contextAttributeListeners)
                            listener.attributeRemoved((ServletContextAttributeEvent)event.event);
                        break;
                    case REPLACE:
                        for (ServletContextAttributeListener listener : contextAttributeListeners)
                            listener.attributeReplaced((ServletContextAttributeEvent)event.event);
                        break;
                }

                event = events.poll();
            }

            initialized = true;
        }
        finally
        {
            replayLock.writeLock().unlock();
        }

    }

    public void sessionCreated(HttpSessionEvent event)
    {
        initialize();

        ArrayList<HttpSessionListener> listeners = sessionListeners;
        for (HttpSessionListener listener : listeners)
            listener.sessionCreated(event);
    }

    public void sessionDestroyed(HttpSessionEvent event)
    {
        initialize();

        ArrayList<HttpSessionListener> listeners = sessionListeners;
        for (HttpSessionListener listener : listeners)
            listener.sessionDestroyed(event);
    }

    public void requestDestroyed(ServletRequestEvent event)
    {
        initialize();

        ArrayList<ServletRequestListener> listeners = requestListeners;
        for (ServletRequestListener listener : listeners)
            listener.requestDestroyed(event);

    }

    public void requestInitialized(ServletRequestEvent event)
    {
        initialize();

        ArrayList<ServletRequestListener> listeners = requestListeners;
        for (ServletRequestListener listener : listeners)
            listener.requestInitialized(event);
    }

    public void contextDestroyed(ServletContextEvent event)
    {
        if (!initialized)
        {
            contextEvents.clear();
            return;
        }

        initialize();

        ArrayList<ServletContextListener> listeners = contextListeners;
        for (ServletContextListener listener : listeners)
            listener.contextDestroyed(event);
    }

    public void contextInitialized(ServletContextEvent event)
    {
        this.context = event.getServletContext();

        // Comma separated list of listeners
        String param = context.getInitParameter(getClass().getName());
        log.debug("Listeners configured for lazy loading: " + param);
        this.names = param == null ? new String[0] : param.split(",");

        try
        {
            replayLock.readLock().lock();

            if (!initialized)
            {
                contextEvents.add(new ContextEvent(ContextEventType.INIT, event));
                return;
            }

            ArrayList<ServletContextListener> listeners = contextListeners;
            for (ServletContextListener listener : listeners)
                listener.contextInitialized(event);
        }
        finally
        {
            replayLock.readLock().unlock();
        }
    }

    public void attributeAdded(HttpSessionBindingEvent event)
    {
        initialize();

        ArrayList<HttpSessionAttributeListener> listeners = sessionAttributeListeners;
        for (HttpSessionAttributeListener listener : listeners)
            listener.attributeAdded(event);
    }

    public void attributeRemoved(HttpSessionBindingEvent event)
    {
        initialize();

        ArrayList<HttpSessionAttributeListener> listeners = sessionAttributeListeners;
        for (HttpSessionAttributeListener listener : listeners)
            listener.attributeRemoved(event);
    }

    public void attributeReplaced(HttpSessionBindingEvent event)
    {
        initialize();

        ArrayList<HttpSessionAttributeListener> listeners = sessionAttributeListeners;
        for (HttpSessionAttributeListener listener : listeners)
            listener.attributeReplaced(event);
    }

    public void attributeAdded(ServletRequestAttributeEvent event)
    {
        initialize();

        ArrayList<ServletRequestAttributeListener> listeners = requestAttributeListeners;
        for (ServletRequestAttributeListener listener : listeners)
            listener.attributeAdded(event);
    }

    public void attributeRemoved(ServletRequestAttributeEvent event)
    {
        initialize();

        ArrayList<ServletRequestAttributeListener> listeners = requestAttributeListeners;
        for (ServletRequestAttributeListener listener : listeners)
            listener.attributeRemoved(event);
    }

    public void attributeReplaced(ServletRequestAttributeEvent event)
    {
        initialize();

        ArrayList<ServletRequestAttributeListener> listeners = requestAttributeListeners;
        for (ServletRequestAttributeListener listener : listeners)
            listener.attributeReplaced(event);

    }

    public void attributeAdded(ServletContextAttributeEvent event)
    {
        try
        {
            replayLock.readLock().lock();

            if (!initialized)
            {
                contextEvents.add(new ContextEvent(ContextEventType.ADD, event));
                return;
            }

            ArrayList<ServletContextAttributeListener> listeners = contextAttributeListeners;
            for (ServletContextAttributeListener listener : listeners)
                listener.attributeAdded(event);
        }
        finally
        {
            replayLock.readLock().unlock();
        }

    }

    public void attributeRemoved(ServletContextAttributeEvent event)
    {

        try
        {
            replayLock.readLock().lock();

            if (!initialized)
            {
                contextEvents.add(new ContextEvent(ContextEventType.REMOVE, event));
                return;
            }

            ArrayList<ServletContextAttributeListener> listeners = contextAttributeListeners;
            for (ServletContextAttributeListener listener : listeners)
                listener.attributeRemoved(event);
        }
        finally
        {
            replayLock.readLock().unlock();
        }
    }

    public void attributeReplaced(ServletContextAttributeEvent event)
    {

        try
        {
            replayLock.readLock().lock();

            if (!initialized)
            {
                contextEvents.add(new ContextEvent(ContextEventType.REPLACE, event));
                return;
            }

            ArrayList<ServletContextAttributeListener> listeners = contextAttributeListeners;
            for (ServletContextAttributeListener listener : listeners)
                listener.attributeRemoved(event);
        }
        finally
        {
            replayLock.readLock().unlock();
        }
    }
}
