/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2010 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.bean;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.jboss.on.embedded.util.MeasurementConversionUtility;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.rhq.core.clientapi.agent.measurement.MeasurementAgentService;
import org.rhq.core.domain.measurement.DataType;
import org.rhq.core.domain.measurement.DisplayType;
import org.rhq.core.domain.measurement.MeasurementData;
import org.rhq.core.domain.measurement.MeasurementDefinition;
import org.rhq.core.domain.measurement.MeasurementUnits;
import org.rhq.core.domain.measurement.composite.MeasurementNumericValueAndUnits;
import org.rhq.core.domain.measurement.composite.MeasurementStringValueAndUnits;
import org.rhq.core.domain.measurement.util.MeasurementConversionException;
import org.rhq.core.domain.resource.ResourceType;
import org.rhq.core.pc.PluginContainer;

/**
 * Utility class to retrieve the Measurement information for the give resource
 * 
 * @author Jessica Sant
 * @author Ian Springer
 */
public class MeasurementUtils {
	/**
	 * the resource being displayed on the metric tab
	 */
	private ResourceListItem resource;

	/**
	 * this resource's type
	 */
	private ResourceType resourceType;

	/**
	 * a map of the resource's metric definitions, keyed off the definition
	 * names
	 */
	private Map<String, MeasurementDefinition> measurementDefinitionMap = new HashMap<String, MeasurementDefinition>();

	public MeasurementUtils(ResourceListItem resource) {
		this.resource = resource;
		this.resourceType = this.resource.getResource().getResourceType();
		this.measurementDefinitionMap = new HashMap();
		Set<MeasurementDefinition> metricDefinitions = this.resourceType
				.getMetricDefinitions();
		for (MeasurementDefinition measurementDefinition : metricDefinitions) {
			if (!measurementDefinition.isPerMinute())
				// Filter out per-minute metrics, since we don't support them in
				// embedded for the time being.
				// This is also important, since we key off the metric names,
				// and a per-minute metric has the
				// same name as its corresponding raw TRENDSUP/TRENDSDOWN
				// metric.
				measurementDefinitionMap.put(measurementDefinition.getName(),
						measurementDefinition);
		}
	}

	/**
	 * Loads the metric values for the current Resource, optionally filtered by
	 * display type and/or data type.
	 * 
	 * @param displayType
	 *            the display type (summary or detail) that the returned metrics
	 *            should have. Null if any display type should be returned.
	 * @param dataType
	 *            the data type (measurement or trait) that the returned metrics
	 *            should have. Null if any data type should be returned.
	 * @return a set of all metrics that have the given display type and data
	 *         type
	 */
	public @Nullable
	List<MeasurementData> loadMeasurementData(
			@Nullable DisplayType displayType, DataType dataType) {
		Map<String, MeasurementDefinition> requestedMetricDefs = new HashMap();

		// Get the definitions of the requested measurements.
		for (MeasurementDefinition metricDef : this.measurementDefinitionMap
				.values()) {
			if (displayType != null && dataType != null) {
				if (metricDef.getDisplayType().equals(displayType)
						&& metricDef.getDataType().equals(dataType))
					requestedMetricDefs.put(metricDef.getName(), metricDef);
			} else if (dataType != null) {
				if (metricDef.getDataType().equals(dataType))
					requestedMetricDefs.put(metricDef.getName(), metricDef);
			} else if (displayType != null) {
				if (metricDef.getDisplayType().equals(displayType))
					requestedMetricDefs.put(metricDef.getName(), metricDef);
			}
		}

		// Get the actual data for the requested measurements.
		MeasurementAgentService measurementAgentService = PluginContainer
				.getInstance().getMeasurementAgentService();
		int resourceId = this.resource.getResource().getId();
		String[] measurementNamesArray = requestedMetricDefs.keySet().toArray(
				new String[requestedMetricDefs.size()]);
		Set<MeasurementData> measurementDataSet = measurementAgentService
				.getRealTimeMeasurementValue(resourceId, dataType,
						measurementNamesArray);
		// @TODO when displayOrder is implemented this should use displayOrder,
		// but right now the displayOrder
		// for all MeasurementData is set to "0" which will cause only the last
		// object of the set to be
		// put into the TreeSet due to the semantics of TreeSet which does not
		// implement the Set interface correctly.
		List<MeasurementData> measurementDataList = null;
		if (measurementDataSet != null) {
			measurementDataList = new ArrayList<MeasurementData>(
					measurementDataSet);
			Collections.sort(measurementDataList, new MeasurementComparator(
					requestedMetricDefs));
		}
		return measurementDataList;
	}

	/**
	 * returns a list of all categories used on this page. This method also
	 * loads the measuremntDataList and the measurementDataMap (a map of
	 * 
	 * @param measurementDefinitionMap
	 * @param measurementDataList
	 * @param measurementDisplayMap
	 * @return
	 */
	public @NotNull
	List<String> loadCategoryList(
			@NotNull Map<String, MeasurementDefinition> measurementDefinitionMap,
			@Nullable List<MeasurementData> measurementDataList,
			@NotNull Map<String, List<MeasurementDisplay>> measurementDisplayMap) {
		// measurementDisplayMap = new HashMap<String,
		// List<MeasurementDisplay>>();

		if (measurementDataList != null) {
			for (MeasurementData measurementData : measurementDataList) {
				MeasurementDefinition measurementDefinition = measurementDefinitionMap
						.get(measurementData.getName());
				String category = measurementDefinition.getCategory()
						.toString();
				List<MeasurementDisplay> list = measurementDisplayMap
						.get(category);

				if (list == null) {
					list = new ArrayList<MeasurementDisplay>();
					measurementDisplayMap.put(category, list);
				}

				list.add(createMeasurementDisplay(measurementData,
						measurementDefinition));
			}
		}
		return new ArrayList<String>(measurementDisplayMap.keySet());
	}

	/**
	 * creates the measurementDisplay items to be used to display the formatted
	 * values
	 * 
	 * @param measurementData
	 * @param measurementDefinition
	 * @return
	 */
	public MeasurementDisplay createMeasurementDisplay(
			MeasurementData measurementData,
			MeasurementDefinition measurementDefinition) {
		MeasurementDisplay measurementDisplay = new MeasurementDisplay(
				measurementData, measurementDefinition);
		measurementDisplay.getMeasurementDefinition().getUnits();

		// TODO: send in the current Locale
		if (measurementDisplay.getMeasurementData().getValue() instanceof Double) {
			try {
				double dataValue = (Double) measurementDisplay
						.getMeasurementData().getValue();
				MeasurementUnits units = measurementDisplay
						.getMeasurementDefinition().getUnits();
				MeasurementNumericValueAndUnits dataValueAndUnits;
				if (units == MeasurementUnits.NONE) {
					Double dbl = Double.valueOf(dataValue);
					if (dataValue != dbl.intValue())
						dataValueAndUnits = MeasurementConversionUtility.fit(dataValue,
								units);
					else
						dataValueAndUnits = new MeasurementIntegerValueAndUnits(
								dataValue, units);
				} else
					dataValueAndUnits = MeasurementConversionUtility.fit(dataValue,
							units);
				measurementDisplay
						.setMeasurementValueAndUnits(dataValueAndUnits);
			} catch (MeasurementConversionException mce) {
				/*
				 * this should normally never happen because the data is coming
				 * from the database where it's already been validated by the
				 * model constraits, but it's possible that we missed something
				 * somewhere else in the system or that someone purposefully
				 * tried to harm the data; in this case, don't blow up
				 * 
				 * TODO: add logging so we know when / if this error occurs
				 */
				measurementDisplay
						.setMeasurementValueAndUnits(MeasurementNumericValueAndUnits.ERROR);
			}
		}
		// if its a string -- just save the string into the Formatted value
		// display
		else if (measurementDisplay.getMeasurementData().getValue() instanceof String) {
			String dataValue = (String) measurementDisplay.getMeasurementData()
					.getValue();

			measurementDisplay
					.setMeasurementValueAndUnits(new MeasurementStringValueAndUnits(
							dataValue, MeasurementUnits.NONE));
		}

		return measurementDisplay;
	}

	public Map<String, MeasurementDefinition> getMeasurementDefinitionMap() {
		return measurementDefinitionMap;

	}
}
