/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.manager;

import java.util.List;
import java.util.Set;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.rhq.core.clientapi.agent.inventory.CreateResourceResponse;
import org.rhq.core.clientapi.agent.inventory.DeleteResourceResponse;
import org.rhq.core.clientapi.server.configuration.ConfigurationUpdateResponse;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.content.transfer.ResourcePackageDetails;
import org.rhq.core.domain.measurement.Availability;
import org.rhq.core.domain.operation.OperationDefinition;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;

/**
 * API used to retrieve backend data for the embedded console.
 *
 * @author Jason Dobies
 */
public interface ResourceManager
{
    String RESOURCE_CREATED_EVENT = "resourceCreated";

    String RESOURCE_UPDATED_EVENT = "resourceUpdated";

    String RESOURCE_DELETED_EVENT = "resourceDeleted";

    String NAV_TREE_INITIALIZED = "navTreeInitialized";

    /**
     * Tells this instance to discover all resources. Calls to methods in this interface will return empty data until
     * resources have been discovered. This is a blocking method.
     */
    void discoverResources();

    /**
     * Non-blocking method which will try to refresh the inventory with the latest services which have been deployed
     */
    void discoverServicesAsync();

    /**
     * Blocking method which will try to refresh the inventory with the latest services which have been deployed
     */
    void discoverServicesSync();

    /**
     * Returns the <code>ResourceType</code> with the specified name, or, if no ResourceType exists with that name,
     * <code>null</code> is returned.
     *
     * @param resourceTypeName name of the resource type.
     * @return the <code>ResourceType</code> with the specified name, or, if no ResourceType exists with that name,
     *         <code>null</code> is returned
     */
    @Nullable
    ResourceType getResourceType(String resourceTypeName);

    /**
     * Returns the set of all known resource types handled by this <code>ResourceManager</code> implementation.
     *
     * @return set of <code>ResourceType</code> instances; empty set if none are registered.
     */
    Set<ResourceType> getAllResourceTypes();

    /**
     * Returns all resources of the given type, that are descendants of the given resource. If no resources exists, this
     * will return an empty Set. The Set of resources will be sorted by name.
     *
     * @param resourceType type of resource.
     * @param resource     parent resource.
     * @return set of <code>Resource</code> instances; empty list if none match the resource type.
     */
    @NotNull
    Set<Resource> getResources(ResourceType resourceType, Resource resource);

    @Nullable
    Resource getResource(Integer id);

    /**
     * Returns the current configuration values of the specified resource. The returned <code>Configuration</code>
     * instance will contain an entry for every property contained in the configuration definition for corresponding
     * resource type.
     *
     * @param resource resource for which to to retrieve values.
     * @return current configuration values for the resource.
     */
    Configuration getResourceConfiguration(Resource resource);

    /**
     * Removes the specified resource.
     *
     * @param resource resource to be removed.
     * @return DeleteResourceResponse returns the response to the delete request.
     */
    DeleteResourceResponse removeResource(Resource resource);

    /**
     * Creates a new resource.
     *
     * @param resourceName          name of the resource being created
     * @param resourceType          type of resource being created.
     * @param ancestorResource      the closest ancestor resource to the one being created, usually parent
     * @param pluginConfiguration   optional values that can be used use when creating the newly created resource.
     * @param resourceConfiguration values to use when configuring the newly created resource.
     * @return returns the CreateResourceRequest, allows the caller to check for errors and status.
     */
    CreateResourceResponse createResource(String resourceName, ResourceType resourceType, Resource ancestorResource, Configuration pluginConfiguration, Configuration resourceConfiguration);

    /**
     * Creates a new resource.
     *
     * @param resourceName        name of the resource being created
     * @param resourceType        type of resource being created.
     * @param parentResource      the parent resource of the one being created
     * @param pluginConfiguration optional values that can be used use when creating the newly created resource.
     * @param packageDetails      PackagedDetails for a content backed resource.
     * @return returns the CreateResourceRequest, allows the caller to check for errors and status.
     */
    CreateResourceResponse createResource(String resourceName, ResourceType resourceType, Resource parentResource, Configuration pluginConfiguration, ResourcePackageDetails packageDetails);

    /**
     * Updates the resource with the specified configuration values.
     *
     * @param resource      resource being updated.
     * @param configuration new values for the resource.
     * @return returns the ConfigurationUpdateRequest, allows the caller to check for errors and status.
     */
    ConfigurationUpdateResponse updateResource(Resource resource, Configuration configuration);

    /**
     * Returns the last known availability state of the resource.
     *
     * @param resource resource's availability to retrieve.
     * @return <code>Availability</code> instance, with type indicating up, down, or unknown.
     */
    Availability getAvailability(Resource resource);

    /**
     * Returns the list of operations that can be performed on this resource
     *
     * @param resource Resource's operations to retrieve
     * @return Set<OperationDefinition> operations for the resource
     */
    List<OperationDefinition> getOperationsForResource(Resource resource);

    /**
     * Invokes an operation on a Resource.
     *
     * @param resource   resource to invoke the operation on.
     * @param operation  operation that is going to be invoked
     * @param parameters
     * @param jobId      id to give the operation so that it can be found later to update the status in
     */
    void invokeOperation(Resource resource, OperationDefinition operation, Configuration parameters, String jobId);

    /**
     * Returns the underlying platform which the Resource Manager is running on
     *
     * @return Resource platform Resource
     */
    Resource getPlatform();
}
                                                          
