/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;

import org.jboss.on.embedded.manager.ResourceManagerFactory;

/**
 * A managed bean that periodically asynchronously executes service discovery scans. The {@link #checkAutodiscovery()}
 * action will get triggered every time the user hits any page (configured in pages.xml), but it will only kick off a
 * scan if the last scan was more than one minute ago.
 *
 * @author Charles Crouch
 */
@Scope(ScopeType.APPLICATION)
@Name("discoveryAction")
public class DiscoveryAction
{
    private final Log log = LogFactory.getLog(this.getClass());

    // the minimum length of time (in milliseconds) between consecutive discoveries
    private static final int DEFAULT_MINIMUM_SCAN_INTERVAL = 60000; // 1 minute

    @In
    private BootstrapAction bootstrapAction;

    private volatile long lastDiscoveryTime;

    private int scanPeriod = DEFAULT_MINIMUM_SCAN_INTERVAL;

    public int getScanPeriod()
    {
        return this.scanPeriod;
    }

    public void setScanPeriod(int scanPeriod)
    {
        this.scanPeriod = scanPeriod;
    }

    public void checkAutodiscovery()
    {
        if (this.bootstrapAction.isInitialized() && (System.currentTimeMillis() - lastDiscoveryTime) > DEFAULT_MINIMUM_SCAN_INTERVAL)
        {
            // Try the timestamp check in a synchronized block.
            tryAutodiscovery();
        }
    }

    synchronized private void tryAutodiscovery()
    {
        log.trace("Checking if service discovery scan is needed...");
        if ((System.currentTimeMillis() - lastDiscoveryTime) > DEFAULT_MINIMUM_SCAN_INTERVAL)
        {
            // Put the timestamp updating above the call to discoverServicesAsync(), so we don't continuously
            // retry on failure, and so we don't block subsequent requests waiting to get into tryAutodiscovery().
            lastDiscoveryTime = System.currentTimeMillis();
            log.trace("Performing service discovery scan...");
            ResourceManagerFactory.resourceManager().discoverServicesAsync();
        }
    }
}
