/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui.configuration.resource;

import java.io.Serializable;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

import javax.faces.application.FacesMessage;

import org.jetbrains.annotations.NotNull;
import org.rhq.core.clientapi.agent.inventory.CreateResourceResponse;
import org.rhq.core.clientapi.server.configuration.ConfigurationUpdateResponse;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.configuration.PropertySimple;
import org.rhq.core.domain.configuration.definition.ConfigurationDefinition;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;
import org.rhq.core.gui.configuration.ConfigHelperUIBean;
import org.rhq.core.gui.util.IdChunkGeneratorUIBean;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Begin;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.core.Conversation;
import org.jboss.seam.faces.FacesMessages;

import org.jboss.on.embedded.bean.ResourceListItem;
import org.jboss.on.embedded.manager.ResourceManager;
import org.jboss.on.embedded.manager.ResourceManagerFactory;
import org.jboss.on.embedded.ui.CommonActionUtil;
import org.jboss.on.embedded.ui.FacesOutcomes;

/**
 * Managed Bean to handle Resource Configuration pages. Both Create and Edit Resource configurations are handled by this
 * class.
 *
 * @author Mark Spritzler
 */
@Name("resourceConfigurationUIBean")
@Scope(ScopeType.CONVERSATION)
public class ResourceConfigurationUIBean extends AbstractResourceConfigurationUIBean implements Serializable
{
    public static final String MANAGED_BEAN_NAME = "resourceConfigurationUIBean";

    //Bijection Variables
    @In(create = true)
    CommonActionUtil commonActionUtil;

    @In(required = false)
    private transient FacesMessages facesMessages;

    @In(value = "IdChunkGeneratorUIBean", create = true)
    private IdChunkGeneratorUIBean idChunkGenerator;

    @In(value = "ConfigHelperUIBean", create = true)
    private ConfigHelperUIBean configHelperUIBean;

    @In(value = "ancestorResource", required = false)
    private Resource ancestorResource;

    @Out(value = "resource", required = false)
    private ResourceListItem listItem;

    private ResourceType serviceType;

    private ResourceBundle messages = ResourceBundle.getBundle("messages");

    /**
     * If the user is creating a new resource, they will come from createResourcePage1.xhtml They would have chosen a
     * Template and clicked continue, which would have outjected a value for "serviceType" and then inject it here.
     * Since only create will inject a serviceType this method also sets the actionType for this instance to CREATE, so
     * that save will know to call create instead of update
     *
     * @param serviceType Resource Type of the Resource that we are trying to create
     */
    @In(value = "serviceType", required = false)
    public void setServiceType(ResourceType serviceType)
    {
        if (serviceType != null)
        {
            this.serviceType = serviceType;
            setConfigurationDefinition(this.serviceType.getResourceConfigurationDefinition());
            actionType = ActionType.CREATE;
        }
    }

    @In(value = "templateConfiguration", required = false)
    public void setTemplateConfiguration(Configuration template)
    {
        if (template != null)
            setConfiguration(template);
    }

    //Instance Variables used by this class but not bijected
    private ResourceManager configurationManager = ResourceManagerFactory.resourceManager();

    private enum ActionType
    {
        CREATE, UPDATE
    }

    private ActionType actionType;

    public ActionType getActionType()
    {
        return actionType;
    }

    /**
     * Method that is called from the tabMenu.xhtml when the user clicks the Configuration tab This method will start
     * the edit resource conversation and also sets the actionType to be UPDATE so that the save will know that it is
     * editing and therefore need to call update instead of create.
     */
    @Begin
    public void resourceConfiguration()
    {
        actionType = ActionType.UPDATE;
        listItem = commonActionUtil.getCurrentResourceListItem();
        Resource resource = listItem.getResource();
        try
        {
            setConfiguration(configurationManager.getResourceConfiguration(resource));
        }
        catch (Exception e)
        {
            setConfiguration(null);
        }

        setConfigurationDefinition(resource.getResourceType().getResourceConfigurationDefinition());
    }

    /**
     * This method is called when the user clicks the Save button on the configuration page. It also ends the
     * conversation.
     *
     * @return String Results of the save success or failure.
     */
    public String saveConfiguration()
    {
        String result = FacesOutcomes.FAILURE;
        switch (actionType)
        {
            case CREATE:
            {
                result = createConfiguration();
                break;
            }
            case UPDATE:
            {
                result = updateConfiguration();
                break;
            }
        }

        if (result.equals("success"))
            Conversation.instance().end(false);
        return result;
    }

    private String updateConfiguration()
    {
        String result;
        Configuration resourceConfig = getConfiguration();

        ConfigurationUpdateResponse pluginResponse = configurationManager.updateResource(this.listItem.getResource(),
                resourceConfig);

        switch (pluginResponse.getStatus())
        {
            case SUCCESS:
            {
                facesMessages.addFromResourceBundle(FacesMessage.SEVERITY_INFO, "resource.status.update.success",
                        this.listItem.getResource().getResourceType().getName(), this.listItem.getResource().getName());
                result = FacesOutcomes.SUCCESS;
                break;
            }
            default:
            {
                String baseMessage = this.messages.getString("resource.status.update.failure");
                addActionFailureMessageToFacesMessages(baseMessage, pluginResponse.getErrorMessage(),
                        pluginResponse.getConfiguration());
                result = FacesOutcomes.FAILURE;
            }
        }

        return result;
    }

    private String createConfiguration()
    {
        String result;
        Configuration resourceConfig = getConfiguration();

        Configuration pluginConfig = null;
        if (serviceType != null)
        {
            ConfigurationDefinition configDef = serviceType.getPluginConfigurationDefinition();
            if (configDef != null)
            {
                pluginConfig = configDef.getDefaultTemplate().getConfiguration();
            }
        }
        CreateResourceResponse pluginResponse = configurationManager.createResource("newResource", serviceType,
                ancestorResource, pluginConfig, resourceConfig);
        switch (pluginResponse.getStatus())
        {
            case SUCCESS:
            {
                facesMessages.addFromResourceBundle(FacesMessage.SEVERITY_INFO, "resource.status.create.success",
                        serviceType.getName());
                result = FacesOutcomes.SUCCESS;
                break;
            }
            default:
            {
                String baseMessage = this.messages.getString("resource.status.create.failure");
                addActionFailureMessageToFacesMessages(baseMessage, pluginResponse.getErrorMessage(),
                        pluginResponse.getResourceConfiguration());
                result = FacesOutcomes.FAILURE;
                break;
            }
        }
        return result;
    }

    public void addMap()
    {
    }

    public void editMap()
    {
    }

    public void cancelEditMap()
    {
    }

    protected IdChunkGeneratorUIBean getIdChunkGenerator()
    {
        return idChunkGenerator;
    }

    protected ConfigHelperUIBean getConfigHelperUIBean()
    {
        return configHelperUIBean;
    }

    private void addActionFailureMessageToFacesMessages(@NotNull String baseMessage, String errorDetails,
                                                        Configuration resourceConfiguration)
    {
        if (errorDetails == null || errorDetails.trim().equals(""))
        {
            if (containsPropertyErrors(resourceConfiguration))
                errorDetails = this.messages.getString("resource.status.invalidProps");
            else
                errorDetails = this.messages.getString("resource.status.failedForUnknownReason");
        }
        // TODO When this is internationalized, the method call "add" will change to addFromResourceBundle
        this.facesMessages.add(new FacesMessage(FacesMessage.SEVERITY_ERROR, baseMessage + " " + errorDetails, null));
    }

    private static boolean containsPropertyErrors(Configuration configuration)
    {
        if (configuration == null)
            return false;
        // TODO: This only handles top-level simples. It should recurse into lists and maps.
        Map<String, PropertySimple> properties = configuration.getSimpleProperties();
        Set<String> propNames = properties.keySet();
        for (String propName : propNames)
        {
            PropertySimple property = properties.get(propName);
            if (property != null)
            {
                String error = property.getErrorMessage();
                if (error != null && !error.equals(""))
                    return true;
            }
        }
        return false;
    }
}
