/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2010 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui.nav;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.atomic.AtomicInteger;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceSubCategory;
import org.rhq.core.domain.resource.ResourceType;
import org.richfaces.model.TreeNode;

import org.jboss.on.embedded.manager.ResourceManager;
import org.jboss.on.embedded.manager.ResourceManagerFactory;
import org.jboss.on.embedded.ui.SummaryAction;
import org.jboss.on.embedded.ui.SummaryAction.SummaryActionOutcome;

/**
 * Base class of all nodes in our nav tree, provides helper methods to subclasses for adding the appropriate child
 * nodes
 *
 * @author ccrouch
 */
public abstract class BaseTreeNode implements JONTreeNode
{
    private JONTreeNode parent;

    // needs to be initialized by sub-class
    protected Map<Object, JONTreeNode> childrenMap;

    // the key used to reference this object in its parents childrenMap
    // in the future this could be useful in allowing the node to indicate
    // how it should be retrieved from its parents map of children.
    private String key;

    // TODO I don't think we have to let users specify what the key is here,
    // they just need to know that the node has a unique key which will be used
    // if the node is added as a child to another node.
    // This is especially true given the problems that can occur if 
    // you specify a "bad" key. See JBMANCON-363
    public BaseTreeNode(String key)
    {
        this.key = key;
    }

    private Map<Object, JONTreeNode> getChildrenMap()
    {
        if (childrenMap == null)
        {
            childrenMap = new LinkedHashMap<Object, JONTreeNode>();
            initChildrenMap();
        }
        return childrenMap;
    }

    protected abstract void initChildrenMap();

    public abstract SummaryActionOutcome getSummaryActionOutcome(SummaryAction action);

    public void reInitializeChildrenMap()
    {
        if (childrenMap != null)
        {
            childrenMap.clear();
            childrenMap = null;
        }
        // eagerly re-initialize the children map
        getChildrenMap();
    }

    public Object getData()
    {
        return this;
    }

    public void setData(Object object)
    {
        // no - op
    }

    public boolean isLeaf()
    {
        return getChildrenMap().isEmpty();
    }

    /**
     * This method is pretty much useless to other clients besides the richfaces tree renderer which requires an
     * Iterator over a Map.Entry in order to work
     */
    public Iterator<Map.Entry<Object, JONTreeNode>> getChildren()
    {
        return getChildrenMap().entrySet().iterator();
    }

    /**
     * This is the much more useful sibling of getChildren()
     */
    public Collection<JONTreeNode> getChildNodes()
    {
        return getChildrenMap().values();
    }

    /**
     * This method is here just to workaround the fact that UIRepeat doesn't support Collections. See
     * https://jira.jboss.org/jira/browse/JBSEAM-1253 for more details This doesn't need to be on the JONTreeNode
     * interface since the method will be accessed via reflection on the object rather than through the interface
     */
    public List<JONTreeNode> getChildNodesAsList()
    {
        return new ArrayList<JONTreeNode>(getChildNodes());
    }

    public JONTreeNode getChild(Object key)
    {
        return getChildrenMap().get(key);
    }

    /**
     * This method should be preferred over addChild(Object key, TreeNode child) as this takes care of using the child's
     * own key
     *
     * @param child the child to be added
     */
    public void addChild(BaseTreeNode child)
    {
        addChild(child.getKey(), child);
    }

    private String getKey()
    {
        return key;
    }

    public void addChild(Object key, TreeNode child)
    {
        child.setParent(this);
        getChildrenMap().put(key, (JONTreeNode)child);
    }

    public void removeChild(Object key)
    {
        JONTreeNode treeNode = getChildrenMap().remove(key);
        if (treeNode != null)
        {
            treeNode.setParent(null);
        }
    }

    public JONTreeNode getParent()
    {
        return parent;
    }

    public void setParent(TreeNode treeNode)
    {
        parent = (JONTreeNode)treeNode;
    }

    // this is just a helper method, could be a static method somewhere else
    private static final AtomicInteger nodeKey = new AtomicInteger();

    public static String getUniqueKey()
    {
        return String.valueOf(nodeKey.incrementAndGet());
    }

    // utility method utilized by some sub classes, these could be made static
    // and have you pass in the parent instance you want to add the child to
    //
    // checks the resource type and if appropriate adds a 
    // tree node to the collection of children
    // if the resourceType is tagged with a subcategory it is ignored
    protected void addChildResourceTypeNode(@NotNull ResourceType resourceType, @NotNull Resource parentResource)
    {
        addChildResourceTypeNode(resourceType, parentResource, true);
    }

    // checks the resource type and if appropriate adds a 
    // tree node to the collection of children
    // if the resourceType is tagged with a subcategory it is not automatically ignored
    protected void addChildResourceTypeNodeWithSubCategory(@NotNull ResourceType resourceType, @NotNull Resource parentResource)
    {
        addChildResourceTypeNode(resourceType, parentResource, false);
    }

    private void addChildResourceTypeNode(@NotNull ResourceType resourceType, @NotNull Resource parentResource, boolean ignoreSubCategorizedResources)
    {
        if (resourceType == null)
        {
            throw new IllegalArgumentException("ResourceType cannot be null");
        }

        if (ignoreSubCategorizedResources && resourceType.getSubCategory() != null)
        {
            // don't include children that are tagged as being part of a subcategory
            // otherwise they will appear twice
            return;
        }

        // get all child resources of this parentResource which map the specified resourceType
        Set<Resource> resources = getResourceManager().getResources(resourceType, parentResource);

        // if resource is a singleton only display it if it actually has an associated resource
        if (resourceType.isSingleton() && !resources.isEmpty())
        {
            if (resources.size() > 1)
            {
                throw new IllegalArgumentException("ResourceType [" + resourceType + "] has more than one resource " +
                        "associated with it, yet it is specified as a Singleton");
            }
            else // just one resource
            {
                addChild(new SingletonResourceTypeTreeNode(getUniqueKey(), resourceType,
                        resources.iterator().next(), parentResource));
            }
        }
        // if its not a singleton, and there are either some child resources already, or its possible
        // to at least create some new ones, then display as NonSingletonResourceTypeTreeNode.
        // The aim here is to avoid displaying nodes that start empty and can never be added
        //
        // TODO the only thing we would miss here are resources that are added outside of the console
        // e.g. copy direct to /deploy, this could be worked through though by calling
        // reInitializeChildrenMap() after we'd run discovery.        
        else if (!resourceType.isSingleton() &&
                (!resources.isEmpty() || resourceType.isCreatable()))
        {
            addChild(new NonSingletonResourceTypeTreeNode(getUniqueKey(), resourceType, parentResource));
        }
    }

    // utility method utilized by some sub classes
    protected void addChildResourceTypeOrSubCategoryNodes(@NotNull ResourceType resourceType, @NotNull Resource parentResource)
    {
        //noinspection ConstantConditions
        if (resourceType == null)
        {
            throw new IllegalArgumentException("ResourceType cannot be null");
        }

        // iterate over child resource types of this type adding child nodes       
        Set<ResourceType> childResourceTypes = resourceType.getChildResourceTypes();
        for (ResourceType childResourceType : childResourceTypes)
        {
            addChildResourceTypeNode(childResourceType, parentResource);
        }

        // iterate over any sub categories defined on this type adding child nodes
        List<ResourceSubCategory> subCategories = resourceType.getChildSubCategories();
        Set<ResourceSubCategory> sortedSubCategories = new TreeSet<ResourceSubCategory>(subCategories);
        for (ResourceSubCategory subCategory : sortedSubCategories)
        {
            // Only add a child node for the subcategory if the subcategory contains either at least one child
            // subcategory or at least one child resource type.
            if (!subCategory.getChildSubCategories().isEmpty() || !subCategory.findTaggedResourceTypes().isEmpty())
            {
                addChild(new SubCategoryTreeNode(getUniqueKey(), subCategory, parentResource));
            }
        }
    }

    public ResourceManager getResourceManager()
    {
        return ResourceManagerFactory.resourceManager();
    }

    // TODO all this node finding stuff should be moved a separate class, e.g. JONTreeNodeFinder

    private interface NodeMatcher
    {
        public boolean matchesNode(JONTreeNode node);
    }

    /**
     * Finds a JONTreeNode based on its path. Looks at this node and its children recursively as necessary
     */
    public
    @Nullable
    JONTreeNode findNode(@Nullable String path)
    {
        if (path != null && !path.equals(""))
        {
            return findNode(this, new PathBasedNodeMatcher(path));
        }
        return null;
    }

    /**
     * Finds a ResourceTypeTreeNode based on its underlying resource type name. Looks at this node and its children
     * recursively as necessary
     */
    // TODO move these to navigation action, as well as the NodeFinder impls
    // In order for this to work it must be executed against an appropriate
    // instance of BaseTreeNode, i.e. the parent, otherwise you are going to have
    // problems due to having the same resource types
    // appearing under multiple resources, see JBMANCON-365
    public
    @Nullable
    JONTreeNode findNodeByTypeName(@Nullable String typeName)
    {
        if (typeName != null && !typeName.equals(""))
        {
            return findNode(this, new ResourceTypeNameNodeMatcher(typeName));
        }
        return null;
    }

    //TODO move these to navigation action
    // In order for this to work it must be executed against an appropriate
    // instance of BaseTreeNode, i.e. the parent, otherwise you are going to have
    // problems due to having the same resource types
    // appearing under multiple resources, see JBMANCON-365
    public ResourceTypeTreeNode findNodeByType(@NotNull ResourceType resourceType)
    {
        if (resourceType != null)
        {
            return (ResourceTypeTreeNode)findNode(this, new ResourceTypeNodeMatcher(resourceType));
        }
        return null;
    }

    private JONTreeNode findNode(JONTreeNode currentNode, NodeMatcher matcher)
    {
        JONTreeNode resultNode = null;
        if (matcher.matchesNode(currentNode))
        {
            resultNode = currentNode;   // the currentNode is the one we are looking for
        }
        else
        {
            for (JONTreeNode child : currentNode.getChildNodes())
            {
                resultNode = findNode(child, matcher);
                if (resultNode != null)
                {
                    break;   // found the node somewhere down the tree
                }
            }
        }
        return resultNode;
    }

    private class PathBasedNodeMatcher implements NodeMatcher
    {
        private String path;

        public PathBasedNodeMatcher(@NotNull String path)
        {
            this.path = path;
        }

        public boolean matchesNode(JONTreeNode node)
        {
            return path.equals(node.getPath());
        }
    }

    private class ResourceTypeNameNodeMatcher implements NodeMatcher
    {
        private String resourceTypeName;

        public ResourceTypeNameNodeMatcher(@NotNull String resourceTypeName)
        {
            this.resourceTypeName = resourceTypeName;
        }

        public boolean matchesNode(JONTreeNode node)
        {
            boolean result = false;

            if (node instanceof TreeNodeWithResourceType)
            {
                TreeNodeWithResourceType nodeWithResourceType = (TreeNodeWithResourceType)node;
                result = resourceTypeName.equals(nodeWithResourceType.getResourceType().getName());
            }

            return result;
        }
    }

    private class ResourceTypeNodeMatcher implements NodeMatcher
    {
        private ResourceType resourceType;

        public ResourceTypeNodeMatcher(@NotNull ResourceType resourceType)
        {
            if (resourceType == null)
            {
                throw new IllegalArgumentException("ResourceType cannot be null");
            }
            this.resourceType = resourceType;
        }

        public boolean matchesNode(JONTreeNode node)
        {
            boolean result = false;

            if (node instanceof TreeNodeWithResourceType)
            {
                TreeNodeWithResourceType nodeWithResourceType = (TreeNodeWithResourceType)node;
                result = resourceType.equals(nodeWithResourceType.getResourceType());
            }

            return result;
        }
    }

    // If you chain keys together (e.g. parent.getKey + getKey) you could get pretty much direct
    // access to an element in the tree without having to search for it.
//   public String getPath()
//   {
//      BaseTreeNode parent = getParent();
//
//      // getKey() doesn't really make any sense if you don't have a parent
//      // so lets just return whatever was specified in the constructor
//      return (parent == null) ? getKey() : getParent().getPath() + "~" + getKey();
//   }
}
