/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.ui.nav;

import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceSubCategory;
import org.rhq.core.domain.resource.ResourceType;

import org.jboss.on.embedded.ui.SummaryAction;
import org.jboss.on.embedded.ui.SummaryAction.SummaryActionOutcome;

/**
 * Class representing a node in the nav tree which holds a sub category.
 * <p/>
 * Supports having zero or more of each of the following as child nodes a) ResourceTypeTreeNode, for types tagged with
 * this sub category b) SubCategoryTreeNode, for sub categories defined as children of this one
 *
 * @author ccrouch
 */
public class SubCategoryTreeNode extends BaseTreeNode
{
    private ResourceSubCategory subCategory;
    private Resource parentResource;

    public SubCategoryTreeNode(String key, ResourceSubCategory subCategory, Resource parentResource)
    {
        super(key);
        if (subCategory == null)
        {
            throw new IllegalArgumentException("SubCategoryTreeNode requires a SubCategory. Null was passed into the constructor");
        }
        this.subCategory = subCategory;
        this.parentResource = parentResource;
    }

    protected void initChildrenMap()
    {
        // iterate over any resource types which have been tagged with this
        // sub category and add appropriate child nodes
        Collection<ResourceType> taggedResourceTypes = getSubCategory().findTaggedResourceTypes();
        for (ResourceType taggedResourceType : taggedResourceTypes)
        {
            addChildResourceTypeNodeWithSubCategory(taggedResourceType, parentResource);
        }

        // iterate over any child sub-categories of this one adding child nodes
        List<ResourceSubCategory> subCategories = getSubCategory().getChildSubCategories();
        Set<ResourceSubCategory> sortedSubCategories = new TreeSet<ResourceSubCategory>(subCategories);
        for (ResourceSubCategory subCategory : sortedSubCategories)
        {
            if (!subCategory.getChildSubCategories().isEmpty() || !subCategory.findTaggedResourceTypes().isEmpty())
            {
                addChild(new SubCategoryTreeNode(getUniqueKey(), subCategory, parentResource));
            }
        }
    }

    public Resource getClosestResource()
    {
        return parentResource;
    }

    public String getPath()
    {
        return getParent() == null ? getName() :
                getParent().getPath() + "/" + getName();
    }

    public String getName()
    {
        return getSubCategory().getName();
    }

    public String getDescription()
    {
        return getSubCategory().getDescription();
    }

    public ResourceSubCategory getSubCategory()
    {
        return this.subCategory;
    }

    public SummaryActionOutcome getSummaryActionOutcome(SummaryAction action)
    {
        return action.new SubCategoryOutcome(this);
    }

    @Override
    public String toString()
    {
        return getPath();
    }
}
