/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.test;

import java.util.HashMap;
import java.util.Map;

import org.rhq.core.domain.configuration.Property;
import org.rhq.core.domain.configuration.PropertySimple;
import org.testng.annotations.Test;

import org.jboss.on.embedded.TextareaMapConverter;

/**
 * @author Jason Dobies
 */
public class TextareaMapConverterTest
{
    // Attributes  --------------------------------------------

    private TextareaMapConverter converter = new TextareaMapConverter();

    // Test Cases  --------------------------------------------

    /**
     * Tests getAsObject when the input data is valid.
     */
    @Test
    // IE7 puts \r\n between lines
    public void testGetAsObjectValidIE7Data()
    {
        String stringValue =
                "property1=value1\r\n" +
                        "property2=value2\r\n" +
                        "property3=\r\n" +
                        "\r\n" +
                        "property6=6+7=13\r\n";

        assertConversionToObject(stringValue);
    }

    @Test
    // Firefox puts \n between lines
    public void testGetAsObjectValidFireFoxData()
    {
        String stringValue =
                "property1=value1\n" +
                        "property2=value2\n" +
                        "property3=\n" +
                        "\n" +
                        "property6=6+7=13\n";

        assertConversionToObject(stringValue);
    }

    private void assertConversionToObject(String stringValue)
    {
        Object uncastedObject = converter.getAsObject(null, null, stringValue);
        Map<String, Property> map = (Map<String, Property>)uncastedObject;

        PropertySimple simple;

        simple = (PropertySimple)map.get("property1");
        assert simple != null : "Property1 not extracted";
        assert simple.getStringValue().equals("value1") : "Property1 had an incorrect value";

        simple = (PropertySimple)map.get("property2");
        assert simple != null : "Property2 not extracted";
        assert simple.getStringValue().equals("value2") : "Property2 had an incorrect value";

        simple = (PropertySimple)map.get("property3");
        assert simple != null : "Property3 not extracted";
        assert simple.getStringValue().equals("") : "Property3 had an incorrect value";

        simple = (PropertySimple)map.get("property4");
        assert simple == null : "Property4 should not have been extracted";

        simple = (PropertySimple)map.get("property6");
        assert simple != null : "Property6 not extracted";
        assert simple.getStringValue().equals("6+7=13") : "Property6 had an incorrect value";
    }

    /**
     * Tests when a property does not follow the format x=y by missing the equals sign.
     */
    @Test
    public void testGetAsObjectInvalidData1()
    {
        String stringValue = "property4";
        try
        {
            converter.getAsObject(null, null, stringValue);
            assert false : "Did not throw an error for invalid data";
        }
        catch (Exception e)
        {
            // Expected
        }
    }

    /**
     * Tests when a property is defined with no name (i.e. begins with the equals sign).
     */
    @Test
    public void testGetAsObjectInvalidData2()
    {
        String stringValue = "=value5";
        try
        {
            converter.getAsObject(null, null, stringValue);
            assert false : "Did not throw an error for invalid data";
        }
        catch (Exception e)
        {
            // Expected
        }
    }

    /**
     * Tests the getAsString method.
     */
    @Test
    public void testGetAsString()
    {
        Map<String, Property> mapValue = new HashMap<String, Property>();
        mapValue.put("property1", new PropertySimple("blah1", "value1"));
        mapValue.put("property2", new PropertySimple("blah2", "value2"));
        mapValue.put("property3", new PropertySimple("blah3", ""));
        mapValue.put("property4", new PropertySimple("blah4", null));

        String stringValue = converter.getAsString(null, null, mapValue);

        /* Need to iterate in the following assertions since we can't be sure of the order in which the map
          will keep the properties. We might want to correct this, as displaying the data back to the user,
          such as for failed validation elsewhere on the form, might prove unsettling to see the entered
          properties out of order.
        */

        String[] lines = stringValue.split("\n");
        for (String line : lines)
        {
            if (line.startsWith("property1"))
            {
                assert line.equals("property1=value1");
            }
            else if (line.startsWith("property2"))
            {
                assert line.equals("property2=value2");
            }
            else if (line.startsWith("property3"))
            {
                assert line.equals("property3=");
            }
            else if (line.startsWith("property4"))
            {
                assert line.equals("property4=null");
            }
            else
            {
                assert false : "Invalid line read: " + line;
            }
        }

    }
}
