/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit;

import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.as5.ResourceTestBase;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import junit.framework.Test;
import org.jboss.mx.util.MBeanServerLocator;
import java.util.Iterator;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import java.io.File;
import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Map;
import java.util.HashMap;
import java.util.LinkedHashMap;
import javax.naming.InitialContext;
import java.sql.Connection;
import javax.sql.DataSource;
import javax.naming.Context;
import java.sql.SQLException;
import java.util.ArrayList;
import javax.management.*;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;

/**
 * This is the base test class for Embedded Jopr Datasource tests.
 * 
 * @author Farah Juma
 * @author Ondrej Zizka
 *
 */
public abstract class DatasourceTestBase extends ResourceTestBase {
    
    // Nav labels for datasources
    public static final String DS_NAV_LABEL = "Datasources";
    public static final String LOCAL_TX_DS_NAV_LABEL = "Local Tx Datasources";
    public static final String NO_TX_DS_NAV_LABEL = "No Tx Datasources";
    public static final String XA_DS_NAV_LABEL = "XA Datasources";
    
    // The root element in a datasource -ds.xml file
    public static final String DS_ROOT_ELEMENT = "datasources";
    
    // Datasource types, as they appear in the left nav
    public enum DatasourceType {

        // Some of these values are specific for AS 5.  TODO: Think up some way to split nicely.
        //                  // label                serviceName  xml-element-name       template <select>
        LOCAL_TX_DATASOURCE(LOCAL_TX_DS_NAV_LABEL, "LocalTxCM", "local-tx-datasource", "default__Local Tx Datasource",
                            "Local Tx Datasource"),
        NO_TX_DATASOURCE(   NO_TX_DS_NAV_LABEL,    "NoTxCM",    "no-tx-datasource",    "default__No Tx Datasource",
                            "No Tx Datasource"),  
        XA_DATASOURCE(      XA_DS_NAV_LABEL,       "XATxCM",    "xa-datasource",       "default__XA Datasource",
                            "XA Datasource");

        protected String label;
        public String getLabel() { return label; }
        
        protected final String serviceName;
        public String getServiceName() { return serviceName; }


        protected final String xmlElementName;
        public String getXmlElementName() {	return xmlElementName; }
        // Also serves as value for HTML Radio for Datasource type selection.
        public String getHtmlRadioValue() {	return xmlElementName; }


        protected final String templateHtmlSelectValue;
        public String getTemplateHtmlSelectValue() { return templateHtmlSelectValue; }
        
        protected final String typeName;
        public String getTypeName() { return typeName; }

        DatasourceType(String label, String serviceName, 
                       String xmlElementName, String htmlSelectValue, String typeName)
        {
            this.label = label;
            this.serviceName = serviceName;
            this.xmlElementName = xmlElementName;
            this.templateHtmlSelectValue = htmlSelectValue;
            this.typeName = typeName;
        }
        
    }// DatasourceTypes

    // --- Datasource Templates --- //
    /*
			AS 4:
			<select id="resourceCreateForm:selectedTemplate" size="1" name="resourceCreateForm:selectedTemplate">
				<option value="">Select Template</option>
				<option value="Oracle Local TX__Datasource">Oracle Local TX (Datasource)</option>
				<option value="Oracle XA__Datasource">Oracle XA (Datasource)</option>
				<option value="default__Datasource">default (Datasource)</option>
			</select>

		  AS 5:  Template selection depends on the active navtree node.

		  <select id="resourceCreateForm:selectedTemplate" size="1" name="resourceCreateForm:selectedTemplate">
				<option value="Oracle No TX__No TX Datasource">Oracle No TX (No TX Datasource)</option>
				<option value="default__No TX Datasource">default (No TX Datasource)</option>
			</select>
			<select id="resourceCreateForm:selectedTemplate" size="1" name="resourceCreateForm:selectedTemplate">
				<option value="Oracle Local TX__Local TX Datasource">Oracle Local TX (Local TX Datasource)</option>
				<option value="default__Local TX Datasource">default (Local TX Datasource)</option>
			</select>
			<select id="resourceCreateForm:selectedTemplate" size="1" name="resourceCreateForm:selectedTemplate">
				<option value="Oracle XA__XA Datasource">Oracle XA (XA Datasource)</option>
				<option value="default__XA Datasource">default (XA Datasource)</option>
			</select>

     */
    public enum DatasourceTemplate {

        /** AS 4 */
        AS4_ORACLE_LOCAL_TX("Oracle Local TX__Datasource"),
        AS4_ORACLE_XA("Oracle XA__Datasource"),
        AS4_DEFAULT("default__Datasource"),

        /** AS 5 */
        AS5_ORACLE_LOCAL_TX("Oracle Local TX__Local Tx Datasource"),
        AS5_DEFAULT_LOCAL_TX("default__Local Tx Datasource"),
        AS5_ORACLE_NO_TX("Oracle No TX__No Tx Datasource"),
        AS5_DEFAULT_NO_TX("default__No Tx Datasource"),
        AS5_ORACLE_XA("Oracle XA__XA Datasource"),
        AS5_DEFAULT_XA("default__XA Datasource");

        protected final String templateHtmlSelectValue;

        DatasourceTemplate(String templateHtmlSelectValue) {
            this.templateHtmlSelectValue = templateHtmlSelectValue;
        }

        /**
         * Value of HTML select option for this template.
         */
        public String getTemplateHtmlSelectValue() {
            return templateHtmlSelectValue;
        }
    }

    // Datasource properties
    protected Map<String, String> datasourceProperties = createDatasourceProperties();

    /**
     * @returns a set of properties created upon initialization by overriden createDatasourceProperties().
     */
    public Map<String, String> getDatasourceProperties() { return datasourceProperties; }

    /**
     * Create a new datasource using the given type, template, and properties.
     * 
     * @param datasourceType must be the name of a datasource type, as it 
     * appears in the left nav (eg. "Local TX Datasources")
     */
    protected abstract void createDatasource(DatasourceType datasourceType,
                                             String datasourceTemplate,
                                             Map<String, String> propertiesMap) 
																						throws IOException, EmbJoprTestException;

    /**
     * Use JMX to check if the datasource given by datasourceName is deployed.
     */
    protected boolean isDatasourceDeployed(String jndiName,  DatasourceType datasourceType) {
        try {

            String[] dsMBeanServices = {"DataSourceBinding",
                                        "ManagedConnectionPool",
                                        "ManagedConnectionFactory",
                                        datasourceType.getServiceName()};

            // Query the MBean server to check if the datasource is deployed
            MBeanServer jmxServer = MBeanServerLocator.locateJBoss();

            // Inspect these MBeans and make sure that the their state indicates successful deployment
            // (e.g. for AS 5, "State" attribute is "DEPLOYED"):
            // 1) "jboss.jca:name=TestDS,service=DataSourceBinding",type=Component
            // 2) "jboss.jca:name=TestDS,service=ManagedConnectionPool",type=Component
            // 3) "jboss.jca:name=TestDS,service=ManagedConnectionFactory",type=Component
            // 4) The fourth MBean inspected depends on the type of datasource.
            for(int i = 0; i < dsMBeanServices.length; i++) {

                // Is this necessary? Can't we just query? See InstanceNotFoundException;
                // And AFAIK, full MBean name is unique. I vote for minimalistic code.
                /*ObjectName objName = new ObjectName( this.getMBeanName(jndiName, dsMBeanServices[i]));

                Set dsMBeans = jmxServer.queryNames(objName, null);
                if (dsMBeans.size() != 1)  return false;

								// Get the first and only one MBean returned.
                ObjectName deploymentMBean = (ObjectName)dsMBeans.iterator().next(); /**/

                String mBeanName = this.getMBeanName(jndiName, dsMBeanServices[i]);
                log.info("Looking for MBean "+mBeanName+"...");
                ObjectName deploymentMBean = new ObjectName( mBeanName );

                if( this.isMBeanStateDeployedImpl( deploymentMBean ) ){
                    log.info("Found, OK");
                }else{
                    log.info("Not found.");
                    return false;
                }

            }

            return true;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * checkProperties reads the *-ds.xml file corresponding to the datasource 
     * given by jndiName and compares the property values in this file to the 
     * given expected values. 
     */
    protected void checkProperties(String jndiName,
                                   DatasourceType datasourceType,
                                   Map<String, String> expectedValuesMap)
    {

        Map<String, String> actualValuesMap = new HashMap<String, String>();


        try {

            // Parse the *-ds.xml file; create appropriate file name for AS 4 or 5.
            File file = new File(this.getDatasourceConfigFile(jndiName));
            log.info("Examining: "+file.getAbsolutePath());

            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(file);

            Element root = doc.getRootElement();
            assertTrue(root.getName().equals("datasources"));

            // Get the datasource element 
            Element datasource = root.getChild(datasourceType.getXmlElementName());

            // Create actualValuesMap by mapping property names to 
            // property values
            Iterator itr = (datasource.getChildren()).iterator();
            while(itr.hasNext()) {
                Element property = (Element)itr.next();
                actualValuesMap.put(property.getName(), property.getValue());
            }

            // Compare the actual values to the expected ones.
            for( String key : expectedValuesMap.keySet() ){
                assertTrue("Property '" + key + "' was not found in the -ds.xml file", actualValuesMap.containsKey(key));
                assertEquals("Incorrect value for '" + key + "' in the -ds.xml file: ", 
                             expectedValuesMap.get(key), actualValuesMap.get(key));
                
            }
        } catch (Exception e) {
            log.fatal("Config file check failed: " + e.getMessage());
            throw new RuntimeException(e);
        }
    }
    
    /**
     * Connect to the database identified by the given JNDI name,
     * using the given username and password. Return the Connection
     * object.
     */
    public Connection connectDB(String jndiName, String username, 
                                String password) throws Exception{
        Context ctx = new InitialContext();
        DataSource ds = (DataSource)ctx.lookup("java:" + jndiName);
        Connection con = ds.getConnection(username, password);
	    // actually exercise the connection
	    con.getMetaData();
	    return con;
    }
    
    /**
     * Create the specified number of connections to the given
     * database.
     *
     * @return the ArrayList of Connections.
     */
    public ArrayList<Connection> createConnections(int numConnections,
                                                   String jndiName,
                                                   String username,
                                                   String password) throws Exception {
        // Establish multiple connections 
        ArrayList<Connection> connections = new ArrayList<Connection>();
        
        for(int i = 0; i < numConnections; i++) {
            Connection con = connectDB(jndiName, username, password); 
            assertNotNull("Null connection", con);
            connections.add(con);
        }
        
        return connections;
    }
    
    /**
     * Disconnect from the database.
     */
    public void disconnectDB(Connection con) throws SQLException {
        if(con != null) {
            con.close();  
        } 
    }
    
    /**
     * Close each Connection in the given ArrayList of Connections.
     */
    public void closeConnections(ArrayList<Connection> connections) throws SQLException {
        if(connections != null) {
            for(int i = 0; i < connections.size(); i++) {
                disconnectDB(connections.get(i));
            }
        }
    }
    
    /**
     * compareFiles is used to check if the two given files are identical.
     */
    public void compareFiles(String actualFile, String expectedFile) throws Exception {
        BufferedReader actual = new BufferedReader(new FileReader(actualFile));
        BufferedReader expected = new BufferedReader(new FileReader(expectedFile));
        String expectedLine;
        String actualLine;
        
        // Compare the two files
        while((actualLine = actual.readLine()) != null) { 
          expectedLine = expected.readLine();
          assertTrue("Extra line in -ds.xml file: " + actualLine, expectedLine != null);
          assertEquals("Incorrect line in -ds.xml file: '" + actualLine + "' -  ",
                       expectedLine, actualLine);
        }
        
        expectedLine = expected.readLine();
        assertTrue("Missing line in -ds.xml file: " + expectedLine, expectedLine == null);
    }
    
    
    
    /**
     * @return the suite of tests being tested
		 * @throws UnsupportedOperationException - you have to override this.
		 *         Can't be abstract, must be static.
     */
    public static Test suite()
    {
      throw new UnsupportedOperationException("This has to be overriden.");
    }

    /**
     * Creates the default properties for the datasource - shared by AS4 and AS5.
     * Overriding method should get result of this method and overwrite it with it's custom properties.
     * @returns a set of default properties.
     */
    protected Map<String, String> createDatasourceProperties()
    {
        Map<String, MetaValue> propertiesMapAsMetaValues = createDatasourcePropertiesAsMetaValues();
        Map<String, String> propertiesMap = formatPropertiesMap(propertiesMapAsMetaValues);
        return propertiesMap;
    }

    /**
     * Creates the default properties for the datasource.
     * Returns a map of String names to MetaValues.
     */
    protected Map<String, MetaValue> createDatasourcePropertiesAsMetaValues() {
        Map<String, MetaValue> propertiesMap = new LinkedHashMap<String, MetaValue>();
        
        propertiesMap.put("jndi-name", SimpleValueSupport.wrap("TestDS"));
        propertiesMap.put("user-name", SimpleValueSupport.wrap("testUser"));
        propertiesMap.put("password", SimpleValueSupport.wrap("password"));
        propertiesMap.put("min-pool-size", SimpleValueSupport.wrap(new Integer(5)));
        propertiesMap.put("driver-class", SimpleValueSupport.wrap("org.hsqldb.jdbcDriver"));
        propertiesMap.put("connection-url", SimpleValueSupport.wrap("jdbc:hsqldb:."));
        propertiesMap.put("idle-timeout-minutes", SimpleValueSupport.wrap(new Integer(20)));
        propertiesMap.put("prepared-statement-cache-size", SimpleValueSupport.wrap(new Integer(2))); 
        propertiesMap.put("valid-connection-checker-class-name", 
                          SimpleValueSupport.wrap("org.jboss.resource.adapter.jdbc.CheckValidConnectionSQL"));
        propertiesMap.put("exception-sorter-class-name", 
                          SimpleValueSupport.wrap("org.jboss.resource.adapter.jdbc.ExceptionSorter"));
        
        return propertiesMap;
    }

    /**
     * Create a basic Local TX Datasource. Return the mapping of property
     * names to property values.
     */
    protected Map<String, String> createLocalTXDatasource(String datasourceName) throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("jndi-name", datasourceName);
        propertiesMap.put("max-pool-size", "20");
        propertiesMap.put("min-pool-size", "5");
        propertiesMap.put("user-name", "sa");
        propertiesMap.put("password", "");
        propertiesMap.put("securityDeploymentType", "DOMAIN");
        propertiesMap.put("domain", "HsqlDbRealm");
        propertiesMap.put("blocking-timeout-millis", "35000");
        propertiesMap.put("idle-timeout-minutes", "20");
        propertiesMap.put("driver-class", "org.hsqldb.jdbcDriver");
        propertiesMap.put("connection-url", "jdbc:hsqldb:${jboss.server.data.dir}${/}hypersonic${/}localDB");
        propertiesMap.put("use-java-context", "true");

        createDatasource(DatasourceType.LOCAL_TX_DATASOURCE,
                         DatasourceType.LOCAL_TX_DATASOURCE.getTemplateHtmlSelectValue(), // "default__Local Tx Datasource",
                         propertiesMap);
        client.click(SAVE_BUTTON);

        return propertiesMap;
    }


    /**
     * Create a basic No TX Datasource. Return the mapping of property
     * names to property values.
     */
    protected Map<String, String> createNoTXDatasource(String datasourceName) throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("jndi-name", datasourceName);
        propertiesMap.put("prefill", "false");
        //propertiesMap.put("track-connection-by-tx", "false"); // 1) Property metamorfed to "interleaving" 2) No use for No-TX.
        propertiesMap.put("max-pool-size", "20");
        propertiesMap.put("min-pool-size", "5");
        propertiesMap.put("blocking-timeout-millis", "55000");
        propertiesMap.put("idle-timeout-minutes", "60");
        propertiesMap.put("user-name", "sa");
        propertiesMap.put("password", "");
        propertiesMap.put("securityDeploymentType", "DOMAIN");
        propertiesMap.put("domain", "HsqlDbRealm");
        propertiesMap.put("driver-class", "org.hsqldb.jdbcDriver");
        propertiesMap.put("connection-url", "jdbc:hsqldb:${jboss.server.data.dir}${/}hypersonic${/}localDB");
        propertiesMap.put("use-java-context", "true");

        createDatasource(DatasourceType.NO_TX_DATASOURCE,
                         DatasourceType.NO_TX_DATASOURCE.getTemplateHtmlSelectValue(), //"default__No Tx Datasource",
                         propertiesMap);
        client.click(SAVE_BUTTON);

        return propertiesMap;
    }

    /**
     * Create a basic XA Datasource. Return the mapping of property
     * name to property values.
     */
    protected Map<String, String> createXADatasource(String datasourceName) throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("jndi-name", datasourceName);
        propertiesMap.put("xa-datasource-class", "org.postgresql.xa.PGXADataSource");
        propertiesMap.put("xa-resource-timeout", "36000");
        propertiesMap.put("max-pool-size", "15");
        propertiesMap.put("min-pool-size", "6");
        propertiesMap.put("set-tx-query-timeout", "false");
        propertiesMap.put("user-name", "testUser");
        propertiesMap.put("use-java-context", "true");

        createDatasource(DatasourceType.XA_DATASOURCE,
                         DatasourceType.XA_DATASOURCE.getTemplateHtmlSelectValue(), //"default__XA Datasource",
                         propertiesMap);
        client.click(SAVE_BUTTON);

        return propertiesMap;
    }

    /**
     *
     * @returns a name of the Datasource's config file to check the properties in.
     */
    protected abstract String getDatasourceConfigFile( String jndiName );

    protected abstract String getMBeanName( String jndiName, String serviceName );



    /**
     * This method is wrapper that handles JMX exceptions at Base level.
     * Intended to keep AS-version-specific method, isMBeanStateDeployedImpl(), as small as possible.
     * @param deploymentMBean
     * @return
     * @throws javax.management.JMException
     * @throws java.io.IOException
     */
    protected boolean isMBeanStateDeployed( ObjectName deploymentMBean ) {

        try {
            return this.isMBeanStateDeployedImpl(deploymentMBean);
        }
        // This super-exception includes JMX operation failures, including:
        // AttributeNotFoundException, InstanceNotFoundException, MalformedObjectNameException, ServiceNotFoundException
        catch (OperationsException ex) {
            log.warn("JMX operation error when retrieving MBean attribute.", ex);
            return false;
        }
        // All other JMX failures...
        catch (JMException ex) {
            log.warn("JMX error when retrieving MBean attribute.", ex);
            return false;
        }
        catch (IOException ex) {
            log.warn("I/O error when retrieving MBean attribute.", ex);
            return false;
        }

    }// isMBeanStateDeployed()

    /**
     * Make sure that the metrics corresponding to the given datasource
     * are correct.
     *
     * @param metricsMap maps metric names to the expected metric values.
     */
    protected void checkMetrics(String datasourceName,
                                DatasourceType datasourceType,
                                Map<String, String> metricsMap) throws IOException, EmbJoprTestException {
        
        //refreshTreeNode(DS_NAV_LABEL);
        if(!ejtt.getNavTree().getNodeByLabel(datasourceType.getLabel()).isExpanded()) {
            expandNavTreeArrow(datasourceType.getLabel());
        }
      
        HtmlAnchor datasource = getNavTreeLink(datasourceName);
        datasource.click();

        // Check values under the "Metrics" tab
        HtmlAnchor metricsLink = (HtmlAnchor)client.getElement(METRICS_TAB);
        metricsLink.click();

        for(Iterator i = metricsMap.keySet().iterator(); i.hasNext();) {
            String metricName = (String)i.next();
            String expected = metricsMap.get(metricName);
            String actual = getMetricValueFromTable(metricName, DATA_TABLE);
            actual = normalizeIfDoubleExpected(expected, actual); // Some LOCALEs encode doubles with coma.
            assertEquals("Incorrect metric value for '" + metricName + "'", expected, actual);
        }

        // Check values under the "Summary" tab
        HtmlAnchor summaryLink = (HtmlAnchor)client.getElement(SUMMARY_TAB);
        summaryLink.click();

        ArrayList<String> summaryMetrics = new ArrayList<String>();
        summaryMetrics.add(AVAILABLE_CONNECTION_COUNT);
        summaryMetrics.add(CONNECTION_COUNT);

        for(Iterator i = summaryMetrics.iterator(); i.hasNext(); ) {
            String metricName = (String)i.next();
            String expected = metricsMap.get(metricName);
            String actual = getMetricValueFromTable(metricName, DATA_TABLE);
            actual = normalizeIfDoubleExpected(expected, actual); // Some LOCALEs encode doubles with coma.
            assertEquals("Incorrect summary metric value for '" + metricName + "'='" + actual + "'," +
                         " expected '"+expected+"'", expected, actual);
        }
    }

    /**
     * This method should query the JMX server and decide whether the given MBean displays deployed resource.
     * @param deploymentMBean  Name of the MBean to examine. Differs between AS4 and 5.
     * @return true  if the MBean indicates that the resource is deployed, false otherwise.
     * @throws javax.management.JMException  upon JMX related error, including invalid
     *                 or non-existent MBean / attribute name.
     * @throws java.io.IOException  upon I/O error.
     */
    protected abstract boolean isMBeanStateDeployedImpl( ObjectName deploymentMBean ) throws JMException, IOException;
		
    /**
     * Parse the -ds.xml file given by jndiName. Return a mapping of property
     * names to property values. 
     */
    public Map<String, String> getPropertyValuesMap(String jndiName, 
                                                    DatasourceType datasourceType) throws Exception {
        File file = new File(this.getDatasourceConfigFile(jndiName));
        
        Map<String, String> actualValuesMap = new HashMap<String, String>();
        
        SAXBuilder builder = new SAXBuilder();
        Document doc = builder.build(file);

        Element root = doc.getRootElement();
        assertTrue(root.getName().equals("datasources"));

        // Get the datasource element 
        Element datasource = root.getChild(datasourceType.getXmlElementName());

        // Create actualValuesMap by mapping property names to 
        // property values
        Iterator itr = (datasource.getChildren()).iterator();
        while(itr.hasNext()) {
            Element property = (Element)itr.next();
            actualValuesMap.put(property.getName(), property.getValue());
        }
        
        return actualValuesMap;
    }
}

