/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit;

import com.gargoylesoftware.htmlunit.BrowserVersion;
import com.gargoylesoftware.htmlunit.Page;
import com.gargoylesoftware.htmlunit.WebClient;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import java.util.List;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.jsfunit.framework.WebClientSpec;
import org.jboss.jsfunit.framework.WebConversationFactory;
import org.jboss.jsfunit.jsfsession.JSFClientSession;
import org.jboss.jsfunit.jsfsession.JSFServerSession;
import org.jboss.jsfunit.jsfsession.JSFSession;

/**
 * Test of multiple user parallel login.
 *
 * @author Ondrej Zizka
 */
public class MultipleUserLoginTest extends EmbjoprTestCase
{

	// Credentials 
	private static final String USER_A = "admin";
	private static final String PASS_A = "admin";
	private static final String USER_B = "admin2";
	private static final String PASS_B = "admin2";


	/**
	* @return the suite of tests being tested
	*/
	public static Test suite() {
		return new TestSuite( MultipleUserLoginTest.class );
	}
	private static final String LABEL_LOGOUT = "Logout";



	/** Disabled, second login solved using HTMLUnit. */
	public void DISABLEDsetUp() throws IOException
	{

		// Announce this test in the JBoss log.
		log.info("========================================================================================");
		log.info("  Setting up test "+this.getClass().getName()+"#"+this.getName());
		log.info("========================================================================================");


		// JVM version
		log.info("Java version: "+System.getProperty("java.version") + "," +
						 " vendor: "     +System.getProperty("java.vendor"));
		log.info("Java home:  "  +System.getProperty("java.home"));

		// JBoss AS version
		String version = Package.getPackage("org.jboss.system.server")
										 .getImplementationVersion();
		log.info("Detected AS version: " + version);
		isJBoss4 = version.startsWith("4");

		// Contrary to super class, this test skips the session creation in setUp().

	}
	/**/

	/**
	 * Instance of JSF session. Keeps server and client session parts, and user's name and pass.
	 */
	class SessionInstance {

		protected JSFSession jsfSession;
		protected JSFClientSession client_;
		protected JSFServerSession server_;
		protected String user;
		protected String pass;

		/** Creates new JSF session, using JoprLoginStrategy. */
		public SessionInstance(String user, String pass) throws IOException {

			// Code taken from EmbJoprTestCase.java.
			// Initial JSF request
			WebClientSpec wcSpec = new WebClientSpec("/", BrowserVersion.FIREFOX_3);
			wcSpec.getWebClient().setThrowExceptionOnFailingStatusCode(false);
			wcSpec.getWebClient().setConfirmHandler(new SimpleConfirmHandler(true));
			wcSpec.setInitialRequestStrategy(new JoprLoginStrategy(user, pass)); // logs in
			this.jsfSession = new JSFSession(wcSpec);
			this.client_ = jsfSession.getJSFClientSession();
			this.server_ = jsfSession.getJSFServerSession();
		}

		//public JSFClientSession getClient() {			return this.client_;		}
		//public JSFServerSession getServer() {			return this.server_;		}
		public JSFClientSession getClient() {			return this.jsfSession.getJSFClientSession();		}
		public JSFServerSession getServer() {			return this.jsfSession.getJSFServerSession();		}
		public String getUser() {			return user;		}
		public String getPass() {			return pass;		}

	}


	/**
	 * Logs in using multiple accounts simultaneously.
	 * @throws java.io.IOException
	 *
	 * PASSED.
	 */
	public void testMultipleUsersLogin() throws IOException
	{
		
		// USER A.

		String pageText = client.getPageAsText();
		assertTrue("Page doesn't contain user's name.", pageText.contains(USER_A) );
		assertTrue("Page doesn't contain '"+LABEL_LOGOUT+"'.", pageText.contains(LABEL_LOGOUT) );

		assertEquals("JSF's '#{identity.username}' should be the user name.",
						USER_A, server.getManagedBeanValue("#{identity.username}") );
		




		// USER B.

		final WebClient wc = new WebClient(BrowserVersion.FIREFOX_3);
		wc.setThrowExceptionOnFailingStatusCode(false);
		wc.setConfirmHandler(new SimpleConfirmHandler(true));
		final WebClientTools sessBclient = new WebClientTools(wc);
		
		final String userNameComponent = "login_form:name";
		final String passwordComponent = "login_form:password";
		final String submitComponent = "login_form:submit";

		String url = WebConversationFactory.getWARURL() + "/";
		final HtmlPage loginPage = wc.getPage(url);
		setValue(loginPage, userNameComponent, USER_B);
		setValue(loginPage, passwordComponent, PASS_B);
		clickSubmitComponent(loginPage, submitComponent);

		String pageBText = sessBclient.getPageAsText();
		assertTrue("Page doesn't contain user's name.", pageBText.contains(USER_B) );
		assertTrue("Page doesn't contain '"+LABEL_LOGOUT+"'.", pageBText.contains(LABEL_LOGOUT) );


		
		// Refresh and log USER_A out.

		//((HtmlPage) client.getContentPage()).refresh();
		//byte[] responseBBody = client.getContentPage().getWebResponse().getResponseBody();

		HtmlAnchor logoutLink = (HtmlAnchor) client.getElement("logoutLink");
		logoutLink.click();


		// Refresh sessB page Check that USER_B is still logged in.

		((HtmlPage) new WebClientTools(wc).getContentPage()).refresh();
		String pageBText2 = sessBclient.getPageAsText();
		assertTrue("Page doesn't contain user's name.", pageBText2.contains(USER_B) );
		assertTrue("Page doesn't contain '"+LABEL_LOGOUT+"'.", pageBText2.contains(LABEL_LOGOUT) );
	}




	/**
	 * Perhaps reveals EMBJOPR-92 - Nav tree is not filled right after login
	 */
	public void testMultipleSuccessiveLogins() throws IOException
	{
		final WebClient wc = new WebClient(BrowserVersion.FIREFOX_3);
		wc.setThrowExceptionOnFailingStatusCode(false);
		wc.setConfirmHandler(new SimpleConfirmHandler(true));
		final WebClientTools clientTools = new WebClientTools(wc);

		final String userNameComponent = "login_form:name";
		final String passwordComponent = "login_form:password";
		final String submitComponent = "login_form:submit";

		final String url = WebConversationFactory.getWARURL() + "/";

		int retries = 10;

		while( retries-- > 0 ) {

			log.info("Logging in, "+retries+" logins remaining after this one.");
			
			// Login
			HtmlPage loginPage = wc.getPage(url);
			setValue(loginPage, userNameComponent, USER_B);
			setValue(loginPage, passwordComponent, PASS_B);
			clickSubmitComponent(loginPage, submitComponent);


			String pageBText = clientTools.getPageAsText();
			assertTrue("Page doesn't contain user's name.", pageBText.contains(USER_B) );
			assertTrue("Page doesn't contain '"+LABEL_LOGOUT+"'.", pageBText.contains(LABEL_LOGOUT) );

			// Logout
			HtmlAnchor logoutLink = (HtmlAnchor) clientTools.getElement("logoutLink");
			logoutLink.click();
		}
	}


	

	// These were taken from FomrAuthenticationStrategy.java; unfortunately not reusable from there.

	protected Page clickSubmitComponent(HtmlPage page, String elementName) throws IOException
	{
		HtmlElement htmlElement = getElement(page, elementName);
		if (!(htmlElement instanceof ClickableElement))
			throw new IllegalStateException("Component with name=" + elementName + " is not a ClickableElement.");

		ClickableElement clickable = (ClickableElement)htmlElement;
		return clickable.click();
	}


	public static void setValue(HtmlPage page, String elementName, String value)
	{
		HtmlElement element = getElement(page, elementName);
		if (!(element instanceof HtmlInput))
		{
			 throw new IllegalStateException("Component with name=" + elementName + " is not an HtmlInput element.");
		}
		HtmlInput inputElement = (HtmlInput)element;
		inputElement.setValueAttribute(value);
	}

	public static HtmlElement getElement(HtmlPage page, String elementName)
	{
		List<HtmlElement> elements = page.getHtmlElementsByName(elementName);
		if (elements.size() == 0)
		{
			 throw new IllegalStateException("Component with name=" + elementName + " was not found on the login page.");
		}
		if (elements.size() > 1)
		{
			 throw new IllegalStateException("More than one component with name=" + elementName + " was found on the login page.");
		}

		return elements.get(0);
	}


}// class



