/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5;

import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import org.jboss.jopr.jsfunit.*;
import java.util.Map;
import org.jboss.metatype.api.values.MetaValue;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import java.io.File;
import org.jboss.managed.api.ComponentType;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedHashMap;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.NavTreeNode;
import org.jboss.jopr.jsfunit.exceptions.*;

/**
 * This class contains basic methods for creating, configuring, and deleting
 * resources (eg. Topics, Queues, etc.). 
 * 
 * @author Farah Juma
 *
 */

public abstract class ResourceTestBase extends EmbjoprTestCase {
   
    // Error messages that get displayed during resource creation/configuration
    public static final String INVALID_VALUE_MESSAGE = "An invalid value was specified for one or more properties";
    public static final String MISSING_VALUE_MESSAGE = "Value is required";
    public static final String OUT_OF_RANGE_MESSAGE = "Specified attribute is not between the expected values";
    public static final String INVALID_INTEGER_MESSAGE = "Value is not a valid integer";
    public static final String ZERO_VALUE_MESSAGE = "Expected non-zero value for ";
    public static final String BELOW_ALLOWABLE_MINIMUM = "Value is less than allowable minimum";
    
    // Success messages that get displayed after creating/deleting/updating a resource
    public static final String ADD_MESSAGE = "Successfully added new ";
    public static final String DELETE_MESSAGE = "Successfully deleted ";
    public static final String UPDATE_MESSAGE = "Successfully updated ";
    
    // Buttons
    public static final String SAVE_BUTTON = "resourceConfigurationForm:saveButton";
    public static final String PARAMETERS_FORM_OK_BUTTON = "parametersForm:okButton";
    public static final String ADD_NEW_RESOURCE_BUTTON = "actionHeaderForm:addNewNotContent";
    public static final String CONTINUE_AFTER_TEMPLATE_SELECTION_BUTTON = "resourceCreateForm:addButton";
    public static final String EDIT_RESOURCE_CONFIGURATION_FORM_OK_BUTTON = "editResourceConfigurationForm:okButton";
    public static final String ADD_NEW_CONFIG_BUTTON = "Add New";
    
    // Template selection
    public static final String SELECT_TEMPLATE = "selectedTemplate";
    
    // These values appear in the operations history table when the operation
    // does not have any results or parameters
    public static final String NO_RESULTS_MESSAGE = "The operation did not return any results";
    public static final String NO_PARAMETERS_MESSAGE = "This operation does not take any parameters";
    
    // Operation history status
    public static final String INPROGRESS = "In Progress";
    public static final String SUCCESSFUL = "Successful";
    public static final String FAILED = "Failed";
    
    // IDs (if these IDs get changed in resourceInstanceOperation.xhtml,
    // resourceTypeSummary.xhtml, etc., these values will need to be updated)
    public static final String OPERATIONS_FORM = "operationsForm";
    public static final String OPERATION_HISTORY_FORM = "historyDetailsPanel";
    public static final String DATA_TABLE = "dataTable";
    public static final String NO_PARAMETERS = "noParameters";
    public static final String NO_RESULTS = "noResults";
    public static final String OPERATION_RESULTS = "operationResults";
    public static final String RESOURCE_SUMMARY_FORM = "resourceSummaryForm";
    public static final String EDIT_RESOURCE_CONFIGURATION_FORM = "editResourceConfigurationForm";
    public static final String PARAMETERS_FORM = "parametersForm";
    public static final String ERROR_MESSAGE = "errorMessage";

    // Pagination constants
    public static final String MAX_ITEMS_PER_PAGE="50";
    public static final String CURRENT_PAGE_SIZE = "currentPageSize";
    public static final String NEXT_PAGE = "nextPage";
    
    // Datasource and ConnectionFactory operations
    public static final String FLUSH = "Flush";
    public static final String LIST_FORMATTED_SUB_POOL_STATISTICS = "List Formatted Sub Pool Statistics";
    public static final String LIST_STATISTICS = "List Statistics";
    
    // Argument boxes on the operations parameters page
    public static final String ARG_0 = "arg#0";
    
    // Datasource and ConnectionFactory metrics
    public static final String AVAILABLE_CONNECTION_COUNT = "Available Connection Count";
    public static final String CONNECTION_COUNT = "Connection Count";
    public static final String CONNECTION_CREATED_COUNT = "Connection Created Count";
    public static final String CONNECTION_DESTROYED_COUNT = "Connection Destroyed Count";
    public static final String IN_USE_CONNECTION_COUNT = "In Use Connection Count";
    public static final String MAX_CONNECTIONS_IN_USE_COUNT = "Max Connections In Use Count";
    public static final String MAX_SIZE = "Max Size";
    public static final String MIN_SIZE = "Min Size";
    
    // Gets displayed in the output of the "List Formatted Sub Pool Statistics" operation
    public static final String POOL_SEPARATOR = "------------------------------------------------------";
    
    // Useful constants for the tests
    public static final int FIRST_ROW = 1;
    public static final int OPERATION_STATUS_COLUMN = 2;
    public static final int NAME_COLUMN = 0;
    public static final int VALUE_COLUMN = 1;
    public static final String EMPTY_LIST = "[]";
    public static final String TABLE_CELL_DELIM = "<td>";
    public static final String TABLE_CELL_END_DELIM = "</td>";
    public static final String TABLE_ROW_DELIM = "<tr>";
    public static final String TABLE_ROW_END_DELIM = "</tr>";
    public static final String STATUS = "Status";
    public static final String UP = "UP";
    public static final String DOWN = "DOWN";
    
    // Datasource and Connection Factory properties to check during the "List Statistics" tests
    public String[] listStatisticsProps = new String[] { "blocking-timeout-millis", "idle-timeout-minutes",
                                                         "max-pool-size", "min-pool-size", "no-tx-separate-pools",
                                                         "prefill" /*, "track-connection-by-tx"  JBAS-6918 */ };
    
    /**
     * Create a new resource using the given type, template, and properties.
     *   
     * @param propertiesMap is a map of property names to property meta values
     * @param resourceType must be the type of datasource, connection factory,
     * or JMS destination, as it appears in the left nav (eg. "Local TX Datasources",
     * "Queues", "Topics", etc").
     *
     */
    protected void createResource(String resourceType,
                                  String resourceTemplate,
                                  Map<String, MetaValue> propertiesMap) throws IOException {
        HtmlAnchor resourceLink = getNavTreeLink(resourceType);
        resourceLink.click();

        // Add a new resource 
        client.click(ADD_NEW_RESOURCE_BUTTON);
        HtmlSelect menu = (HtmlSelect)client.getElement(SELECT_TEMPLATE);
        menu.setSelectedAttribute(resourceTemplate, Boolean.TRUE);
        client.click(CONTINUE_AFTER_TEMPLATE_SELECTION_BUTTON);
        
        // Configure the properties associated with this resource
        setResourceProperties(propertiesMap);  
    }
    
    /**
     * Set the properties associated with this resource.
     */
    protected void setResourceProperties(Map<String, MetaValue> propertiesMap) {
        Map<String, String> formattedPropertiesMap = formatPropertiesMap(propertiesMap);
        fillOutForm(formattedPropertiesMap, RESOURCE_CONFIGURATION_FORM);  
    }


    /**
     * Configure the properties associated with this resource
     * and verify that the properties were updated appropriately.
     *
     * Assumes the resource tree node (eg. "JMS Destinations", "Datasources",
     * etc.) is already expanded.
     */
    protected void configureResource(String category, String subCategory, String name, Map<String, MetaValue> propsMap,
        Map<String, MetaValue> propsMapChanges, ComponentType componentType, String expectedMessage) throws Exception {
			configureResource(category, subCategory, name, propsMap, propsMapChanges, componentType, expectedMessage, true);
		}
		
    /**
     * Configure the properties associated with this resource
     * and verify that the properties were updated appropriately.
     *
     * Assumes the resource tree node (eg. "JMS Destinations", "Datasources",
     * etc.) is already expanded.
     */
    protected void configureResource(String resourceCategory,
                                     String resourceSubCategory,
                                     String resourceName,
                                     Map<String, MetaValue> propertiesMap,
                                     Map<String, MetaValue> propertiesMapChanges,
                                     ComponentType componentType,
                                     String expectedMessage,
																		 boolean secured) throws Exception {
        
        // Navigate to the configuration page for the resource
        navigateToPage(resourceCategory, resourceSubCategory, 
                       resourceName, CONFIGURATION_TAB);
        
        setResourceProperties(propertiesMapChanges);
        client.click(SAVE_BUTTON);
        
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);
        
        // Verify that the properties were set correctly
        checkComponentProperties(propertiesMapChanges, resourceName, componentType, secured);
    }


    /**
     * Unset the specified properties for the given resource.
     * @param propertiesMap is a map of property names to expected property values
     */
    protected void unsetResourceProperties(String category, String subCategory, String name, Map<String, MetaValue> propsMap,
					String[] propsToUnset, ComponentType componentType, String expectedMessage) throws Exception {
			unsetResourceProperties(subCategory, subCategory, name, propsMap, propsToUnset, componentType, expectedMessage, true);
		}

    /**
     * Unset the specified properties for the given resource.
     * @param propertiesMap is a map of property names to expected property values
     */
    protected void unsetResourceProperties(String resourceCategory,
                                           String resourceSubCategory,
                                           String resourceName,
                                           Map<String, MetaValue> propertiesMap,
                                           String[] propertiesToUnset,
                                           ComponentType componentType,
                                           String expectedMessage,
																					 boolean secured) throws Exception {
        
        // Navigate to the configuration page for the resource
        navigateToPage(resourceCategory, resourceSubCategory, 
                       resourceName, CONFIGURATION_TAB);
        
        // Unset the given properties
        for(int i = 0; i < propertiesToUnset.length; i++) {
            String propertyName = propertiesToUnset[i];
            enableOrDisableFormInput(propertyName, Boolean.FALSE, RESOURCE_CONFIGURATION_FORM);
        }
        
        client.click(SAVE_BUTTON);
        
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);
        
        // Make sure the rest of the properties remained unchanged
        checkComponentProperties(propertiesMap, resourceName, componentType, secured);
    }

		
    /**
     * Delete the given resource. 
     * 
     * @param resourceFormName The name of the form that contains the delete button.
     */
    protected void deleteResource(String resourceFormName, String resourceName) throws IOException {
        
        HtmlSelect menu = (HtmlSelect)client.getElement(CURRENT_PAGE_SIZE);
        if(menu != null) {
            menu.setSelectedAttribute(MAX_ITEMS_PER_PAGE, Boolean.TRUE);
        }
        
        HtmlButtonInput deleteButton;
        try {
            deleteButton = getDeleteButton(resourceFormName, resourceName);
            deleteButton.click();
        } catch (IllegalStateException e) { 
            
            // The delete button was not found on this page
            client.click(NEXT_PAGE);
            deleteButton = getDeleteButton(resourceFormName, resourceName);
            deleteButton.click();
        }  
    }
    
    /**
     * containsElement returns whether or not the given deployment contains 
     * the given element.
     * 
     * @param fileName is the full path to the deployment
     * @param elementName is the element we are looking for
     */
    protected boolean containsElement(String fileName, String elementName, 
                                      String expectedRootElement) {
        try {
            File file = new File(fileName);
            SAXBuilder builder = new SAXBuilder();
            Document doc = builder.build(file);

            Element root = doc.getRootElement();
            assertTrue(root.getName().equals(expectedRootElement));
            return root.getChild(elementName) != null;
        } catch(Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    /**
     * Navigate to the given tab (eg. "configurationTab", "controlTab") for 
     * the given resource.
     * 
     * navigateToPage() assumes that the resource tree node (eg. "JMS Destinations", 
     * "Datasources", etc.) is already expanded when this method is called.
     */
    protected void navigateToPage(String resourceCategory, 
                                  String resourceSubCategory, 
                                  String resourceName,
                                  String tabName) throws Exception {
        //refreshTreeNode(resourceCategory);        
    	NavTreeNode resourceSubCategoryNode = ejtt.getNavTree().getNodeByLabel(resourceSubCategory);
    	if(!resourceSubCategoryNode.isExpanded()) {
    		expandNavTreeArrow(resourceSubCategory);
    	}
    	
        clickNavTreeLink(resourceName);
      
        HtmlAnchor tabLink = (HtmlAnchor)client.getElement(tabName);
        tabLink.click();
    }
    
    /**
     * Perform the given operation on the given resource.
     * 
     * performResourceOperation() assumes that the resource tree 
     * node (eg. "JMS Destinations", "Datasources", etc.) is already 
     * expanded when this method is called.
     */
    protected void performResourceOperation(String resourceCategory, 
                                            String resourceSubCategory,
                                            String resourceName,
                                            String operationName,
                                            Boolean takesParameters) throws Exception{

        navigateToPage(resourceCategory, resourceSubCategory, 
                       resourceName, CONTROL_TAB);

        HtmlForm form = (HtmlForm)client.getElement(OPERATIONS_FORM);
        String xpath = ".//input[@value=\"" + operationName + "\"]";

        HtmlButtonInput operationButton = (HtmlButtonInput)form.getFirstByXPath(xpath);
				if( null == operationButton )
					throw new HtmlElementNotFoundException( String.format("Button '%s' not found using xPath: "+ xpath, operationName));

        operationButton.click();
        
        if(takesParameters) {
            
            // Use default values
            client.click(PARAMETERS_FORM_OK_BUTTON);
        }
        
        waitForResourceOperationToFinish();

        checkOperationStatus();
    }
    
    /**
     * Perform the given operation on the given resource using the given
     * parameters.
     * 
     * performResourceOperation() assumes that the resource tree 
     * node (eg. "JMS Destinations", "Datasources", etc.) is already 
     * expanded when this method is called.
     * 
     * @param parametersMap maps parameter names to parameter values
     */
    protected void performResourceOperationWithParameters(String resourceCategory, 
                                                          String resourceSubCategory,
                                                          String resourceName,
                                                          String operationName,
                                                          Map<String, String> parametersMap) throws Exception{

        navigateToPage(resourceCategory, resourceSubCategory, 
                       resourceName, CONTROL_TAB);

        HtmlForm form = (HtmlForm)client.getElement(OPERATIONS_FORM);
        String xpath = ".//input[@value=\"" + operationName + "\"]";

        HtmlButtonInput operationButton = (HtmlButtonInput)form.getFirstByXPath(xpath);
        operationButton.click();
        
        fillOutForm(parametersMap, PARAMETERS_FORM);
        client.click(PARAMETERS_FORM_OK_BUTTON);
        
        waitForResourceOperationToFinish();

        checkOperationStatus();
    }
    
    /**
     * Check the status of the latest operation. If the operation was not 
     * successful, output the error message.
     */
    private void checkOperationStatus() throws HtmlElementNotFoundException {
        
        // Make sure the operation completed successfully
        String status = getLatestOperationStatus();

        String errorMessage = "";

        // Get the error message for operations that are not successful
        if(status.contains(FAILED)) {
            errorMessage = ((HtmlDivision)client.getElement(ERROR_MESSAGE)).getTextContent();
        } else if(status.contains(INPROGRESS)) {
            errorMessage = "The operation did not complete in the given time";
        }

        assertTrue("The operation was not successful, the result was [" + errorMessage + "].", 
                   status.contains(SUCCESSFUL));
    }
    
    /**
     * Check if the latest resource operation has completed.
     */
    protected void waitForResourceOperationToFinish() throws HtmlElementNotFoundException, IOException, InterruptedException {
        
        for(int i = 0; i < 40; i++) {
            
            String status = getLatestOperationStatus();
            
            if(status.contains(INPROGRESS)) {
                Thread.sleep(100);
            } else { 
                
                // The operation has completed
                break;
            }
        }
    }
    
    /**
     * Get the status of the latest operation.
     */
    protected String getLatestOperationStatus() throws HtmlElementNotFoundException {
        return ejtt.getTabMenu().getTabContentBox().getOperationsHistoryTable().getTableCell(FIRST_ROW, OPERATION_STATUS_COLUMN);
    }
    
    /**
     * Make sure that the metrics corresponding to the given resource
     * are correct.
     *
     * @param metricsMap maps metric names to the expected metric values.
     * 
     * Assumes the resource tree node (eg. "JMS Destinations", "Datasources",
     * etc.) is already expanded.
     */
    protected void checkResourceMetrics(String resourceCategory, 
                                        String resourceSubCategory,
                                        String resourceName,
                                        Map<String, String> metricsMap,
                                        ArrayList<String> summaryMetrics) throws Exception {
        
        // Navigate to the metrics page for the resource
        navigateToPage(resourceCategory, resourceSubCategory, 
                       resourceName, METRICS_TAB);
        
        checkMetricValues(metricsMap.keySet().iterator(), metricsMap, 
                          "Incorrect metric value for ");

        if(summaryMetrics.size() > 0) {

            // Check values under the "Summary" tab
            HtmlAnchor summaryLink = (HtmlAnchor)client.getElement(SUMMARY_TAB);
            summaryLink.click();

            checkMetricValues(summaryMetrics.iterator(), metricsMap,
                              "Incorrect summary metric value for ");
        }
    }
    
    /**
     * Compare the actual metric values for this resource to
     * the expected values.
     * 
     * @param i is an Iterator over the metrics we want to check
     * @param metricsMap maps metric names to their expected values
     */
    protected void checkMetricValues(Iterator i, Map<String, String> metricsMap, String errorMessage) {

        while(i.hasNext()) {
            String metricName = (String)i.next();

            String expected = metricsMap.get(metricName);
            String actual = getMetricValueFromTable(metricName, DATA_TABLE);
            actual = normalizeIfDoubleExpected(expected, actual);
            assertEquals(errorMessage + " '" + metricName + "'", expected, actual);
        }
    }   

    /**
     * Make sure that the specified metrics corresponding to the given resource
     * are not equal to nullValue. 
     *
     * @param i is an Iterator over the metrics we want to check
     * 
     */
    protected void checkResourceMetricsNotNull(String resourceCategory, 
                                               String resourceSubCategory,
                                               String resourceName,
                                               Iterator i,
                                               String errorMessage,
                                               String nullValue) throws Exception {

        // Navigate to the metrics page for the resource
        navigateToPage(resourceCategory, resourceSubCategory, 
                       resourceName, METRICS_TAB);

        while(i.hasNext()) {
            String metricName = (String)i.next();
            String actual = getMetricValueFromTable(metricName, DATA_TABLE);
            assertTrue(errorMessage + "'" + metricName + "' but was: " + actual, !actual.equals(nullValue));
        }
    }
    
    /**
     * Perform the given operation for the given resource and
     * then make sure that the metrics associated with the resource
     * were updated appropriately.
     */
    protected void performResourceOperationAndCheckMetrics(String resourceCategory,
                                                           String resourceSubCategory,
                                                           String resourceName,
                                                           String operationName,
                                                           Map<String, String> expectedMetrics,
                                                           ArrayList<String> summaryMetrics,
                                                           Boolean takesParameters) throws Exception {
        performResourceOperation(resourceCategory, resourceSubCategory, 
                                 resourceName, operationName, takesParameters);

        // Assumes the operation takes no parameters and does not return any
        // results
        checkNoParametersAndNoResults();

        // Verify that the metrics were updated appropriately
        checkResourceMetrics(resourceCategory, resourceSubCategory, resourceName,
                             expectedMetrics, summaryMetrics);
    }
    
    /**
     * Perform the given operation for the given resource and 
     * then check that the "Value" box in the operations history table
     * contains the correct text.
     */
    protected void performResourceOperationAndCheckValueBox(String resourceCategory,
                                                            String resourceSubCategory,
                                                            String resourceName,
                                                            String operationName,
                                                            Boolean takesParameters,
                                                            StringBuffer expectedBuffer,
                                                            String errorMessage) throws Exception {
        performResourceOperation(resourceCategory, resourceSubCategory, 
                                 resourceName, operationName, takesParameters);
        
        // Get the result of the operation
        HtmlDivision historyPanel = (HtmlDivision)client.getElement(OPERATION_RESULTS);
        HtmlTextArea resultTextBox = (HtmlTextArea)historyPanel.getFirstByXPath(".//textarea[@class='property-value-input']"); 

        assertNotNull("Could not get the result of the operation", resultTextBox);
        
        String actualResult = resultTextBox.getText();
        
        assertTrue(errorMessage + " - \nexpected the result to contain:\n\n" 
                   + expectedBuffer.toString() + "\n\nbut was:\n\n" + actualResult,
                   actualResult.contains(expectedBuffer.toString()));
    }
    
    /**
     * Perform the given operation for the given resource and
     * then check that the resulting table contains the correct values.
     */
    protected void performResourceOperationAndCheckTable(String resourceCategory,
                                                         String resourceSubCategory,
                                                         String resourceName,
                                                         String operationName,
                                                         Boolean takesParameters,
                                                         Map<String, String> expectedValues) throws Exception {
        performResourceOperation(resourceCategory, resourceSubCategory, 
                                 resourceName, operationName, takesParameters);
        
        // Get the resulting table: it will have two columns, Name and Value
        HtmlDivision historyPanel = (HtmlDivision)client.getElement(OPERATION_RESULTS);
        HtmlTable table = (HtmlTable)historyPanel.getFirstByXPath(".//table[@class='openmap-properties-table']");
        
        List<HtmlTableRow> rows = table.getRows();
        Iterator i = rows.iterator();
        HtmlTableRow headerRow = (HtmlTableRow)i.next(); // Skip over the row that contains column names

        while(i.hasNext()) {
            
            // Get the actual value
            HtmlTableRow row = (HtmlTableRow)i.next();
            String rowName = ((HtmlTableCell)row.getCell(NAME_COLUMN)).asText();
            String rowValue = ((HtmlTableCell)row.getCell(VALUE_COLUMN)).asText();
            
            String expectedValue = expectedValues.get(rowName);
            assertEquals("Incorrect value for '" + rowName + "' - ", expectedValue, rowValue);
        }
    }
    
    /**
     * Make sure the appropriate text gets displayed in the operations
     * history table when the operation has no parameters and returns
     * no results.
     */
    protected void checkNoParametersAndNoResults() throws IOException {
        
        // Make sure the operation parameters and results are displayed
        // correctly
        String actualOperationParameters = ((HtmlSpan)client.getElement(NO_PARAMETERS)).getTextContent();
        String actualOperationResults = ((HtmlDivision)client.getElement(NO_RESULTS)).getTextContent();
      
        assertTrue("Incorrect operation parameters displayed in the operations history table: "
                   + "expected: " + NO_PARAMETERS_MESSAGE + " but was: " + actualOperationParameters,
                   actualOperationParameters.contains(NO_PARAMETERS_MESSAGE));
        assertTrue("Incorrect operation results displayed in the operations history table: "
                   + "expected: " + NO_RESULTS_MESSAGE + " but was: " + actualOperationResults,
                   actualOperationResults.contains(NO_RESULTS_MESSAGE));
    }
    
    /**
     * Return table cell value in html format.
     * Used for resource operations that return results in html format.
     */
    protected String formatTableCell(String cellValue) {
        String cellValueAsHtml = TABLE_CELL_DELIM + cellValue + TABLE_CELL_END_DELIM;
        return cellValueAsHtml;
    }
    
    /**
     * Format the expected results of the "List Statistics" operation
     * for datasources and connection factories.
     */
    protected Map<String, String> formatListStatisticsResults(String jndiName, 
                                                              ComponentType componentType,
                                                              String[] specificProps) throws Exception {
				Map<String, MetaValue> componentProps = getSpecificComponentProperties(
								jndiName, componentType, listStatisticsProps, true); // true means JAAS secured.
        Map<String, String> propertiesMap = formatPropertiesMap( componentProps );
        
        // Set up the expected values
        Map<String, String> expectedStatistics = new LinkedHashMap<String, String>();
        Integer idleTimeoutMillis = new Integer(Integer.valueOf(propertiesMap.get("idle-timeout-minutes")) * 60000);
        
        expectedStatistics.put("idleTimeout", idleTimeoutMillis.toString());
        expectedStatistics.put("blockingTimeout", propertiesMap.get("blocking-timeout-millis"));
        expectedStatistics.put("max", propertiesMap.get("max-pool-size"));
        expectedStatistics.put("min", propertiesMap.get("min-pool-size"));
        
        // @TODO: Currently, "noTxnSeparatePool" is spelled incorrectly in the table
        // We are using the incorrect spelling below as a workaround until the typo is fixed
        expectedStatistics.put("noTxnSeperatePool", propertiesMap.get("no-tx-separate-pools")); 
        
        expectedStatistics.put("prefill", propertiesMap.get("prefill"));
        // expectedStatistics.put("trackByTxn", propertiesMap.get("track-connection-by-tx"));  // Deprecated - JBAS-6918
        
        return expectedStatistics;
    }
}
