/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.connfactories;

import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.managed.api.ComponentType;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;
import java.util.Map;
import java.util.LinkedHashMap;


/**
 * This class contains tests for configuring Connection Factories with JBoss AS 5.
 *
 * Note:  Fixed to work with JAAS. Could have been coded more nicely  by providing some TestInfo interface,
 *        but I got that idea when it was already done through "boolean secured"...
 *        So now all methods involved in this test working with ProfileService have this param added.
 *
 * @author Farah Juma
 *
 */
public class ConnFactoryConfigurationTest extends ConnFactoryTest {



    /*
     * CONFIGURATION TESTS
     */

    /**
     * Test Name: testConfigureTxConnectionFactoryChangeProperties
     * Assertion: Verify the ability to change Tx Connection Factory property 
     * values.
     */
    public void testConfigureTxConnectionFactoryChangeProperties() throws Exception {
        String jndiName = "ChangePropertiesTxCF";
        
        Map<String, MetaValue> propertiesMap = createTxConnFactory(jndiName);
        
        // Change some property values that are already set 
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("xa-resource-timeout", SimpleValueSupport.wrap(new Integer(60000)));
        propertiesMapChanges.put("no-tx-separate-pools", SimpleValueSupport.wrap(Boolean.TRUE));

        changeConnFactoryProperties(jndiName, CFType.LOCAL_TX_CF, LOCAL_TX_COMPONENT_TYPE,
                                    propertiesMapChanges, propertiesMap);
    }
    
    /**
     * Test Name: testConfigureTxConnectionFactoryChangePropertiesUsingExistingDSFile
     * Assertion: Verify the ability to change Tx Connection Factory property 
     * values. Use an existing -ds.xml file.
     */
    public void testConfigureTxConnectionFactoryChangePropertiesUsingExistingDSFile() throws Exception {
        String jndiName = "ChangePropertiesExistingTxCF";
        Map<String, MetaValue> propertiesMap = getComponentProperties(jndiName, LOCAL_TX_COMPONENT_TYPE, SECURED);
        
        // Change some property values that are already set 
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("xa-resource-timeout", SimpleValueSupport.wrap(new Integer(60000)));
        propertiesMapChanges.put("prefill", SimpleValueSupport.wrap(Boolean.FALSE));
        
        expandNavTreeArrow(CF_NAV_LABEL);

        changeConnFactoryProperties(jndiName, CFType.LOCAL_TX_CF, LOCAL_TX_COMPONENT_TYPE,
                                    propertiesMapChanges, propertiesMap);
    }
    
    /**
     * Test Name: testConfigureNoTxConnectionFactoryChangeProperties
     * Assertion: Verify the ability to change No Tx Connection Factory property 
     * values.
     */
    public void testConfigureNoTxConnectionFactoryChangeProperties() throws Exception {
        String jndiName = "ChangePropertiesNoTxCF";
        
        Map<String, MetaValue> propertiesMap = createNoTxConnFactoryWithPrefill(jndiName, Boolean.TRUE);
        
        // Change some property values that are already set 
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("max-pool-size", SimpleValueSupport.wrap(new Integer(25)));
        propertiesMapChanges.put("prefill", SimpleValueSupport.wrap(Boolean.FALSE));

        changeConnFactoryProperties(jndiName, CFType.NO_TX_CF, NO_TX_COMPONENT_TYPE,
                                    propertiesMapChanges, propertiesMap);
    }
    
    /**
     * Test Name: testConfigureNoTxConnectionFactoryChangePropertiesUsingExistingDSFile
     * Assertion: Verify the ability to change No Tx Connection Factory property 
     * values. Use an existing -ds.xml file.
     */
    public void testConfigureNoTxConnectionFactoryChangePropertiesUsingExistingDSFile() throws Exception {
        String jndiName = "ChangePropertiesExistingNoTxCF";
        Map<String, MetaValue> propertiesMap = getComponentProperties(jndiName, NO_TX_COMPONENT_TYPE, SECURED);
        
        // Change some property values that are already set 
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("idle-timeout-minutes", SimpleValueSupport.wrap(new Integer(35)));
        propertiesMapChanges.put("prefill", SimpleValueSupport.wrap(Boolean.FALSE));
        
        expandNavTreeArrow(CF_NAV_LABEL);

        changeConnFactoryProperties(jndiName, CFType.NO_TX_CF, NO_TX_COMPONENT_TYPE,
                                    propertiesMapChanges, propertiesMap);
    }
    
    
    /**
     * Common code for changing the property values of a connection factory.
     */
    private void changeConnFactoryProperties(String jndiName,
                                             CFType cfType,
                                             ComponentType componentType,
                                             Map<String, MetaValue> propertiesMapChanges,
                                             Map<String, MetaValue> propertiesMap) throws Exception {
        
        // The success message we should see
        String expectedMessage = UPDATE_MESSAGE + cfType.getName() + " '"+jndiName+"'";
        
        // Change some property values and check that the properties of
        // the ManagedComponent were updated appropriately
        configureResource(CF_NAV_LABEL, cfType.getLabel(), jndiName, 
                          propertiesMap, propertiesMapChanges, componentType,
                          expectedMessage, SECURED);
  
        assertTrue("The connection factory is not deployed ", isDeployed(jndiName + "-ds.xml", SECURED));
        
        // Clean up
        deleteConnFactory(cfType, jndiName); 
    }
    
    /**
     * Test Name: testConfigureTxConnectionFactoryUnsetProperties
     * Assertion: Verify the ability to unset Tx Connection Factory property values. 
     * 
     * Strategy: Create a new Tx Connection Factory. Unset some property values for the 
     * connection factory. Make sure other properties for the ManagedComponent 
     * remain unchanged.
     */
    public void testConfigureTxConnectionFactoryUnsetProperties() throws Exception {
        String jndiName = "UnsetPropertiesTxCF";
        
        // Create the connection factory first
        createTxConnFactory(jndiName);
        
        unsetConnFactoryProperties(CFType.LOCAL_TX_CF, jndiName, LOCAL_TX_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testConfigureTxConnectionFactoryUnsetPropertiesUsingExistingDSFile
     * Assertion: Verify the ability to unset Tx Connection Factory property values. 
     * 
     * Strategy: Unset some property values for an existing Tx Connection Factory. 
     * Make sure other properties for the ManagedComponent remain unchanged.
     */
    public void testConfigureTxConnectionFactoryUnsetPropertiesUsingExistingDSFile() throws Exception {
        String jndiName = "UnsetPropertiesExistingTxCF";
         
        expandNavTreeArrow(CF_NAV_LABEL);
        unsetConnFactoryProperties(CFType.LOCAL_TX_CF, jndiName, LOCAL_TX_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testConfigureNoTxConnectionFactoryUnsetProperties
     * Assertion: Verify the ability to unset No Tx Connection Factory property values. 
     * 
     * Strategy: Create a new No Tx Connection Factory. Unset some property values for the 
     * connection factory. Make sure other properties for the ManagedComponent 
     * remain unchanged.
     */
    public void testConfigureNoTxConnectionFactoryUnsetProperties() throws Exception {
        String jndiName = "UnsetPropertiesNoTxCF";
        
        // Create the connection factory first
        createNoTxConnFactory(jndiName);
        
        unsetConnFactoryProperties(CFType.NO_TX_CF, jndiName, NO_TX_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testConfigureNoTxConnectionFactoryUnsetPropertiesUsingExistingDSFile
     * Assertion: Verify the ability to unset No Tx Connection Factory property values. 
     * 
     * Strategy: Unset some property values for an existing No Tx Connection Factory. 
     * Make sure other properties for the ManagedComponent remain unchanged.
     */
    public void testConfigureNoTxConnectionFactoryUnsetPropertiesUsingExistingDSFile() throws Exception {
        String jndiName = "UnsetPropertiesExistingNoTxCF";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        unsetConnFactoryProperties(CFType.NO_TX_CF, jndiName, NO_TX_COMPONENT_TYPE);
    }
    
    /**
     * Common code for the testConfigure*UnsetProperties* tests.
     */
    private void unsetConnFactoryProperties(CFType cfType,
                                            String jndiName,
                                            ComponentType componentType) throws Exception {
        Map<String, MetaValue> propertiesMap = getSpecificComponentProperties(jndiName, componentType);
        
        String[] propertiesToUnset = new String[] { "min-pool-size" }; 
        propertiesMap.remove("min-pool-size");
        
        // The success message we should see
        String expectedMessage = UPDATE_MESSAGE + cfType.getName() + " '"+jndiName+"'";
        
        unsetResourceProperties(CF_NAV_LABEL, cfType.getLabel(), 
                                jndiName, propertiesMap, propertiesToUnset, 
                                componentType, expectedMessage, SECURED);
        
        assertTrue("The connection factory is not deployed ", isDeployed(jndiName + "-ds.xml", SECURED));
     
        // Clean up
        deleteConnFactory(cfType, jndiName);   
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(ConnFactoryConfigurationTest.class);
    }
}
