/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.connfactories;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.jopr.jsfunit.exceptions.*;
import org.jboss.jopr.jsfunit.as5.ResourceTestBase;
import org.jboss.managed.api.ComponentType;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;
import java.util.Map;
import java.util.HashMap;


/**
 * This class contains tests for creating Connection Factories with JBoss AS 5.
 * 
 * @author Shelly McGowan
 * @author Farah Juma
 *
 */

public class ConnFactoryCreationTest extends ConnFactoryTest {

    /*
     * CREATION TESTS
     */

    /**
     * Test Name: testCreateTxConnectionFactory
     * Assertion: Create a Tx ConnectionFactory.
     */
    public void testCreateTxConnectionFactory() throws Exception {
        
        String jndiName = "CreateTxConnFactory";
        Map<String, MetaValue> propertiesMapAsMetaValues = createConnFactoryPropertiesAsMetaValues(jndiName);
        
        // Add an additional property
        //propertiesMapAsMetaValues.put("track-connection-by-tx", SimpleValueSupport.wrap(Boolean.TRUE));
				//This property supercedes the "track-connection-by-tx" property from earlier versions of JBoss AS.
        propertiesMapAsMetaValues.put("interleaving", SimpleValueSupport.wrap(Boolean.FALSE));
        
        createConnFactoryCommon(jndiName, CFType.LOCAL_TX_CF, 
                                LOCAL_TX_COMPONENT_TYPE, propertiesMapAsMetaValues);
    }
    
    /**
     * Test Name: testCreateNoTxConnectionFactory
     * Assertion: Create a No Tx ConnectionFactory.
     */
    public void testCreateNoTxConnectionFactory() throws Exception {
        
        String jndiName = "CreateNoTxConnFactory";     
        Map<String, MetaValue> propertiesMapAsMetaValues = createConnFactoryPropertiesAsMetaValues(jndiName);
        
        createConnFactoryCommon(jndiName, CFType.NO_TX_CF, 
                                NO_TX_COMPONENT_TYPE, propertiesMapAsMetaValues);
    }
    
    /**
     * Common code for the creation tests.
     */
    private void createConnFactoryCommon(String jndiName, CFType cfType, 
                                         ComponentType componentType,
                                         Map<String, MetaValue> propertiesMapAsMetaValues) throws Exception {
        
        createConnFactory(cfType, propertiesMapAsMetaValues);
        client.click(SAVE_BUTTON);

        // Check for the appropriate success messages
        String expectedMessage = ADD_MESSAGE;
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);

        // Make sure the ManagedComponent was created and that the properties are
        // set correctly
        assertTrue("Connection Factory is not deployed (isDeployed() returned false).",
                   isDeployed(jndiName + "-ds.xml"));

        checkComponentProperties(propertiesMapAsMetaValues, jndiName, componentType);

        // Clean up
        deleteConnFactory(cfType, jndiName);
        
    }
    
    /**
     * Test Name: testCreateConnFactoryMissingRequiredValues
     * Assertion: Verify the ability to handle a missing required value when
     * creating a connection factory. Make sure the appropriate error message occurs.
     */
    public void testCreateConnFactoryMissingRequiredValues() throws IOException, EmbJoprTestException {   
        Map<String, MetaValue> propertiesMap = new HashMap<String, MetaValue>();
        
        // Leave jndi-name and connection-definition unset
        propertiesMap.put("rar-name", SimpleValueSupport.wrap(EMBJOPR_TEST_ADAPTER));
        
        createConnFactory(CFType.NO_TX_CF, propertiesMap);
        client.click(SAVE_BUTTON);
        
        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE,
                                     MISSING_VALUE_MESSAGE,  true);
    }
    
    /**
     * Test Name: testCreateConnFactoryPropertyOutOfRange
     * Assertion: Verify the ability to handle a property value below its
     * expected range of values when creating a connection factory. Make sure the 
     * appropriate error message occurs.
     */
    public void testCreateConnFactoryPropertyOutOfRange() throws IOException, EmbJoprTestException {
        String jndiName = "PropertyOutOfRangeCF";

        Map<String, MetaValue> propertiesMap = new HashMap<String, MetaValue>();
        
        propertiesMap.put("jndi-name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("rar-name", SimpleValueSupport.wrap(EMBJOPR_TEST_ADAPTER));
        propertiesMap.put("connection-definition", SimpleValueSupport.wrap(TEST_CONNECTION_DEFINITION));
        propertiesMap.put("min-pool-size", SimpleValueSupport.wrap(new Integer(5)));
        
        // This number is too small
        propertiesMap.put("max-pool-size", SimpleValueSupport.wrap(new Integer(-5)));
        
        createConnFactory(CFType.LOCAL_TX_CF, propertiesMap);
        client.click(SAVE_BUTTON);

        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE,
                                     BELOW_ALLOWABLE_MINIMUM,
                                     true);
    }
    
    /**
     * Test Name: testCreateConnFactoryInvalidPropertyType
     * Assertion: Verify the ability to handle a property value that has an
     * invalid type when creating a connection factory. Make sure the appropriate 
     * error message occurs.
     */
    public void testCreateConnFactoryInvalidPropertyType() throws IOException, EmbJoprTestException {
        String jndiName = "InvalidPropertyCF";
        
        // The properties we want to configure
        Map<String, MetaValue> propertiesMap = new HashMap<String, MetaValue>();
        
        propertiesMap.put("jndi-name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("rar-name", SimpleValueSupport.wrap(EMBJOPR_TEST_ADAPTER));
        propertiesMap.put("connection-definition", SimpleValueSupport.wrap(TEST_CONNECTION_DEFINITION));
        
        // This property should be an integer
        propertiesMap.put("min-pool-size", SimpleValueSupport.wrap("abcde"));
        
        createConnFactory(CFType.LOCAL_TX_CF, propertiesMap);
        client.click(SAVE_BUTTON);
        
        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE,
                                     INVALID_INTEGER_MESSAGE, true);
    }
    
    /**
     * Test Name: testCreateConnFactoryDuplicateJNDIName
     * Assertion: Verify the ability to handle a duplicate JNDI name 
     * when creating a connection factory. Make sure the appropriate error message 
     * occurs.
     */
    public void testCreateConnFactoryDuplicateJNDIName() throws IOException, EmbJoprTestException {
        
        // A connection factory with this JNDI name already exists
        String jndiName = "TestCF";
            
        // The properties we want to configure
        Map<String, MetaValue> propertiesMap = new HashMap<String, MetaValue>();

        propertiesMap.put("jndi-name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("rar-name", SimpleValueSupport.wrap(EMBJOPR_TEST_ADAPTER));
        propertiesMap.put("connection-definition", SimpleValueSupport.wrap(TEST_CONNECTION_DEFINITION));

        createConnFactory(CFType.LOCAL_TX_CF, propertiesMap);
        client.click(SAVE_BUTTON);
        
        String expectedMessage = "A " + CFType.LOCAL_TX_CF.getName() + " named '" 
                                 + jndiName + "' already exists";
        checkClientAndServerMessages(expectedMessage, expectedMessage, true);
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(ConnFactoryCreationTest.class);
    }
}
