/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.connfactories;

import com.gargoylesoftware.htmlunit.html.*;
import org.jboss.jopr.jsfunit.*;
import java.io.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.jopr.jsfunit.exceptions.*;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.*;
import org.jboss.jopr.jsfunit.as5.ResourceTestBase;
import org.jboss.managed.api.ComponentType;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;


/**
 * This class contains tests for deleting Connection Factories with JBoss AS 5.
 * 
 * @author Shelly McGowan
 * @author Farah Juma
 *
 */

public class ConnFactoryDeletionTest extends ConnFactoryTest {

    /*
     * DELETE TESTS
     */

     /**
      * Test Name: testDeleteTxConnectionFactory
      * Assertion: Verify the ability to delete a TxConnectionFactory. 
      * 
      * Strategy: Create a Tx ConnectionFactory and then delete it. The resource adaptor
      * should remain deployed.  
      */
     public void testDeleteTxConnectionFactory() throws Exception {
         String jndiName = "DeleteTxConnFactory";

         createTxConnFactory(jndiName);
         deleteConnFactoryCommon(jndiName, CFType.LOCAL_TX_CF, LOCAL_TX_COMPONENT_TYPE, 
                                 EMBJOPR_TEST_ADAPTER);
     }
     
     /**
      * Test Name: testDeleteTxConnectionFactoryUsingExistingDSFile
      * Assertion: Verify the ability to delete a TxConnectionFactory. 
      * 
      * Strategy: Delete a Tx ConnectionFactory that already exists. 
      * The resource adaptor should remain deployed. 
      */
     public void testDeleteTxConnectionFactoryUsingExistingDSFile() throws Exception {
         String jndiName = "DeleteTxCFExistingFile";

         expandNavTreeArrow(CF_NAV_LABEL);
         deleteConnFactoryCommon(jndiName, CFType.LOCAL_TX_CF, LOCAL_TX_COMPONENT_TYPE, 
                                 EMBJOPR_TEST_ADAPTER);  
     }
     
     /**
      * Test Name: testDeleteNoTxConnectionFactory
      * Assertion: Verify the ability to delete a No Tx ConnectionFactory. 
      * 
      * Strategy: Create a No Tx ConnectionFactory and then delete it. The resource adaptor
      * should remain deployed.  
      */
     public void testDeleteNoTxConnectionFactory() throws Exception {
         String jndiName = "DeleteNoTxConnFactory";

         createNoTxConnFactory(jndiName);
         deleteConnFactoryCommon(jndiName, CFType.NO_TX_CF, NO_TX_COMPONENT_TYPE, 
                                 EMBJOPR_TEST_ADAPTER);
     }
     
     /**
      * Test Name: testDeleteNoTxConnectionFactoryUsingExistingDSFile
      * Assertion: Verify the ability to delete a No Tx ConnectionFactory. 
      * 
      * Strategy: Delete a No Tx ConnectionFactory that already exists. 
      * The resource adaptor should remain deployed.
      */
     public void testDeleteNoTxConnectionFactoryUsingExistingDSFile() throws Exception {
         String jndiName = "DeleteNoTxCFExistingFile";

         expandNavTreeArrow(CF_NAV_LABEL);
         deleteConnFactoryCommon(jndiName, CFType.NO_TX_CF, NO_TX_COMPONENT_TYPE, 
                                 EMBJOPR_TEST_ADAPTER);  
     }
     
     /**
      * Common code for the testDelete* tests.
      */
     public void deleteConnFactoryCommon(String jndiName, 
                                         CFType cfType, 
                                         ComponentType componentType,
                                         String resourceAdaptorName) throws Exception {
         deleteConnFactory(cfType, jndiName);
         
         // Check for the appropriate success messages
         String expectedMessage = DELETE_MESSAGE + cfType.getName() + " '" + jndiName + "'";
         checkClientAndServerMessages(expectedMessage, expectedMessage, false);
         
         // Make sure the resource adaptor is still deployed
         assertTrue("The resource adaptor is no longer deployed (isDeployed() returned false).",
                    isDeployed(resourceAdaptorName));
         
         // Make sure the ManagedComponent was removed
         assertNull("The connection factory was not removed after deletion (the ManagedComponent is non-null)",
                    getManagedComponent(jndiName, componentType));
         
         // The entry should still be found in the -ds.xml file
         assertTrue("The connection factory was removed from the -ds.xml file",
                    containsElement(getConnFactoryConfigFile(jndiName),
                                    cfType.getXmlElementName(),
                                    CF_ROOT_ELEMENT));
     }

     /**
      * @return the suite of tests being tested
      */
     public static Test suite() {
         return new TestSuite(ConnFactoryDeletionTest.class);
     }
}
