/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.connfactories;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.jopr.jsfunit.exceptions.*;
import org.jboss.jopr.jsfunit.as5.ResourceTestBase;
import java.util.ArrayList;
import java.util.Map;
import java.util.LinkedHashMap;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.Connection;

/**
 * This class contains metrics tests for Connection Factories.
 * This test class should be run against JBoss AS 5.
 * 
 * @author Farah Juma
 */

public class ConnFactoryMetricsTest extends ConnFactoryTest {
   
    /**
     * Test Name: testTxConnectionFactoryMetricsAfterCreation
     * Assertion: Verify that the metrics are correct after creating a new
     * Tx Connection Factory. 
     */
    public void testMetricsAfterTxConnectionFactoryCreation() throws Exception {
        String jndiName = "MetricsCreateTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);

        checkMetricsAfterConnectionFactoryCreation(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryMetricsAfterCreation
     * Assertion: Verify that the metrics are correct after creating a new
     * No Tx Connection Factory. 
     */
    public void testMetricsAfterNoTxConnectionFactoryCreation() throws Exception {
        String jndiName = "MetricsCreateNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);

        checkMetricsAfterConnectionFactoryCreation(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Common code for the creation tests.
     */
    protected void checkMetricsAfterConnectionFactoryCreation(String jndiName, CFType cfType) throws Exception {
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "20");
        expectedMetrics.put(CONNECTION_COUNT, "0");
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "0");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "0");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "0");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5"); 

        checkConnFactoryMetrics(jndiName, expectedMetrics, cfType, null);
    }
    
    /**
     * Test Name: testTxConnectionFactoryMetricsAfterInitialConnection
     * Assertion: Verify that the metrics for Tx Connection Factories 
     * are correct after requesting a connection.
     */
    public void testTxConnectionFactoryMetricsAfterInitialConnection() throws Exception {
        String jndiName = "MetricsInitialConnectionTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactoryWithPrefill(jndiName, Boolean.FALSE);;
        
        checkMetricsAfterInitialConnection(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testTxConnectionFactoryMetricsAfterInitialConnectionUsingExistingDSFile
     * Assertion: Verify that the metrics for Tx Connection Factories are correct after 
     * requesting a connection. Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryMetricsAfterInitialConnectionUsingExistingDSFile() throws Exception {
        
        // Use an existing connection factory
        String jndiName = "MetricsTxCFInitialConnectionExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterInitialConnection(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryMetricsAfterInitialConnection
     * Assertion: Verify that the metrics for No Tx Connection Factories 
     * are correct after requesting a connection.
     */
    public void testNoTxConnectionFactoryMetricsAfterInitialConnection() throws Exception {
        String jndiName = "MetricsInitialConnectionNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactoryWithPrefill(jndiName, Boolean.FALSE);;
        
        checkMetricsAfterInitialConnection(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryMetricsAfterInitialConnectionUsingExistingDSFile
     * Assertion: Verify that the metrics for No Tx Connection Factories are correct after 
     * requesting a connection. Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryMetricsAfterInitialConnectionUsingExistingDSFile() throws Exception {
        
        // Use an existing connection factory
        String jndiName = "MetricsNoTxCFInitialConnectionExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterInitialConnection(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Common code for the test*ConnectionFactoryMetricsAfterInitialConnection* tests.
     */
    private void checkMetricsAfterInitialConnection(String jndiName, CFType cfType) throws Exception {

        // Create the first connection
        ArrayList<Connection> connections = createConnections(1, jndiName);

        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "19");
        expectedMetrics.put(CONNECTION_COUNT, "5");
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "5");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "1");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "1");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5"); 

        checkConnFactoryMetrics(jndiName, expectedMetrics, cfType, connections);
    }
    
    /**
     * Test Name: testTxConnectionFactoryMetricsAfterMultipleConnections
     * Assertion: Verify that the metrics for Tx Connection Factories are correct 
     * after requesting multiple connections.
     */
    public void testTxConnectionFactoryMetricsAfterMultipleConnections() throws Exception {
        String jndiName = "MetricsMultipleConnectionsTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);
        
        checkMetricsAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testTxConnectionFactoryMetricsAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the metrics for Tx Connection Factories are correct after 
     * requesting multiple connections. Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryMetricsAfterMutlipleConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing connection factory
        String jndiName = "MetricsTxCFMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryMetricsAfterMultipleConnections
     * Assertion: Verify that the metrics for No Tx Connection Factories are correct 
     * after requesting multiple connections.
     */
    public void testNoTxConnectionFactoryMetricsAfterMultipleConnections() throws Exception {
        String jndiName = "MetricsMultipleConnectionsNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);
        
        checkMetricsAfterMultipleConnections(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryMetricsAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the metrics for No Tx Connection Factories are correct after 
     * requesting multiple connections. Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryMetricsAfterMutlipleConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing connection factory
        String jndiName = "MetricsNoTxCFMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterMultipleConnections(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Common code for the test*ConnectionFactoryMetricsAfterMultipleConnections* tests.
     */
    private void checkMetricsAfterMultipleConnections(String jndiName, CFType cfType) throws Exception {

        // Create multiple connections
        ArrayList<Connection> connections = createConnections(6, jndiName);

        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "14");
        expectedMetrics.put(CONNECTION_COUNT, "6");
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "6");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "6");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "6");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5");

        checkConnFactoryMetrics(jndiName, expectedMetrics, cfType, connections);
    }
    
    /**
     * Test Name: testTxConnectionFactoryMetricsAfterClosingConnections
     * Assertion: Verify that the metrics for Tx Connection Factories are 
     * correct after requesting multiple connections and then closing some of them.
     */
    public void testTxConnectionFactoryMetricsAfterClosingConnections() throws Exception {
        String jndiName = "MetricsCloseConnectionsTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);
        
        checkMetricsAfterClosingConnections(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testTxConnectionFactoryMetricsAfterClosingConnectionsUsingExistingDSFile
     * Assertion: Verify that the metrics for Tx Connection Factories are correct after 
     * requesting multiple connections and then closing some of them. 
     * Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryMetricsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing connection factory
        String jndiName = "MetricsTxCFClosingConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterClosingConnections(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryMetricsAfterClosingConnections
     * Assertion: Verify that the metrics for No Tx Connection Factories are 
     * correct after requesting multiple connections and then closing some of them.
     */
    public void testNoTxConnectionFactoryMetricsAfterClosingConnections() throws Exception {
        String jndiName = "MetricsCloseConnectionsNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);
        
        checkMetricsAfterClosingConnections(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryMetricsAfterClosingConnectionsUsingExistingDSFile
     * Assertion: Verify that the metrics for No Tx Connection Factories are correct after 
     * requesting multiple connections and then closing some of them. 
     * Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryMetricsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing connection factory
        String jndiName = "MetricsNoTxCFClosingConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterClosingConnections(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Common code for the test*ConnectionFactoryMetricsAfterMultipleConnections* tests.
     */
    private void checkMetricsAfterClosingConnections(String jndiName, CFType cfType) throws Exception {

        // Create multiple connections
        ArrayList<Connection> connections = createConnections(10, jndiName);
        
        // Close some connections
        for(int i = 0; i < 2; i++) {
            connections.get(i).close();
            connections.set(i, null);
        }

        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "12"); 
        expectedMetrics.put(CONNECTION_COUNT, "10");          
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "10");    
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");   
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "8");     
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "10");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5");
        
        checkConnFactoryMetrics(jndiName, expectedMetrics, cfType, connections);
    }
      
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(ConnFactoryMetricsTest.class);
    }
}
