/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.connfactories;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.jopr.jsfunit.exceptions.*;
import org.jboss.jopr.jsfunit.as5.ResourceTestBase;
import java.util.ArrayList;
import java.util.Map;
import java.util.LinkedHashMap;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.Connection;
import org.jboss.metatype.api.values.MetaValue;
import org.jboss.managed.api.ComponentType;

/**
 * This class contains operations tests for Connection Factories.
 * This test class should be run against JBoss AS 5.
 * 
 * @author Farah Juma
 */

public class ConnFactoryOperationsTest extends ConnFactoryTest {
   
    /* 
     * OPERATION #1 - FLUSH
     * Applies to both Tx and No Tx Connection Factories.
     */
    
    /**
     * Test Name: testTxConnectionFactoryFlushOperationAfterCreation
     * Assertion: Verify that the metrics for Tx Connection Factories 
     * are correct after executing the "Flush" operation on a newly created 
     * connection factory.
     */
    public void testTxConnectionFactoryFlushOperationAfterCreation() throws Exception { 
        String jndiName = "FlushAfterTxCFCreation";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);

        performFlushAfterCreation(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryFlushOperationAfterCreation
     * Assertion: Verify that the metrics for No Tx Connection Factories 
     * are correct after executing the "Flush" operation on a newly created 
     * connection factory.
     */
    public void testNoTxConnectionFactoryFlushOperationAfterCreation() throws Exception { 
        String jndiName = "FlushAfterNoTxCFCreation";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);

        performFlushAfterCreation(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Common code for the test*ConnectionFactoryFlushOperationAfterCreation tests.
     */
    private void performFlushAfterCreation(String jndiName, 
                                           CFType cfType) throws Exception {
        
        // Make sure the metrics are updated appropriately
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "20");
        expectedMetrics.put(CONNECTION_COUNT, "0"); 
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "0");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "0");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "0");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5");

        performConnFactoryOperationAndCheckMetrics(cfType, jndiName, FLUSH, expectedMetrics,
                                                   null);
        
    }
    
    /**
     * Test Name: testTxConnectionFactoryFlushOperationAfterMultipleConnections
     * Assertion: Verify that the metrics for Tx Connection Factories are correct 
     * after executing the "Flush" operation when there are multiple connections.
     */
    public void testTxConnectionFactoryFlushOperationAfterMultipleConnections() throws Exception { 
        String jndiName = "FlushAfterMultipleConnectionsTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);

        performFlushAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF);   
    }
    
    /**
     * Test Name: testTxConnectionFactoryFlushOperationAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the metrics for Tx Connection Factories are 
     * correct after executing the "Flush" operation when there are multiple 
     * connections. Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryFlushOperationAfterMultipleConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "FlushTxCFAfterMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performFlushAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryFlushOperationAfterMultipleConnections
     * Assertion: Verify that the metrics for No Tx Connection Factories are correct 
     * after executing the "Flush" operation when there are multiple connections.
     */
    public void testNoTxConnectionFactoryFlushOperationAfterMultipleConnections() throws Exception { 
        String jndiName = "FlushAfterMultipleConnectionsNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);

        performFlushAfterMultipleConnections(jndiName, CFType.NO_TX_CF);   
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryFlushOperationAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the metrics for No Tx Connection Factories are 
     * correct after executing the "Flush" operation when there are multiple 
     * connections. Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryFlushOperationAfterMultipleConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "FlushNoTxCFAfterMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performFlushAfterMultipleConnections(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Common code for the testConnectionFactoryFlushOperationAfterMultipleConnections* tests.
     */
    private void performFlushAfterMultipleConnections(String jndiName, CFType cfType) throws Exception {
        
        // Create some connections and then close them. This will
        // result in non-zero values for "Connection Created Count" and
        // "Max Connections In Use Count".
        ArrayList<Connection> connections = createConnections(5, jndiName); 
        closeConnections(connections); 
        
        // Make sure the metrics are updated appropriately
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "20");
        expectedMetrics.put(CONNECTION_COUNT, "0"); 
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "0");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "0");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "0");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5");   

        performConnFactoryOperationAndCheckMetrics(cfType, jndiName, FLUSH, expectedMetrics,
                                                   connections);
    }
    
    /* 
     * OPERATION #2 - LIST FORMATTED SUB POOL STATISTICS
     * Applies to both Tx and No Tx Connection Factories.
     */
    
    /**
     * Test Name: testTxConnectionFactoryListFormattedSubPoolStatisticsAfterCreation
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation on a newly
     * created Tx Connection Factory.
     */
    public void testTxConnectionFactoryListFormattedSubPoolStatisticsAfterCreation() throws Exception {
        String jndiName = "ListFormattedSubPoolStatisticsAfterTxCFCreation";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);

        performListFormattedSubPoolStatisticsAfterCreation(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterCreation
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation on a newly
     * created No Tx Connection Factory.
     */
    public void testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterCreation() throws Exception {
        String jndiName = "ListFormattedSubPoolStatisticsAfterNoTxCFCreation";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);

        performListFormattedSubPoolStatisticsAfterCreation(jndiName, CFType.NO_TX_CF);
    }
    
    /**
     * Common code for the 
     * test*ConnectionFactoryListFormattedSubPoolStatisticsAfterCreation tests.
     */
    private void performListFormattedSubPoolStatisticsAfterCreation(String jndiName,
                                                                    CFType cfType) throws Exception {
        // Set up the expected results
        StringBuffer expectedBuffer = new StringBuffer();
        expectedBuffer.append("Sub Pool Statistics: \n");
        expectedBuffer.append("Sub Pool Count: 0\n");
        expectedBuffer.append(POOL_SEPARATOR);
        expectedBuffer.append("\n\n");
        
        String errorMessage = "Incorrect sub pool statistics";

        performResourceOperationAndCheckValueBox(CF_NAV_LABEL, cfType.getLabel(),
                                                 jndiName, LIST_FORMATTED_SUB_POOL_STATISTICS,
                                                 Boolean.TRUE, expectedBuffer, errorMessage);
        
        // Clean up
        deleteConnFactory(cfType, jndiName);
    }
    
    /**
     * Test Name: testTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a Tx Connection
     * Factory when there are multiple connections.
     */
    public void testTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnections() throws Exception {
        String jndiName = "ListFormattedSubPoolStatisticsAfterMultipleConnectionsTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);

        performListFormattedSubPoolStatisticsAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF);   
    }
    
    /**
     * Test Name: testTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a Tx Connection 
     * Factory when there are multiple connections. Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnectionsUsingExistingDSFile() 
        throws Exception {
        String jndiName = "ListFormattedTxCFAfterMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListFormattedSubPoolStatisticsAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a No Tx Connection
     * Factory when there are multiple connections.
     */
    public void testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnections() throws Exception {
        String jndiName = "ListFormattedSubPoolStatisticsAfterMultipleConnectionsNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);

        performListFormattedSubPoolStatisticsAfterMultipleConnections(jndiName, CFType.NO_TX_CF);   
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a No Tx Connection 
     * Factory when there are multiple connections. Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnectionsUsingExistingDSFile() 
        throws Exception {
        String jndiName = "ListFormattedNoTxCFAfterMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListFormattedSubPoolStatisticsAfterMultipleConnections(jndiName, CFType.NO_TX_CF);
    }

    /**
     * Common code for the 
     * testConnectionFactoryListFormattedSubPoolStatisticsAfterMultipleConnections* tests.
     */
    private void performListFormattedSubPoolStatisticsAfterMultipleConnections(String jndiName,
                                                                               CFType cfType) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(6, jndiName); 
        
        try { 
            
            // Set up the expected results
            StringBuffer expectedBuffer = new StringBuffer();
            expectedBuffer.append("Sub Pool Statistics: \n");
            expectedBuffer.append("Sub Pool Count: 1\n");
            expectedBuffer.append(POOL_SEPARATOR);
            expectedBuffer.append("\n\n");

						// Deprecated property, was removed - JBAS-6918
            /*if(cfType == CFType.NO_TX_CF) {
                expectedBuffer.append("Track By Transaction: false\n");
            } else {
                expectedBuffer.append("Track By Transaction: true\n");
            }*/
            expectedBuffer.append("Available Connections Count: 14\n");
            expectedBuffer.append("Max Connections In Use Count:6\n");
            expectedBuffer.append("Connections Destroyed Count:0\n");
            expectedBuffer.append("Connections In Use Count:6\n");
            
            String errorMessage = "Incorrect sub pool statistics";

            performResourceOperationAndCheckValueBox(CF_NAV_LABEL, cfType.getLabel(),
                                                     jndiName, LIST_FORMATTED_SUB_POOL_STATISTICS,
                                                     Boolean.TRUE, expectedBuffer, errorMessage);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteConnFactory(cfType, jndiName);
        }
    }
    
    /**
     * Test Name: testTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a Tx Connection 
     * Factory after closing connections.
     */
    public void testTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnections() throws Exception {
        String jndiName = "ListFormattedSubPoolStatisticsAfterClosingConnectionsTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);

        performListFormattedSubPoolStatisticsAfterClosingConnections(jndiName, CFType.LOCAL_TX_CF); 
    }
    
    /**
     * Test Name: testTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a Tx Connection Factory 
     * after closing connections. Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "ListFormattedTxCFAfterClosingConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListFormattedSubPoolStatisticsAfterClosingConnections(jndiName, CFType.LOCAL_TX_CF); 
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a No Tx Connection 
     * Factory after closing connections.
     */
    public void testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnections() throws Exception {
        String jndiName = "ListFormattedSubPoolStatisticsAfterClosingConnectionsNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);

        performListFormattedSubPoolStatisticsAfterClosingConnections(jndiName, CFType.NO_TX_CF); 
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Formatted Sub Pool Statistics" operation for a No Tx Connection Factory 
     * after closing connections. Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnectionsUsingExistingDSFile() 
        throws Exception {
        
        String jndiName = "ListFormattedNoTxCFAfterClosingConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListFormattedSubPoolStatisticsAfterClosingConnections(jndiName, CFType.NO_TX_CF); 
    }
    
    /**
     * Common code for the 
     * testConnectionFactoryListFormattedSubPoolStatisticsAfterClosingConnections* tests.
     */
    private void performListFormattedSubPoolStatisticsAfterClosingConnections(String jndiName,
                                                                              CFType cfType) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(10, jndiName);
        
        try {
            
            // Close some connections
            for(int i = 0; i < 2; i++) {
                connections.get(i).close();
                connections.set(i, null);
            }
            
            // Set up the expected results
            StringBuffer expectedBuffer = new StringBuffer();
            expectedBuffer.append("Sub Pool Statistics: \n");
            expectedBuffer.append("Sub Pool Count: 1\n");
            expectedBuffer.append(POOL_SEPARATOR);
            expectedBuffer.append("\n\n");

						// Deprecated property, was removed - JBAS-6918
            /*if(cfType == CFType.NO_TX_CF) {
                expectedBuffer.append("Track By Transaction: false\n");
            } else {
                expectedBuffer.append("Track By Transaction: true\n");
            }*/
            
            expectedBuffer.append("Available Connections Count: 12\n");
            expectedBuffer.append("Max Connections In Use Count:10\n");
            expectedBuffer.append("Connections Destroyed Count:0\n");
            expectedBuffer.append("Connections In Use Count:8");
        
            String errorMessage = "Incorrect sub pool statistics";

            performResourceOperationAndCheckValueBox(CF_NAV_LABEL, cfType.getLabel(),
                                                     jndiName, LIST_FORMATTED_SUB_POOL_STATISTICS,
                                                     Boolean.TRUE, expectedBuffer, errorMessage);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteConnFactory(cfType, jndiName);
        }
    }
    
    /* 
     * OPERATION #3 - LIST STATISTICS
     * Applies to Tx and No Tx Connection Factories.
     */
     
    /**
     * Test Name: testTxConnectionFactoryListStatisticsAfterCreation
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation on a newly created Tx Connection Factory.
     */
    public void testTxConnectionFactoryListStatisticsAfterCreation() throws Exception {
        String jndiName = "ListStatisticsTxCFAfterCreation";
        
        createTxConnFactory(jndiName);
        
        performListStatisticsAfterCreation(jndiName, CFType.LOCAL_TX_CF, LOCAL_TX_COMPONENT_TYPE);
    } 
    
    /**
     * Test Name: testNoTxConnectionFactoryListStatisticsAfterCreation
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation on a newly created No Tx Connection Factory.
     */
    public void testNoTxConnectionFactoryListStatisticsAfterCreation() throws Exception {
        String jndiName = "ListStatisticsNoTxCFAfterCreation";
        
        createNoTxConnFactory(jndiName);
        
        performListStatisticsAfterCreation(jndiName, CFType.NO_TX_CF, NO_TX_COMPONENT_TYPE);
    } 
    
    /**
     * Common code for the test*ConnectionFactoryListStatisticsAfterCreation tests.
     */
    private void performListStatisticsAfterCreation(String jndiName, 
                                                    CFType cfType, 
                                                    ComponentType componentType) throws Exception {
        // Set up the expectedStatistics
        Map<String, String> expectedStatistics = formatListStatisticsResults(jndiName, 
                                                                             componentType, 
                                                                             listStatisticsProps);
        
        // Additional statistics that need to be checked
        expectedStatistics.put("criteria", "ByApplication");
        expectedStatistics.put("name", "jboss.jca:service=ManagedConnectionFactory,name=" 
                               + jndiName);
        expectedStatistics.put("subPoolCount", "0"); 
        expectedStatistics.put("totalConnectionsInUseCount", "0");
        expectedStatistics.put("totalMaxConnectionsInUseCount", "0");
        
        performResourceOperationAndCheckTable(CF_NAV_LABEL, cfType.getLabel(), jndiName,
                                              LIST_STATISTICS, Boolean.FALSE, expectedStatistics);
        
        deleteConnFactory(cfType, jndiName);
    }
    
    
    /**
     * Test Name: testTxConnectionFactoryListStatisticsAfterMultipleConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a Tx Connection Factory when 
     * there are multiple connections.
     */
    public void testTxConnectionFactoryListStatisticsAfterMultipleConnections() throws Exception {
        String jndiName = "ListStatisticsAfterMultipleConnectionsTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);
        performListStatisticsAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF, 
                                                      LOCAL_TX_COMPONENT_TYPE);   
    } 
    
    /**
     * Test Name: testTxConnectionFactoryListStatisticsAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a Tx Connection Factory when there 
     * are multiple connections. Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryListStatisticsAfterMultipleConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "ListStatisticsTxCFAfterMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListStatisticsAfterMultipleConnections(jndiName, CFType.LOCAL_TX_CF, 
                                                      LOCAL_TX_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryListStatisticsAfterMultipleConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a No Tx Connection Factory when 
     * there are multiple connections.
     */
    public void testNoTxConnectionFactoryListStatisticsAfterMultipleConnections() throws Exception {
        String jndiName = "ListStatisticsAfterMultipleConnectionsNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);
        performListStatisticsAfterMultipleConnections(jndiName, CFType.NO_TX_CF, 
                                                      NO_TX_COMPONENT_TYPE);   
    } 
    
    /**
     * Test Name: testNoTxConnectionFactoryListStatisticsAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a No Tx Connection Factory when there 
     * are multiple connections. Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryListStatisticsAfterMultipleConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "ListStatisticsNoTxCFAfterMultipleConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListStatisticsAfterMultipleConnections(jndiName, CFType.NO_TX_CF, 
                                                      NO_TX_COMPONENT_TYPE);
    }

    /**
     * Common code for the
     * testConnectionFactoryListStatisticsAfterMultipleConnections* tests.
     */
    private void performListStatisticsAfterMultipleConnections(String jndiName,
                                                               CFType cfType,
                                                               ComponentType componentType) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(6, jndiName); 
        
        try { 
            
            // Set up the expectedStatistics
            Map<String, String> expectedStatistics = formatListStatisticsResults(jndiName, 
                                                                                 componentType, 
                                                                                 listStatisticsProps);
            
            // Additional statistics that need to be checked
            expectedStatistics.put("criteria", "ByApplication");
            expectedStatistics.put("name", "jboss.jca:service=ManagedConnectionFactory,name=" 
                                   + jndiName);
            expectedStatistics.put("subPoolCount", "1"); 
            expectedStatistics.put("totalConnectionsInUseCount", "6");
            expectedStatistics.put("totalMaxConnectionsInUseCount", "6");
              
            performResourceOperationAndCheckTable(CF_NAV_LABEL, cfType.getLabel(), jndiName,
                                                  LIST_STATISTICS, Boolean.FALSE, expectedStatistics);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteConnFactory(cfType, jndiName);
        }
    }
    
    /**
     * Test Name: testTxConnectionFactoryListStatisticsAfterClosingConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a Tx Connection Factory after 
     * requesting multiple connections and then closing some of them.
     */
    public void testTxConnectionFactoryListStatisticsAfterClosingConnections() throws Exception {
        String jndiName = "ListStatisticsAfterClosingConnectionsTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createTxConnFactory(jndiName);

        performListStatisticsAfterClosingConnections(jndiName, CFType.LOCAL_TX_CF, 
                                                     LOCAL_TX_COMPONENT_TYPE); 
    } 
    
    /**
     * Test Name: testTxConnectionFactoryListStatisticsAfterClosingConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a Tx Connection Factory after requesting 
     * multiple connections and then closing some of them. Use an existing -ds.xml file.
     */
    public void testTxConnectionFactoryListStatisticsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "ListStatisticsTxCFAfterClosingConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListStatisticsAfterClosingConnections(jndiName, CFType.LOCAL_TX_CF, 
                                                     LOCAL_TX_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testNoTxConnectionFactoryListStatisticsAfterClosingConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a No Tx Connection Factory after 
     * requesting multiple connections and then closing some of them.
     */
    public void testNoTxConnectionFactoryListStatisticsAfterClosingConnections() throws Exception {
        String jndiName = "ListStatisticsAfterClosingConnectionsNoTxCF";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTxConnFactory(jndiName);

        performListStatisticsAfterClosingConnections(jndiName, CFType.NO_TX_CF, 
                                                     NO_TX_COMPONENT_TYPE); 
    } 
    
    /**
     * Test Name: testNoTxConnectionFactoryListStatisticsAfterClosingConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a No Tx Connection Factory after requesting 
     * multiple connections and then closing some of them. Use an existing -ds.xml file.
     */
    public void testNoTxConnectionFactoryListStatisticsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "ListStatisticsNoTxCFAfterClosingConnectionsExistingFile";
        
        expandNavTreeArrow(CF_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        performListStatisticsAfterClosingConnections(jndiName, CFType.NO_TX_CF, 
                                                     NO_TX_COMPONENT_TYPE);
    }

    /**
     * Common code for the
     * testConnectionFactoryListStatisticsAfterClosingConnections* tests.
     */
    private void performListStatisticsAfterClosingConnections(String jndiName,
                                                              CFType cfType,
                                                              ComponentType componentType) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(10, jndiName); 
        
        try { 
            
            // Close some connections
            for(int i = 0; i < 2; i++) {
                connections.get(i).close();
                connections.set(i, null);
            }
            
            // Set up the expectedStatistics
            Map<String, String> expectedStatistics = formatListStatisticsResults(jndiName, 
                                                                                 componentType, 
                                                                                 listStatisticsProps);
            
            // Additional statistics that need to be checked
            expectedStatistics.put("criteria", "ByApplication");
            expectedStatistics.put("name", "jboss.jca:service=ManagedConnectionFactory,name=" 
                                   + jndiName);
            expectedStatistics.put("subPoolCount", "1"); 
            expectedStatistics.put("totalConnectionsInUseCount", "8");
            expectedStatistics.put("totalMaxConnectionsInUseCount", "10");
            
            performResourceOperationAndCheckTable(CF_NAV_LABEL, cfType.getLabel(), jndiName,
                                                  LIST_STATISTICS, Boolean.FALSE, expectedStatistics);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteConnFactory(cfType, jndiName);
        }
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(ConnFactoryOperationsTest.class);
    }
}
