/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.connfactories;

import org.jboss.jopr.jsfunit.exceptions.*;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.*;
import org.jboss.jopr.jsfunit.as5.ResourceTestBase;
import org.jboss.deployers.spi.management.KnownComponentTypes;
import org.jboss.managed.api.ComponentType;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.io.IOException;
import javax.resource.cci.ConnectionFactory;
import javax.resource.cci.Connection;
import javax.resource.ResourceException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * This class is the base test class for configuring Connection Factories 
 * with JBoss AS 5.
 */

/**
 * 
 * @author Shelly McGowan
 * @author Farah Juma
 *
 */
public abstract class ConnFactoryTest extends ResourceTestBase {

		protected static final boolean SECURED = true;
		protected static final boolean UNSECURED = false;
		

		public enum CFType {

        NO_TX_CF("No Tx ConnectionFactories", NO_TXCF_DEFAULT_TEMPLATE, 
                 "No Tx ConnectionFactory", "no-tx-connection-factory"),
        LOCAL_TX_CF("Tx ConnectionFactories", TXCF_LOCAL_TEMPLATE, 
                    "Tx ConnectionFactory", "tx-connection-factory"),
        XA_TX_CF("Tx ConnectionFactories", TXCF_XA_TEMPLATE, 
                 "Tx ConnectionFactory", "tx-connection-factory"),
        DEFAULT_TX_CF("Tx ConnectionFactories", TXCF_DEFAULT_TEMPLATE, 
                      "Tx ConnectionFactory", "tx-connection-factory");

        public final String label;
        public final String templateName;
        public final String name;
        public final String xmlElementName;

        CFType(String label, String templateName, String name, String xmlElementName)
        {
            this.label = label;
            this.templateName = templateName;
            this.name = name;
            this.xmlElementName = xmlElementName;
        }

        public String getLabel()
        {
            return this.label;
        }

        public String getTemplateName()
        {
            return this.templateName;
        }
        
        public String getName()
        {
            return this.name;
        }
        
        public String getXmlElementName()
        {
            return this.xmlElementName;
        }

    }

    // Component Types 
    public static final ComponentType LOCAL_TX_COMPONENT_TYPE = KnownComponentTypes.ConnectionFactoryTypes.Tx.getType();
    public static final ComponentType NO_TX_COMPONENT_TYPE = KnownComponentTypes.ConnectionFactoryTypes.NoTx.getType();
    public static final ComponentType XA_COMPONENT_TYPE = KnownComponentTypes.ConnectionFactoryTypes.Tx.getType();


    // Constants 
    public static final String TXCF_DEFAULT_TEMPLATE="default__Tx ConnectionFactory";
    public static final String TXCF_LOCAL_TEMPLATE="Local Transaction__Tx ConnectionFactory";
    public static final String TXCF_XA_TEMPLATE="XA Transaction__Tx ConnectionFactory";
    public static final String NO_TXCF_DEFAULT_TEMPLATE="default__No Tx ConnectionFactory";

    public static final String CF_NAV_LABEL="Connection Factories";
    
    public static final String EMBJOPR_TEST_ADAPTER = "embjoprtestadapter.rar";
    public static final String TEST_CONNECTION_DEFINITION = "javax.resource.cci.ConnectionFactory";
    
    // The root element in a connection factory -ds.xml file
    public static final String CF_ROOT_ELEMENT = "connection-factories";

    /**
     * Create a new connection factory using the given type and properties.
     */
    protected void createConnFactory(CFType cfType,
                                     Map<String, MetaValue> propertiesMap)
					 throws IOException, EmbJoprTestException {
        expandNavTreeArrow(CF_NAV_LABEL);
        createResource(cfType.getLabel(), cfType.getTemplateName(),
                       propertiesMap);
    }

    /**
     * Delete the connection factory given by name.
     */
    protected void deleteConnFactory(CFType cfType,
                                     String connFactoryName) throws IOException,
                                     HtmlElementNotFoundException, EmbJoprTestException {
        clickNavTreeLink(cfType.getLabel());
        deleteResource(RESOURCE_SUMMARY_FORM, connFactoryName);
    }
    
    /**
     * Create a basic No Tx Connection Factory.
     */
    protected Map<String, MetaValue> createNoTxConnFactory(String jndiName) throws IOException, 
                                                                                   EmbJoprTestException {
        return createNoTxConnFactoryWithPrefill(jndiName, Boolean.TRUE);
    }
    
    /**
     * Create a basic Tx Connection Factory.
     */
    protected Map<String, MetaValue> createTxConnFactory(String jndiName) throws IOException, 
                                                                                 EmbJoprTestException {
        return createTxConnFactoryWithPrefill(jndiName, Boolean.TRUE);
    }
    
    /**
     * Create a basic Tx Connection Factory. Set "prefill" to the given boolean
     * value. This method is used during the metrics/operations tests.
     */
    protected Map<String, MetaValue> createTxConnFactoryWithPrefill(String jndiName,
                                                                    boolean prefill) 
                                                                    throws IOException, 
                                                                    EmbJoprTestException {
        Map<String, MetaValue> propertiesMap = new HashMap<String, MetaValue>();
        
        propertiesMap.put("jndi-name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("rar-name", SimpleValueSupport.wrap(EMBJOPR_TEST_ADAPTER));
        propertiesMap.put("connection-definition", SimpleValueSupport.wrap(TEST_CONNECTION_DEFINITION));
        propertiesMap.put("min-pool-size", SimpleValueSupport.wrap(new Integer(5)));
        propertiesMap.put("max-pool-size", SimpleValueSupport.wrap(new Integer(20)));
        propertiesMap.put("xa-resource-timeout", SimpleValueSupport.wrap(new Integer(0)));
        propertiesMap.put("prefill", SimpleValueSupport.wrap(prefill));
        propertiesMap.put("idle-timeout-minutes", SimpleValueSupport.wrap(new Integer(60)));
        propertiesMap.put("allocation-retry", SimpleValueSupport.wrap(new Integer(10)));
        //propertiesMap.put("track-connection-by-tx", SimpleValueSupport.wrap(Boolean.TRUE));
				//This property supercedes the "track-connection-by-tx" property from earlier versions of JBoss AS.
        propertiesMap.put("interleaving", SimpleValueSupport.wrap(Boolean.FALSE));
				propertiesMap.put("use-java-context", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("no-tx-separate-pools", SimpleValueSupport.wrap(Boolean.FALSE));
        propertiesMap.put("blocking-timeout-millis", SimpleValueSupport.wrap(new Integer(30000)));
        propertiesMap.put("securityDeploymentType", SimpleValueSupport.wrap("APPLICATION"));
        
        createConnFactory(CFType.LOCAL_TX_CF, propertiesMap);
        client.click(SAVE_BUTTON);
        
        return propertiesMap;
    }
    
    /**
     * Create a basic No Tx Connection Factory.
     */
    protected Map<String, MetaValue> createNoTxConnFactoryWithPrefill(String jndiName,
                                                                      boolean prefill) 
                                                                      throws IOException, 
                                                                             EmbJoprTestException {
        Map<String, MetaValue> propertiesMap = new HashMap<String, MetaValue>();
        
        propertiesMap.put("jndi-name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("rar-name", SimpleValueSupport.wrap(EMBJOPR_TEST_ADAPTER));
        propertiesMap.put("connection-definition", SimpleValueSupport.wrap(TEST_CONNECTION_DEFINITION));
        propertiesMap.put("min-pool-size", SimpleValueSupport.wrap(new Integer(5)));
        propertiesMap.put("max-pool-size", SimpleValueSupport.wrap(new Integer(20)));
        propertiesMap.put("idle-timeout-minutes", SimpleValueSupport.wrap(new Integer(20)));
        propertiesMap.put("prefill", SimpleValueSupport.wrap(prefill));
        propertiesMap.put("use-java-context", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("securityDeploymentType", SimpleValueSupport.wrap("APPLICATION"));
        
        createConnFactory(CFType.NO_TX_CF, propertiesMap);
        client.click(SAVE_BUTTON);
        
        return propertiesMap;
    }
     
    /**
     * Lookup the given ConnectionFactory and then create the
     * given number of connections.
     */
    public ArrayList<Connection> createConnections(int numConnections, 
                                                   String jndiName) throws NamingException, ResourceException {
        ArrayList<Connection> connections = new ArrayList<Connection>();
        
        InitialContext ctx = new InitialContext();
        ConnectionFactory cf = (ConnectionFactory)ctx.lookup("java:" + jndiName);
        
        for(int i = 0; i < numConnections; i++) {
            Connection con = cf.getConnection();
        
            // Actually exercise the connection
            con.getMetaData();
            
            assertNotNull("Null connection", con);
            connections.add(con);
        }
        
        return connections;
    }

    /**
     * Close each Connection in the given ArrayList of Connections.
     */
    public void closeConnections(ArrayList<Connection> connections) throws ResourceException {
        if(connections != null) {
            for(int i = 0; i < connections.size(); i++) {
                if(connections.get(i) != null) {
                    connections.get(i).close();
                }
            }
        }
    }
    
    /**
     * Creates the default properties for the connection factory.
     * Returns a map of String names to MetaValues.
     */
    protected Map<String, MetaValue> createConnFactoryPropertiesAsMetaValues(String jndiName) {

        Map<String, MetaValue> propertiesMap = new LinkedHashMap<String, MetaValue>();

        propertiesMap.put("jndi-name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("rar-name", SimpleValueSupport.wrap(EMBJOPR_TEST_ADAPTER));
        propertiesMap.put("connection-definition", SimpleValueSupport.wrap(TEST_CONNECTION_DEFINITION));
        propertiesMap.put("min-pool-size", SimpleValueSupport.wrap(new Integer(5)));
        propertiesMap.put("max-pool-size", SimpleValueSupport.wrap(new Integer(20)));
        propertiesMap.put("prefill", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("idle-timeout-minutes", SimpleValueSupport.wrap(new Integer(60)));
        propertiesMap.put("allocation-retry", SimpleValueSupport.wrap(new Integer(10)));
        propertiesMap.put("use-java-context", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("no-tx-separate-pools", SimpleValueSupport.wrap(Boolean.FALSE));

        return propertiesMap;
    }
    
    /**
     * Get the list of metrics that appear on the summary
     * page for connection factories.
     */
    protected ArrayList<String> getConnFactorySummaryMetrics() {
        ArrayList<String> connFactorySummaryMetrics = new ArrayList<String>();
        
        // The metrics that appear on the "Summary" page
        connFactorySummaryMetrics.add(AVAILABLE_CONNECTION_COUNT);
        connFactorySummaryMetrics.add(CONNECTION_COUNT);
        
        return connFactorySummaryMetrics;
    }
    
    /**
     * Check that the metrics for the given connection factory are correct.
     */
    protected void checkConnFactoryMetrics(String jndiName,
                                           Map<String, String> expectedMetrics,
                                           CFType cfType,
                                           ArrayList<Connection> connections) throws Exception {
        try {
            checkResourceMetrics(CF_NAV_LABEL, cfType.getLabel(), 
                                 jndiName, expectedMetrics, getConnFactorySummaryMetrics());
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteConnFactory(cfType, jndiName); 
        }
    }
    
    /**
     * Perform the given operation on the given connection factory and then
     * verify that the metrics are correct.
     */
    protected void performConnFactoryOperationAndCheckMetrics(CFType cfType, 
                                                              String jndiName,
                                                              String operationName,
                                                              Map<String, String> expectedMetrics,
                                                              ArrayList<Connection> connections) throws Exception {
        try {
            performResourceOperationAndCheckMetrics(CF_NAV_LABEL, cfType.getLabel(), 
                                                    jndiName, operationName, expectedMetrics,
                                                    getConnFactorySummaryMetrics(), Boolean.FALSE);  
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteConnFactory(cfType, jndiName);
        }
    }
    
    /**
     * Get the name of the -ds.xml file that corresponds to the connection 
     * factory given by jndiName.
     */
    protected String getConnFactoryConfigFile(String jndiName) {
        return System.getProperty("jsfunit.deploy.dir") + "/" + jndiName + "-ds.xml";
    }
    
    /**
     * Create a map of property names to property values for a particular
     * connection factory. The desired property names are given by specificProperties.
     * (This is useful when we need to check the values of some specific properties
     * only - eg. the configuration tests)
     */
    protected Map<String, MetaValue> getSpecificComponentProperties(String componentName,
                                                                    ComponentType type) throws Exception {
        
        String[] specificProperties = new String[] {"jndi-name", "rar-name", "connection-definition", 
                                                    "min-pool-size", "max-pool-size", 
                                                    "prefill", "idle-timeout-minutes",
                                                    "allocation-retry", "use-java-context", 
                                                    "no-tx-separate-pools", "blocking-timeout-millis" };
        return super.getSpecificComponentProperties(componentName, type, specificProperties, SECURED);
    }
}
