/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.datasources;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import java.util.Map;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;

/**
 * This class contains configuration tests for datasources.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class DatasourceConfigurationTest extends DatasourceTest {

    /**
     * Change the value of some already set properties.
     */
    public void testConfigureDatasourceChangeSetProperties() throws Exception {
        String jndiName = "ChangeSetPropertiesDS";
        Map<String, String> propertiesMap = createXADatasource(jndiName);
        
        // Change some property values that are already set 
        Map<String, MetaValue> propertiesMapChangesAsMetaValues = new LinkedHashMap<String, MetaValue>();
        propertiesMapChangesAsMetaValues.put("xa-resource-timeout", SimpleValueSupport.wrap(new Integer(60000)));
        propertiesMapChangesAsMetaValues.put("set-tx-query-timeout", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMapChangesAsMetaValues.put("user-name", SimpleValueSupport.wrap("newUser"));

        navigateToPage(propertiesMap.get("jndi-name"), 
                       DatasourceType.XA_DATASOURCE, 
                       CONFIGURATION_TAB);
                       
        // See EMBJOPR-58               
        assertFalse("The configuration page could not be displayed",
                    client.getPageAsText().contains("There was an error retrieving the configuration for this resource"));

        Map<String, String> propertiesMapChanges = formatPropertiesMap(propertiesMapChangesAsMetaValues);
        fillOutForm(propertiesMapChanges, RESOURCE_CONFIGURATION_FORM);
        client.click(SAVE_BUTTON);

        // Check for the appropriate success messages
        String expectedMessage = UPDATE_MESSAGE + DatasourceType.XA_DATASOURCE.getTypeName() 
                                 + " '" + propertiesMap.get("jndi-name") + "'";
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);
        
        checkComponentProperties(propertiesMapChangesAsMetaValues, jndiName, XA_COMPONENT_TYPE);

        assertTrue(isDatasourceDeployed(propertiesMap.get("jndi-name"), 
                                        DatasourceType.XA_DATASOURCE)); 

        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.XA_DATASOURCE);  
    }
    
    /**
     * Change the value of some previously unset properties.
     */
    public void testConfigureDatasourceChangeUnsetProperties() throws Exception {
        String jndiName = "ChangeUnsetPropertiesDS";
        
        Map<String, String> propertiesMap = createLocalTXDatasource(jndiName);

        // Set some property values that were previously unset 
        Map<String, MetaValue> propertiesMapChangesAsMetaValues = new LinkedHashMap<String, MetaValue>();
        propertiesMapChangesAsMetaValues.put("valid-connection-checker-class-name", 
                                             SimpleValueSupport.wrap("org.jboss.resource.adapter.jdbc.CheckValidConnectionSQL"));
        propertiesMapChangesAsMetaValues.put("set-tx-query-timeout", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMapChangesAsMetaValues.put("background-validation", SimpleValueSupport.wrap(Boolean.FALSE));
        propertiesMapChangesAsMetaValues.put("allocation-retry", SimpleValueSupport.wrap(new Integer(10000)));

        navigateToPage(propertiesMap.get("jndi-name"), 
                       DatasourceType.LOCAL_TX_DATASOURCE, 
                       CONFIGURATION_TAB);
                       
        // See EMBJOPR-58
        assertFalse("The configuration page could not be displayed",
                    client.getPageAsText().contains("There was an error retrieving the configuration for this resource"));

        Map<String, String> propertiesMapChanges = formatPropertiesMap(propertiesMapChangesAsMetaValues);
        fillOutForm(propertiesMapChanges, RESOURCE_CONFIGURATION_FORM);
        client.click(SAVE_BUTTON);

        // Check for the appropriate success messages
        String expectedMessage = UPDATE_MESSAGE + DatasourceType.LOCAL_TX_DATASOURCE.getTypeName() 
                                 + " '" + propertiesMap.get("jndi-name") + "'";
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);
        
        checkComponentProperties(propertiesMapChangesAsMetaValues, jndiName, LOCAL_TX_COMPONENT_TYPE);

        assertTrue(isDatasourceDeployed(propertiesMap.get("jndi-name"), 
                                        DatasourceType.LOCAL_TX_DATASOURCE)); 
        
        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE);   
    }

   /**
    * Unset the value of some previously set properties. 
    */ 
    public void testConfigureDatasourceUnsetProperties() throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = createNoTXDatasource("UnsetPropertiesDS");

        // Unset some property values
        navigateToPage(propertiesMap.get("jndi-name"), 
                       DatasourceType.NO_TX_DATASOURCE, 
                       CONFIGURATION_TAB);
                       
        // See EMBJOPR-58
        assertFalse("The configuration page could not be displayed",
                    client.getPageAsText().contains("There was an error retrieving the configuration for this resource"));

        enableOrDisableFormInput("user-name", Boolean.FALSE, RESOURCE_CONFIGURATION_FORM);
        enableOrDisableFormInput("idle-timeout-minutes", Boolean.FALSE, RESOURCE_CONFIGURATION_FORM);
        //enableOrDisableFormInput("track-connection-by-tx", Boolean.FALSE, RESOURCE_CONFIGURATION_FORM);
        client.click(SAVE_BUTTON);

        // Check for the appropriate success messages
        String expectedMessage = UPDATE_MESSAGE + DatasourceType.NO_TX_DATASOURCE.getTypeName() 
                                 + " '" + propertiesMap.get("jndi-name") + "'";
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);

        assertTrue(isDatasourceDeployed(propertiesMap.get("jndi-name"), 
                                        DatasourceType.NO_TX_DATASOURCE)); 
        
        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.NO_TX_DATASOURCE);   
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(DatasourceConfigurationTest.class);
    }
}

