/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.datasources;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import javax.management.ObjectName;
import java.util.Map;
import javax.management.JMException;
import java.sql.Connection;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.deployers.spi.management.KnownComponentTypes;
import org.jboss.managed.api.ComponentType;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;
import org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException;

/**
 * This class contains creation tests for datasources.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class DatasourceCreationTest extends DatasourceTest {

    /**
     * Create a new datasource. Leave some property values, that aren't
     * required, unset.
     */
    public void testCreateDatasource() throws Exception {

        // The properties we want to configure
        Map<String, MetaValue> propertiesMapAsMetaValues = createDatasourcePropertiesAsMetaValues();
        Map<String, String> propertiesMap = formatPropertiesMap(propertiesMapAsMetaValues);

        createDatasource(DatasourceType.LOCAL_TX_DATASOURCE,
                         DatasourceType.LOCAL_TX_DATASOURCE.getTemplateHtmlSelectValue(), // TODO: Redundant - remove.
                         propertiesMap);
        client.click(SAVE_BUTTON);

        // Check for the appropriate success messages
        String expectedMessage = ADD_MESSAGE + DatasourceType.LOCAL_TX_DATASOURCE.getTypeName();
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);

        assertTrue("Datasource is not deployed (isDatasourceDeployed() returned false).",
                   isDatasourceDeployed(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE));
        
        checkComponentProperties(propertiesMapAsMetaValues, propertiesMap.get("jndi-name"), LOCAL_TX_COMPONENT_TYPE);
        
        checkProperties(propertiesMap.get("jndi-name"),
                        DatasourceType.LOCAL_TX_DATASOURCE,
                        propertiesMap);

        // TODO: need to verify that appropriate default values were
        // set for properties that were not specified above

        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE);
    } 
    
    /**
     * Attempt to create a new datasource but leave at least one required
     * value unset. An error should occur.
     */
    public void testCreateDatasourceMissingRequiredValues() throws IOException, EmbJoprTestException {
        
        // Leave jndi-name and connection-url unset
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("user-name", "testUser");
        propertiesMap.put("max-pool-size", "10");
        propertiesMap.put("idle-timeout-minutes", "20");
        if(!isJBoss4){
            propertiesMap.put("prefill", "true");
            propertiesMap.put("set-tx-query-timeout", "true");
            propertiesMap.put("query-timeout", "1800");
        }
        
        createDatasource(DatasourceType.NO_TX_DATASOURCE, 
                        //"default__No Tx Datasource",
                        isJBoss4 ?
                                DatasourceTemplate.AS4_DEFAULT.getTemplateHtmlSelectValue() :
                                DatasourceTemplate.AS5_DEFAULT_NO_TX.getTemplateHtmlSelectValue(),
                        propertiesMap);
        client.click(SAVE_BUTTON);
        
        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE,
                                     MISSING_VALUE_MESSAGE,  true);
    }
    

    /**
     * Attempt to create a new datasource but set a property value
     * beyond its expected range of values. An error should occur.
     */
    public void testCreateDatasourcePropertyOutOfRange() throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("jndi-name", "PropertyOutOfRangeDS");
        propertiesMap.put("user-name", "testUser");
        propertiesMap.put("password", "password");
        propertiesMap.put("xa-datasource-class", "org.postgresql.xa.PGXADataSource");
        propertiesMap.put("xa-resource-timeout", "36000");

        // This number is too big
        propertiesMap.put("max-pool-size", "100000000000000");

        createDatasource(DatasourceType.XA_DATASOURCE,
                         DatasourceTemplate.AS5_DEFAULT_XA.getTemplateHtmlSelectValue(),
                         propertiesMap);
        client.click(SAVE_BUTTON);

        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE,
                                     OUT_OF_RANGE_MESSAGE,
                                     true);
    }
    
    /**
     * Attempt to create a new datasource but set a property value
     * to an invalid type. An error should occur.
     */
    public void testCreateDatasourceInvalidPropertyType() throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("jndi-name", "InvalidPropertyTypeDS");
        propertiesMap.put("min-pool-size", "10");
        propertiesMap.put("max-pool-size", "20");
        propertiesMap.put("driver-class", "org.hsqldb.jdbcDriver");
        propertiesMap.put("connection-url", "jdbc:hsqldb:.");
        propertiesMap.put("share-prepared-statements", "false"); 
        
        // This property value is supposed to be an integer
        propertiesMap.put("background-validation-millis", "abcde"); 
        
        createDatasource(DatasourceType.LOCAL_TX_DATASOURCE,
                         DatasourceType.LOCAL_TX_DATASOURCE.getTemplateHtmlSelectValue(), // TODO: Redundant, remove.
                         propertiesMap);
        client.click(SAVE_BUTTON);
        
        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE, 
                                     INVALID_INTEGER_MESSAGE, 
                                     true);
    }

    /**
     * Attempt to create a new datasource but set a property value to a value
     * that is the correct type but is not an allowed value for that
     * property. An error should occur.
     */
    public void testCreateDatasourcePropertyNotAllowed() throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("jndi-name", "PropertyNotAllowedDS");
        propertiesMap.put("driver-class", "org.hsqldb.jdbcDriver");
        propertiesMap.put("connection-url", "jdbc:hsqldb:.");

        // This number is a valid integer but is below the
        // allowable minimum value
        propertiesMap.put("max-pool-size", "-25");

        createDatasource(DatasourceType.NO_TX_DATASOURCE,
                         DatasourceType.NO_TX_DATASOURCE.getTemplateHtmlSelectValue(), //"default__No Tx Datasource",
                         propertiesMap);
        client.click(SAVE_BUTTON);

        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE,
                                     BELOW_ALLOWABLE_MINIMUM,
                                     true);
    }

    /**
     * Attempt to create a new datasource using a JNDI name that already exists.
     * An error should occur.
     */
    public void testCreateDatasourceDuplicateJNDIName() throws IOException, EmbJoprTestException {
        Map<String, String> propertiesMap = new HashMap<String, String>();
        propertiesMap.put("jndi-name", "DefaultDS");
        propertiesMap.put("driver-class", "org.hsqldb.jdbcDriver");
        propertiesMap.put("connection-url", "jdbc:hsqldb:.");

        createDatasource(DatasourceType.LOCAL_TX_DATASOURCE,
                         DatasourceType.LOCAL_TX_DATASOURCE.getTemplateHtmlSelectValue(), //"default__Local Tx Datasource",
                         propertiesMap);
        client.click(SAVE_BUTTON);

        // Check for the appropriate error messages
        String expectedMessage = "A " + DatasourceType.LOCAL_TX_DATASOURCE.getTypeName() + " named "
                                 + "'DefaultDS' already exists";
        checkClientAndServerMessages(expectedMessage, expectedMessage, true);
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(DatasourceCreationTest.class);
    }
}

