/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.datasources;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import java.util.Map;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.managed.api.ComponentType;

/**
 * This class contains deletion tests for datasources.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class DatasourceDeletionTest extends DatasourceTest {

    /**
     * Remove a Local TX Datasource.
     */
    public void testDeleteLocalTXDatasource() throws Exception {
        Map<String, String> propertiesMap = createLocalTXDatasource("DeleteLocalTXDS");
     
        deleteDS(propertiesMap, LOCAL_TX_COMPONENT_TYPE, DatasourceType.LOCAL_TX_DATASOURCE);
    }
    
    /**
     * Remove a No TX Datasource.
     */
    public void testDeleteNoTXDatasource() throws Exception {
        Map<String, String> propertiesMap = createNoTXDatasource("DeleteNoTXDS");
        
        deleteDS(propertiesMap, NO_TX_COMPONENT_TYPE, 
                 DatasourceType.NO_TX_DATASOURCE);
    }

    /**
     * Remove an XA Datasource.
     */
    public void testDeleteXADatasource() throws Exception {
        Map<String, String> propertiesMap = createXADatasource("DeleteXADS");
                
        deleteDS(propertiesMap, XA_COMPONENT_TYPE, DatasourceType.XA_DATASOURCE);
    }
    
    /**
     * Remove a Local TX Datasource. Use an existing -ds.xml file.
     */
    public void testDeleteLocalTXDatasourceUsingExistingDSFile() throws Exception {
        
        // Use an existing -ds.xml file
        Map<String, String> propertiesMap = getPropertyValuesMap("DeleteLocalTXExistingFile",
                                                                 DatasourceType.LOCAL_TX_DATASOURCE);
               
        deleteDS(propertiesMap, LOCAL_TX_COMPONENT_TYPE, DatasourceType.LOCAL_TX_DATASOURCE);
    }
    
    /**
     * Remove a No TX Datasource. Use an existing -ds.xml file.
     */
    public void testDeleteNoTXDatasourceUsingExistingDSFile() throws Exception {
        
        // Use an existing -ds.xml file
        Map<String, String> propertiesMap = getPropertyValuesMap("DeleteNoTXExistingFile",
                                                                 DatasourceType.NO_TX_DATASOURCE);
                
        deleteDS(propertiesMap, NO_TX_COMPONENT_TYPE, 
                 DatasourceType.NO_TX_DATASOURCE);
    }
    
    /**
     * Remove an XA Datasource. Use an existing -ds.xml file.
     */
    public void testDeleteXADatasourceUsingExistingDSFile() throws Exception {
        
        // Use an existing -ds.xml file
        Map<String, String> propertiesMap = getPropertyValuesMap("DeleteXAExistingFile",
                                                                 DatasourceType.XA_DATASOURCE);
          
        deleteDS(propertiesMap, XA_COMPONENT_TYPE, 
                 DatasourceType.XA_DATASOURCE);
    }
    
    /**
     * Common code for the testDelete* tests.
     */
    private void deleteDS(Map<String, String> propertiesMap, 
                          ComponentType componentType, DatasourceType datasourceType) throws Exception {
        String expectedMessage = DELETE_MESSAGE + datasourceType.getTypeName() 
                                 + " '" + propertiesMap.get("jndi-name") + "'";
        
        deleteDatasource(propertiesMap.get("jndi-name"), datasourceType);

        checkClientAndServerMessages(expectedMessage, expectedMessage, false);

        // Make sure the ManagedComponent was removed
        assertNull("The datasource was not removed after deletion (the ManagedComponent is non-null)",
                   getManagedComponent(propertiesMap.get("jndi-name"), componentType));
        
        // The entry should still be found in the -ds.xml file
        assertTrue("The datasource was removed from the -ds.xml file",
                   containsElement(getDatasourceConfigFile(propertiesMap.get("jndi-name")),
                                   datasourceType.getXmlElementName(),
                                   DS_ROOT_ELEMENT));
    }
    
    /**
     * Test the case in which the -ds.xml file that contains the datasource
     * to be deleted contains more than one datasource. Make sure the entire
     * -ds.xml file does not get deleted. (See JBAS-6227.)
     */
    public void testDeleteDatasourceComplexDSDotXmlFile() throws Exception {
        String jndiName = "NoTXDeletionTest";
        
        // MultipleDatasources-ds.xml contains two Local TX Datasources and
        // a No TX Datasource. We will delete the No TX Datasource (NoTXDeletionTest-ds.xml).
        deleteDatasource(jndiName, DatasourceType.NO_TX_DATASOURCE);
        
        // Check for the appropriate success messages
        String expectedMessage = DELETE_MESSAGE 
                                 + DatasourceType.NO_TX_DATASOURCE.getTypeName() + " '" 
                                 + jndiName + "'";
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);
        
        // Make sure the ManagedComponent was removed
        assertNull("The datasource was not removed after deletion (the ManagedComponent is non-null)",
                   getManagedComponent(jndiName, NO_TX_COMPONENT_TYPE));
        
        // Make sure the other datasources defined in the -ds.xml file were not
        // removed
        assertNotNull("Another datasource in the same -ds.xml file was also removed (the ManagedComponent is null)",
                      getManagedComponent("LocalTX1", LOCAL_TX_COMPONENT_TYPE));
        assertNotNull("Another datasource in the same -ds.xml file was also removed (the ManagedComponent is null)",
                      getManagedComponent("LocalTX2", LOCAL_TX_COMPONENT_TYPE));
        
        // Make sure the -ds.xml file remained unchanged
        String actualDSFile = System.getProperty("jsfunit.deploy.dir") 
                              + "/MultipleDatasources-ds.xml";
        String expectedDSFile = System.getProperty("jsfunit.testdata") 
                                                   + "/datasources/expectedFiles/ExpectedMultipleDatasourcesAfterDeletion-ds.xml";
        compareFiles(actualDSFile, expectedDSFile); 
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(DatasourceDeletionTest.class);
    }
}

