/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.datasources;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import java.util.Map;
import java.sql.Connection;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;

/**
 * This class contains metrics tests for datasources.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class DatasourceMetricsTest extends DatasourceTest {

    /**
     * Check that the metrics are correct after creating a new datasource.
     */
    public void testMetricsAfterDatasourceCreation() throws IOException, EmbJoprTestException {

        // Min pool size will be 5, max pool size will be 20
        Map<String, String> propertiesMap = createLocalTXDatasource("MetricsCreateDS");

        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "20");
        expectedMetrics.put(CONNECTION_COUNT, "0");
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "0");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "0");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "0");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5");

        checkMetrics(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE, expectedMetrics);

        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE);
    }

    /**
     * Make an initial request for a connection to a database. Make
     * sure the metrics are updated accordingly.
     */
    public void testMetricsAfterInitialDBConnection() throws Exception {

        // Create the datasource
        Map<String, String> propertiesMap = createLocalTXDatasource("MetricsInitalConnectionDS");

        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterInitialDBConnection(propertiesMap);
    }
    
    /**
     * Make an initial request for a connection to a database. Use an
     * existing datasource. Make sure the metrics are updated accordingly. 
     */
    public void testMetricsAfterInitialDBConnectionUsingExistingDSFile() throws Exception {
        
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap("MetricsInitialConnectionExistingFile",
                                                                 DatasourceType.LOCAL_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterInitialDBConnection(propertiesMap);
    }
    
    /**
     * Common code for the testMetricsAfterInitialDBConnection* tests.
     */
    private void checkMetricsAfterInitialDBConnection(Map<String, String> propertiesMap) throws Exception {
        
        // Create the first connection
        ArrayList<Connection> connections = createConnections(1, propertiesMap.get("jndi-name"),
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password"));
        
        try {
            
            // Set up the expected values
            Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
            expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "19");
            expectedMetrics.put(CONNECTION_COUNT, "5");
            expectedMetrics.put(CONNECTION_CREATED_COUNT, "5");
            expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
            expectedMetrics.put(IN_USE_CONNECTION_COUNT, "1");
            expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "1");
            expectedMetrics.put(MAX_SIZE, "20");
            expectedMetrics.put(MIN_SIZE, "5");

            checkMetrics(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE, expectedMetrics);
            
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE);
        }
    }

    /**
     * Establish multiple connections to a database. Make sure
     * the metrics are updated accordingly.
     */
    public void testMetricsAfterMultipleDBConnections() throws Exception {

        // Create the datasource
        Map<String, String> propertiesMap = createNoTXDatasource("MetricsMultipleConnectionDS");

        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterMultipleDBConnections(propertiesMap);
    }
    
    /**
     * Establish multiple connections to a database. Use an existing
     * datasource. Make sure the metrics are updated accordingly.
     */
    public void testMetricsAfterMultipleDBConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap("MetricsMultipleConnectionsExistingFile",
                                                                 DatasourceType.NO_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterMultipleDBConnections(propertiesMap);
    }
    
    /**
     * Common code for the testMetricsAfterMultipleDBConnections* tests.
     */
    private void checkMetricsAfterMultipleDBConnections(Map<String, String> propertiesMap) throws Exception {
        
        // Establish multiple connections
        ArrayList<Connection> connections = createConnections(6, propertiesMap.get("jndi-name"),
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password"));
        try {
            
            // Set up the expected values
            Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
            expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "14");
            expectedMetrics.put(CONNECTION_COUNT, "6");
            expectedMetrics.put(CONNECTION_CREATED_COUNT, "6");
            expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
            expectedMetrics.put(IN_USE_CONNECTION_COUNT, "6");
            expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "6");
            expectedMetrics.put(MAX_SIZE, "20");
            expectedMetrics.put(MIN_SIZE, "5");

            checkMetrics(propertiesMap.get("jndi-name"), DatasourceType.NO_TX_DATASOURCE, expectedMetrics);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.NO_TX_DATASOURCE);
        }
    }

    /**
     * Make sure the metrics are updated appropriately after
     * closing some connections.
     */
    public void testMetricsAfterClosingConnections() throws Exception {
        
        // Create the datasource
        Map<String, String> propertiesMap = createNoTXDatasource("MetricsCloseConnectionsDS");
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterClosingConnections(propertiesMap);
    }
    
    /**
     * Make sure the metrics are updated appropriately after
     * closing some connections. Use an existing datasource.
     */
    public void testMetricsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap("MetricsCloseConnectionsExistingFile",
                                                                 DatasourceType.NO_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        checkMetricsAfterClosingConnections(propertiesMap);
    }
    
    /**
     * Common code for the testMetricsAfterClosingConnections* tests.
     */
    private void checkMetricsAfterClosingConnections(Map<String, String> propertiesMap) throws Exception {
        
        // Establish some connections
        ArrayList<Connection> connections = createConnections(10, propertiesMap.get("jndi-name"),
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password"));

        try {
            
            // Close some connections
            for(int i = 0; i < 2; i++) {
                disconnectDB(connections.get(i));
                connections.set(i, null);
            }
      
            // Set up the expected values
            Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
            expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "12"); 
            expectedMetrics.put(CONNECTION_COUNT, "10");          
            expectedMetrics.put(CONNECTION_CREATED_COUNT, "10");    
            expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");   
            expectedMetrics.put(IN_USE_CONNECTION_COUNT, "8");     
            expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "10");
            expectedMetrics.put(MAX_SIZE, "20");
            expectedMetrics.put(MIN_SIZE, "5");

            checkMetrics(propertiesMap.get("jndi-name"), DatasourceType.NO_TX_DATASOURCE, expectedMetrics); 
            
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.NO_TX_DATASOURCE);
        }
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(DatasourceMetricsTest.class);
    }
}

