/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.datasources;

import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import java.util.Map;
import java.sql.Connection;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import java.io.IOException;
import org.jboss.managed.api.ComponentType;

/**
 * This class contains operations tests for datasources.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class DatasourceOperationsTest extends DatasourceTest {

    /* FLUSH OPERATION */
    
    /**
     * Test the "Flush" operation after creating a new datasource.
     */
    public void testFlushOperationAfterDatasourceCreation() throws IOException, EmbJoprTestException { 
        
        // Min pool size will be 6, max pool size will be 15
        Map<String, String> propertiesMap = createXADatasource("FlushAfterDatasourceCreationDS");

        performDatasourceOperation(propertiesMap.get("jndi-name"), DatasourceType.XA_DATASOURCE, FLUSH);

        // Make sure the metrics are updated appropriately
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "15");
        expectedMetrics.put(CONNECTION_COUNT, "0"); 
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "0");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "0");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "0");
        expectedMetrics.put(MAX_SIZE, "15");
        expectedMetrics.put(MIN_SIZE, "6");

        checkMetrics(propertiesMap.get("jndi-name"), DatasourceType.XA_DATASOURCE, expectedMetrics);
        
        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.XA_DATASOURCE);   
    }
    
    /**
     * Test the "Flush" operation after multiple connections to a database 
     * have been created.  
     */
    public void testFlushOperationAfterMultipleConnections() throws Exception { 
        
        // Min pool size will be 5, max pool size will be 20
        Map<String, String> propertiesMap = createLocalTXDatasource("FlushAfterMultipleConnectionsDS");

        flushOperationAfterMultipleConnections(propertiesMap);   
    }
    
    /**
     * Test the "Flush" operation after multiple connections to a database 
     * have been created. Use an existing datasource.
     */ 
    public void testFlushOperationAfterMultipleConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap("FlushAfterMultipleConnectionsExistingFile",
                                                                 DatasourceType.LOCAL_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        flushOperationAfterMultipleConnections(propertiesMap);
    }
    
    /**
     * Common code for the testFlushOperationAfterMultipleConnections* tests.
     */
    private void flushOperationAfterMultipleConnections(Map<String, String> propertiesMap) throws Exception {
        
        // Create some connections and then close them. This will
        // result in non-zero values for "Connection Created Count" and
        // "Max Connections In Use Count".
        ArrayList<Connection> connections = createConnections(5, propertiesMap.get("jndi-name"), 
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password")); 
        
        closeConnections(connections); 

        performDatasourceOperation(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE, FLUSH);

        // Make sure the metrics are updated appropriately
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(AVAILABLE_CONNECTION_COUNT, "20");
        expectedMetrics.put(CONNECTION_COUNT, "0"); 
        expectedMetrics.put(CONNECTION_CREATED_COUNT, "0");
        expectedMetrics.put(CONNECTION_DESTROYED_COUNT, "0");
        expectedMetrics.put(IN_USE_CONNECTION_COUNT, "0");
        expectedMetrics.put(MAX_CONNECTIONS_IN_USE_COUNT, "0");
        expectedMetrics.put(MAX_SIZE, "20");
        expectedMetrics.put(MIN_SIZE, "5");

        checkMetrics(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE, expectedMetrics);

        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE);   
    }
    
    /* LIST FORMATTED SUB POOL STATISTICS OPERATION */
    
    /**
     * Test the "List Formatted Sub Pool Statistics" after creating a new
     * datasource.
     */
    public void testListFormattedSubPoolStatisticsAfterDatasourceCreation() throws Exception {

        // Min pool size will be 5, max pool size will be 20
        Map<String, String> propertiesMap = createNoTXDatasource("ListFormattedSubPoolStatisticsAfterDatasourceCreationDS");

        // Set up the expected results
        StringBuffer expectedBuffer = new StringBuffer();
        expectedBuffer.append("Sub Pool Statistics: \n");
        expectedBuffer.append("Sub Pool Count: 0\n");
        expectedBuffer.append(POOL_SEPARATOR);
        expectedBuffer.append("\n\n");
        
        String errorMessage = "Incorrect sub pool statistics";

        performResourceOperationAndCheckValueBox(DS_NAV_LABEL,
                                                 DatasourceType.NO_TX_DATASOURCE.getLabel(),
                                                 propertiesMap.get("jndi-name"),
                                                 LIST_FORMATTED_SUB_POOL_STATISTICS,
                                                 Boolean.TRUE,
                                                 expectedBuffer,
                                                 errorMessage);
        
        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.NO_TX_DATASOURCE);   
    }

    /**
     * Test the "List Formatted Sub Pool Statistics" operation after multiple
     * connections to a database have been established.
     */
    public void testListFormattedSubPoolStatisticsAfterMultipleConnections() throws Exception {

        // Min pool size will be 5, max pool size will be 20
        Map<String, String> propertiesMap = createLocalTXDatasource("ListFormattedSubPoolStatisticsAfterMultipleConnectionsDS");

        listFormattedSubPoolStatisticsAfterMultipleConnections(propertiesMap);   
    }
    
    /**
     * Test the "List Formatted Sub Pool Statistics" operation after multiple
     * connections to a database have been established. Use an existing datasource. 
     */
    public void testListFormattedSubPoolStatisticsAfterMultipleConnectionsUsingExistingDSFile() throws Exception {
        
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap("ListFormattedAfterMultipleConnectionsExistingFile",
                                                                 DatasourceType.LOCAL_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        listFormattedSubPoolStatisticsAfterMultipleConnections(propertiesMap);
    }

    /**
     * Common code for the 
     * testListFormattedSubPoolStatisticsAfterMultipleConnections* tests.
     */
    private void listFormattedSubPoolStatisticsAfterMultipleConnections(Map<String, String> propertiesMap) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(4, propertiesMap.get("jndi-name"), 
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password")); 
        try { 
            
            // Set up the expected results
            StringBuffer expectedBuffer = new StringBuffer();
            expectedBuffer.append("Sub Pool Statistics: \n");
            expectedBuffer.append("Sub Pool Count: 1\n");
            expectedBuffer.append(POOL_SEPARATOR);
            expectedBuffer.append("\n\n");
            //expectedBuffer.append("Track By Transaction: true\n");  // Deprecated property - JBAS-6918
            expectedBuffer.append("Available Connections Count: 16\n");
            expectedBuffer.append("Max Connections In Use Count:4\n");
            expectedBuffer.append("Connections Destroyed Count:0\n");
            expectedBuffer.append("Connections In Use Count:4\n");
            
            String errorMessage = "Incorrect sub pool statistics";

            performResourceOperationAndCheckValueBox(DS_NAV_LABEL,
                                                     DatasourceType.LOCAL_TX_DATASOURCE.getLabel(),
                                                     propertiesMap.get("jndi-name"),
                                                     LIST_FORMATTED_SUB_POOL_STATISTICS,
                                                     Boolean.TRUE,
                                                     expectedBuffer,
                                                     errorMessage);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE);
        }
    }
    
    /**
     * Test the "List Formatted Sub Pool Statistics" operation after
     * closing some connections.
     */
    public void testListFormattedSubPoolStatisticsAfterClosingConnections() throws Exception {

        // Min pool size will be 5, max pool size will be 20
        Map<String, String> propertiesMap = createLocalTXDatasource("ListFormattedSubPoolStatisticsAfterClosingConnectionsDS");

        listFormattedSubPoolStatisticsAfterClosingConnections(propertiesMap); 
    }
    
    /**
     * Test the "List Formatted Sub Pool Statistics" operation after
     * closing some connections. Use an existing datasource.
     */
    public void testListFormattedSubPoolStatisticsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap("ListFormattedAfterClosingConnectionsExistingFile",
                                                                 DatasourceType.LOCAL_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);
        
        // Min pool size will be 5, max pool size will be 20
        listFormattedSubPoolStatisticsAfterClosingConnections(propertiesMap); 
    }
    
    /**
     * Common code for the 
     * testListFormattedSubPoolStatisticsAfterClosingConnections* tests.
     */
    private void listFormattedSubPoolStatisticsAfterClosingConnections(Map<String, String> propertiesMap) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(10, propertiesMap.get("jndi-name"), 
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password")); 
        try {
            
            // Close some connections
            for(int i = 0; i < 2; i++) {
                disconnectDB(connections.get(i));
                connections.set(i, null);
            }
            
            // Set up the expected results
            StringBuffer expectedBuffer = new StringBuffer();
            expectedBuffer.append("Sub Pool Statistics: \n");
            expectedBuffer.append("Sub Pool Count: 1\n");
            expectedBuffer.append(POOL_SEPARATOR);
            expectedBuffer.append("\n\n");
            //expectedBuffer.append("Track By Transaction: true\n"); // Deprecated property - JBAS-6918
            expectedBuffer.append("Available Connections Count: 12\n");
            expectedBuffer.append("Max Connections In Use Count:10\n");
            expectedBuffer.append("Connections Destroyed Count:0\n");
            expectedBuffer.append("Connections In Use Count:8");
        
            String errorMessage = "Incorrect sub pool statistics";

            performResourceOperationAndCheckValueBox(DS_NAV_LABEL,
                                                     DatasourceType.LOCAL_TX_DATASOURCE.getLabel(),
                                                     propertiesMap.get("jndi-name"),
                                                     LIST_FORMATTED_SUB_POOL_STATISTICS,
                                                     Boolean.TRUE,
                                                     expectedBuffer,
                                                     errorMessage);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.LOCAL_TX_DATASOURCE);
        }
    }


		
    /* LIST STATISTICS OPERATION */
    
    /**
     * Test Name: testListStatisticsAfterCreation
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation on a newly created datasource.
     */
    public void testDatasourceListStatisticsAfterCreation() throws Exception {
        String jndiName = "ListStatisticsAfterDatasourceCreationDS";
        
        // Min pool size will be 5, max pool size will be 20
        createNoTXDatasource(jndiName);
        
        // Set up the expectedStatistics
        Map<String, String> expectedStatistics = formatListStatisticsResults(jndiName, 
                                                                             NO_TX_COMPONENT_TYPE, 
                                                                             listStatisticsProps);
        
        // Additional statistics that need to be checked
        expectedStatistics.put("criteria", "ByContainer");
        expectedStatistics.put("name", "jboss.jca:service=ManagedConnectionFactory,name="+jndiName);
        expectedStatistics.put("subPoolCount", "0"); 
        expectedStatistics.put("totalConnectionsInUseCount", "0");
        expectedStatistics.put("totalMaxConnectionsInUseCount", "0");
        
        performResourceOperationAndCheckTable(DS_NAV_LABEL, DatasourceType.NO_TX_DATASOURCE.getLabel(), 
                                              jndiName, LIST_STATISTICS, Boolean.FALSE, 
                                              expectedStatistics);
        
        deleteDatasource(jndiName, DatasourceType.NO_TX_DATASOURCE);
    }
    
    /**
     * Test Name: testListStatisticsAfterMultipleConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a datasource when there are 
     * multiple connections.
     */
    public void testDatasourceListStatisticsAfterMultipleConnections() throws Exception {
        String jndiName = "ListStatisticsAfterMultipleConnectionsDS";
        
        // Min pool size will be 5, max pool size will be 20
        Map<String, String> propertiesMap = createLocalTXDatasource(jndiName);
        
        performListStatisticsAfterMultipleConnections(jndiName, DatasourceType.LOCAL_TX_DATASOURCE, 
                                                      LOCAL_TX_COMPONENT_TYPE, propertiesMap);   
    } 
    
    /**
     * Test Name: testListStatisticsAfterMultipleConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a datasource when there are 
     * multiple connections. Use an existing -ds.xml file.
     */
    public void testDatasourceListStatisticsAfterMultipleConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "ListStatisticsMultipleConnectionsExistingFile";
        
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap(jndiName,
                                                                 DatasourceType.LOCAL_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);
        
        performListStatisticsAfterMultipleConnections(jndiName, DatasourceType.LOCAL_TX_DATASOURCE, 
                                                      LOCAL_TX_COMPONENT_TYPE, propertiesMap);   
    } 
    
    /**
     * Common code for the
     * testListStatisticsAfterMultipleConnections* tests.
     */
    private void performListStatisticsAfterMultipleConnections(String jndiName,
                                                               DatasourceType datasourceType,
                                                               ComponentType componentType,
                                                               Map<String, String> propertiesMap) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(6, propertiesMap.get("jndi-name"), 
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password")); 
        
        try { 
            
            // Set up the expected statistics
            Map<String, String> expectedStatistics = formatListStatisticsResults(jndiName, 
                                                                                 componentType, 
                                                                                 listStatisticsProps);
            
            // Additional statistics that need to be checked
            expectedStatistics.put("criteria", "ByContainer");
            expectedStatistics.put("name", "jboss.jca:service=ManagedConnectionFactory,name=" 
                                   + jndiName);
            expectedStatistics.put("subPoolCount", "1"); 
            expectedStatistics.put("totalConnectionsInUseCount", "6");
            expectedStatistics.put("totalMaxConnectionsInUseCount", "6");
              
            performResourceOperationAndCheckTable(DS_NAV_LABEL, datasourceType.getLabel(), jndiName,
                                                  LIST_STATISTICS, Boolean.FALSE, expectedStatistics);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteDatasource(jndiName, DatasourceType.LOCAL_TX_DATASOURCE);
        }
    }
    
    /**
     * Test Name: testListStatisticsAfterClosingConnections
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a datasource after 
     * requesting multiple connections and then closing some of them.
     */
    public void testDatasourceListStatisticsAfterClosingConnections() throws Exception {
        String jndiName = "ListStatisticsAfterClosingConnectionsDS";
        
        // Min pool size will be 5, max pool size will be 20
        Map<String, String> propertiesMap = createNoTXDatasource(jndiName);

        performListStatisticsAfterClosingConnections(jndiName, DatasourceType.NO_TX_DATASOURCE, 
                                                     NO_TX_COMPONENT_TYPE, propertiesMap); 
    } 
    
    /**
     * Test Name: testListStatisticsAfterClosingConnectionsUsingExistingDSFile
     * Assertion: Verify that the results are correct after executing 
     * the "List Statistics" operation for a datasource after 
     * requesting multiple connections and then closing some of them. Use an
     * existing -ds.xml file.
     */
    public void testDatasourceListStatisticsAfterClosingConnectionsUsingExistingDSFile() throws Exception {
        String jndiName = "ListStatisticsClosingConnectionsExistingFile";
        
        // Use an existing datasource
        Map<String, String> propertiesMap = getPropertyValuesMap(jndiName,
                                                                 DatasourceType.NO_TX_DATASOURCE);
        
        expandNavTreeArrow(DS_NAV_LABEL);

        performListStatisticsAfterClosingConnections(jndiName, DatasourceType.NO_TX_DATASOURCE, 
                                                     NO_TX_COMPONENT_TYPE, propertiesMap); 
    } 
    
    /**
     * Common code for the
     * testListStatisticsAfterClosingConnections* tests.
     */
    private void performListStatisticsAfterClosingConnections(String jndiName,
                                                              DatasourceType datasourceType,
                                                              ComponentType componentType,
                                                              Map<String, String> propertiesMap) throws Exception {
        
        // Create some connections
        ArrayList<Connection> connections = createConnections(10, propertiesMap.get("jndi-name"), 
                                                              propertiesMap.get("user-name"), 
                                                              propertiesMap.get("password")); 
        
        try { 
            
            // Close some connections
            for(int i = 0; i < 2; i++) {
                disconnectDB(connections.get(i));
                connections.set(i, null);
            }
            
            // Set up the expectedStatistics
            Map<String, String> expectedStatistics = formatListStatisticsResults(jndiName, 
                                                                                 componentType, 
                                                                                 listStatisticsProps);
            
            // Additional statistics that need to be checked
            expectedStatistics.put("criteria", "ByContainer");
            expectedStatistics.put("name", "jboss.jca:service=ManagedConnectionFactory,name=" 
                                   + jndiName);
            expectedStatistics.put("subPoolCount", "1"); 
            expectedStatistics.put("totalConnectionsInUseCount", "8");
            expectedStatistics.put("totalMaxConnectionsInUseCount", "10");
            
            performResourceOperationAndCheckTable(DS_NAV_LABEL, datasourceType.getLabel(), jndiName,
                                                  LIST_STATISTICS, Boolean.FALSE, expectedStatistics);
        } finally {
            
            // Clean up
            closeConnections(connections);
            deleteDatasource(propertiesMap.get("jndi-name"), DatasourceType.NO_TX_DATASOURCE);
        }
    }
    
    /**
     * @return the suite of tests being tested
     */
    public static Test suite() {
        return new TestSuite(DatasourceOperationsTest.class);
    }
}

