/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.datasources;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import javax.management.ObjectName;
import java.util.Map;
import javax.management.JMException;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.deployers.spi.management.KnownComponentTypes;
import org.jboss.managed.api.ComponentType;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;
import org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException;

/**
 * This is the base test class for datasource tests for AS 5.
 * 
 * @author Farah Juma
 *
 */
public abstract class DatasourceTest extends DatasourceTestBase {

    // Note: Nav panel labels, names of metrics, and operation names for 
    // datasources are found in DatasourceTestBase.java
    
    // Component types
    public static final ComponentType LOCAL_TX_COMPONENT_TYPE = KnownComponentTypes.DataSourceTypes.LocalTx.getType();
    public static final ComponentType NO_TX_COMPONENT_TYPE = KnownComponentTypes.DataSourceTypes.NoTx.getType();
    public static final ComponentType XA_COMPONENT_TYPE = KnownComponentTypes.DataSourceTypes.XA.getType();

    /**
     * Create a new datasource using the given type, template, and properties.
     *
     * AS 5 version differs in menu tree structure - one more step.
     * Thus, this method is overriden from the Base class.
     * 
     * @param datasourceType must be a datasource type from DatasourceType enum.
     * It's label from the left nav (eg. "Local Tx Datasources") is in getLabel() property.
     *
     */
    @Override
    protected void createDatasource(DatasourceType datasourceType,
                                    String datasourceTemplate,
                                    Map<String, String> propertiesMap) throws IOException, EmbJoprTestException {

        // Expand the "Datasources" tree node
        ClickableElement datasourcesArrow = getNavTreeArrow(DS_NAV_LABEL);
        datasourcesArrow.click();

        HtmlAnchor datasourceLink = getNavTreeLink(datasourceType.getLabel());
        datasourceLink.click();
        
        // Add a new datasource 
        client.click(ADD_NEW_RESOURCE_BUTTON);
        HtmlSelect menu = (HtmlSelect)client.getElement(SELECT_TEMPLATE);
        menu.setSelectedAttribute(datasourceTemplate, Boolean.TRUE);
        client.click(CONTINUE_AFTER_TEMPLATE_SELECTION_BUTTON);
        
        // Configure the properties associated with this datasource
        fillOutForm(propertiesMap, RESOURCE_CONFIGURATION_FORM);  
    }

    /**
     * Properties needed for testCreateDatasource.
     * @returns the properties from the base class overriden by properties for AS 5.
     */
    @Override
    protected Map<String, MetaValue> createDatasourcePropertiesAsMetaValues() {
        Map<String, MetaValue> propertiesMap = super.createDatasourcePropertiesAsMetaValues();
        
        // Additional properties
        propertiesMap.put("query-timeout", SimpleValueSupport.wrap(new Integer(180)));
        propertiesMap.put("share-prepared-statements", SimpleValueSupport.wrap(Boolean.FALSE));
        propertiesMap.put("stale-connection-checker-class-name", 
                          SimpleValueSupport.wrap("org.jboss.resource.adapter.jdbc.StaleConnectionChecker"));
        propertiesMap.put("allocation-retry", SimpleValueSupport.wrap(new Integer(10000)));
        propertiesMap.put("allocation-retry-wait-millis", SimpleValueSupport.wrap(new Long(10000)));
        propertiesMap.put("background-validation-millis", SimpleValueSupport.wrap(new Long(15000)));
        propertiesMap.put("prefill", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("use-try-lock", SimpleValueSupport.wrap(new Long(61000)));
        
        return propertiesMap;
    }
    
    /**
     * Perform the given operation on the given datasource.
     * 
     * performDatasourceOperation() assumes that the "Datasources"
     * tree node is already expanded when this method is called.
     */
    protected void performDatasourceOperation(String datasourceName,
                                              DatasourceType datasourceType,
                                              String operationName) throws IOException, EmbJoprTestException {

        navigateToPage(datasourceName, datasourceType, CONTROL_TAB);

        HtmlForm form = (HtmlForm)client.getElement(OPERATIONS_FORM);
        String xpath = ".//input[@value=\"" + operationName + "\"]";

        HtmlButtonInput operationButton = (HtmlButtonInput)form.getFirstByXPath(xpath);
        operationButton.click();
    }

    /**
     * Navigate to the given tab (eg. "configurationTab", "controlTab") for 
     * the given datasource.
     * 
     * navigateToPage() assumes that the "Datasources" tree node is already 
     * expanded when this method is called.
     */
    protected void navigateToPage(String datasourceName,
                                  DatasourceType datasourceType,
                                  String tabName) throws IOException, EmbJoprTestException {
        
        //refreshTreeNode(DS_NAV_LABEL);
        
        if(!ejtt.getNavTree().getNodeByLabel(datasourceType.getLabel()).isExpanded()) {
            expandNavTreeArrow(datasourceType.getLabel());
        }
       
        HtmlAnchor datasource = getNavTreeLink(datasourceName);
        datasource.click();

        HtmlAnchor tabLink = (HtmlAnchor)client.getElement(tabName);
        tabLink.click(); 
    }
    
    /**
     * Delete the datasource given by datasourceName.
     */
    protected void deleteDatasource(String datasourceName, 
                                    DatasourceType datasourceType) throws IOException, 
                                    HtmlElementNotFoundException, EmbJoprTestException {
        if(!ejtt.getNavTree().getNodeByLabel(DS_NAV_LABEL).isExpanded()) {
            expandNavTreeArrow(DS_NAV_LABEL);
        }
        
        HtmlAnchor datasourceLink = getNavTreeLink(datasourceType.getLabel()); 
        datasourceLink.click();

        HtmlSelect menu = (HtmlSelect)client.getElement(CURRENT_PAGE_SIZE);
        if(menu != null) {
            menu.setSelectedAttribute(MAX_ITEMS_PER_PAGE, Boolean.TRUE);
        }

        HtmlButtonInput deleteButton;
        try {
            deleteButton = getDeleteButton(RESOURCE_SUMMARY_FORM, datasourceName);
            deleteButton.click();
        } catch (IllegalStateException e) { 
            
            // The delete button was not found on this page
            client.click(NEXT_PAGE);
            deleteButton = getDeleteButton(RESOURCE_SUMMARY_FORM, datasourceName);
            deleteButton.click();
        }  
    }

    /**
     * This method should query the JMX server and decide whether the given 
     * MBean displays deployed resource.
     * @param deploymentMBean  Name of the MBean to examine. Differs between AS4 and 5.
     * @return true  if the MBean indicates that the resource is deployed, false otherwise.
     * @throws javax.management.JMException  upon JMX related error, including invalid
     *                 or non-existent MBean / attribute name.
     * @throws java.io.IOException  upon I/O error.
     */
    @Override
    protected boolean isMBeanStateDeployedImpl(ObjectName deploymentMBean)  throws JMException, IOException {

        JMXUtils jmxUtils = JMXUtils.getInstanceForLocalJBoss();

        Object state = jmxUtils.getMBeanAttribute(deploymentMBean, "State");
        return ("DEPLOYED".equals( state.toString() ));

    }

    /**
     * Return the location of the -ds.xml file that corresponds to the 
     * datasource given by jndiName.
     */
    @Override
    protected String getDatasourceConfigFile( String jndiName ) {
        return System.getProperty("jsfunit.deploy.dir") + "/" + jndiName + "-ds.xml";
    }

    /**
     * Return the name of the MBean that corresponds to the given
     * JNDI name and service.
     */
    @Override
    protected String getMBeanName( String jndiName, String serviceName ){
        return "jboss.deployment:id=\"jboss.jca:name=" + jndiName + "," +
        "service=" + serviceName + "\",type=Component";
    }
}

