/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.jmsDestinations;

import java.util.Map;
import java.util.LinkedHashMap;
import org.jboss.metatype.api.values.MetaValue;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.managed.api.ComponentType;

/**
 * This class contains configuration tests for JMS topics and queues.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class JMSConfigurationTest extends JMSTest {
    
    /**
     * Test Name: testConfigureQueueChangePropertiesUsingExistingServiceFile
     * Assertion: Verify the ability to change queue property values. 
     * Use a queue that already exists.
     * 
     * Strategy: Change some property values for the queue. 
     * Make sure the properties of the ManagedComponent are updated accordingly.
     */
    public void testConfigureQueuePropertiesUsingExistingServiceFile() throws Exception {
        String jndiName = "ChangePropertiesExistingQueue";
        Map<String, MetaValue> propertiesMap = getSpecificComponentProperties(jndiName, QUEUE_COMPONENT_TYPE);
        
        // The properties we are going to change
        // (Before: maxDeliveryAttempts=-1, redeliveryDelay=-1 )
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("maxDeliveryAttempts", SimpleValueSupport.wrap(new Integer(0)));
        propertiesMapChanges.put("redeliveryDelay", SimpleValueSupport.wrap(new Long(40000)));
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        
        changeDestinationProperties(jndiName, DestinationType.QUEUE, 
                                    QUEUE_COMPONENT_TYPE, propertiesMapChanges,
                                    propertiesMap);
    }
    
    /**
     * Test Name: testConfigureTopicChangePropertiesUsingExistingServiceFile
     * Assertion: Verify the ability to change topic property values. 
     * Use a topic that already exists.
     * 
     * Strategy: Change some property values for the topic.
     * Make sure the properties of the ManagedComponent are updated accordingly.
     */
    public void testConfigureTopicChangePropertiesUsingExistingServiceFile() throws Exception {
        String jndiName = "ChangePropertiesExistingTopic";
        Map<String, MetaValue> propertiesMap = getSpecificComponentProperties(jndiName, TOPIC_COMPONENT_TYPE);
      
        // The properties we are going to change
        // (Before: maxDeliveryAttempts=-1, redeliveryDelay=-1 )
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("maxDeliveryAttempts", SimpleValueSupport.wrap(new Integer(15)));
        propertiesMapChanges.put("redeliveryDelay", SimpleValueSupport.wrap(new Long(40000)));
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        
        changeDestinationProperties(jndiName, DestinationType.TOPIC, 
                                    TOPIC_COMPONENT_TYPE, propertiesMapChanges,
                                    propertiesMap);
    }
   
    /**
     * Test Name: testConfigureQueueChangeProperties
     * Assertion: Verify the ability to change queue property values.
     * 
     * Strategy: Create a new queue. Change some property 
     * values for this queue. Make sure the properties of the ManagedComponent 
     * are updated accordingly.
     */
    public void testConfigureQueueChangeProperties() throws Exception {
        String jndiName = "ChangePropertiesQueue";
        
        // Create the queue first
        Map<String, MetaValue> propertiesMap = createQueue(jndiName);
        
        // The properties we are going to change
        // (Before: maxDeliveryAttempts=5, redeliveryDelay=-1 )
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("maxDeliveryAttempts", SimpleValueSupport.wrap(new Integer(0)));
        propertiesMapChanges.put("redeliveryDelay", SimpleValueSupport.wrap(new Long(40000)));
        
        changeDestinationProperties(jndiName, DestinationType.QUEUE, 
                                    QUEUE_COMPONENT_TYPE, propertiesMapChanges,
                                    propertiesMap);
    }
    
    /**
     * Test Name: testConfigureTopicChangeProperties
     * Assertion: Verify the ability to change topic property values.
     * 
     * Strategy: Create a new topic. Change some property 
     * values for this topic. Make sure the properties of the ManagedComponent 
     * are updated accordingly.
     */
    public void testConfigureTopicChangeProperties() throws Exception {
        String jndiName = "ChangePropertiesTopic";
        
        // Create the topic first
        Map<String, MetaValue> propertiesMap = createTopic(jndiName);
        
        // The properties we are going to change
        // (Before: redeliveryDelay=60000, maxDeliveryAttempts=-1 )
        Map<String, MetaValue> propertiesMapChanges = new LinkedHashMap<String, MetaValue>();
        propertiesMapChanges.put("redeliveryDelay", SimpleValueSupport.wrap(new Long(40000)));
        propertiesMapChanges.put("maxDeliveryAttempts", SimpleValueSupport.wrap(new Integer(15)));
        
        changeDestinationProperties(jndiName, DestinationType.TOPIC, 
                                    TOPIC_COMPONENT_TYPE, propertiesMapChanges,
                                    propertiesMap);
    }
      
    /**
     * Common code for changing a destination's property values.
     */
    private void changeDestinationProperties(String jndiName, 
                                             DestinationType destinationType,
                                             ComponentType componentType,
                                             Map<String, MetaValue> propertiesMapChanges,
                                             Map<String, MetaValue> propertiesMap) throws Exception {
        
        // The success message we should see
        String expectedMessage = UPDATE_MESSAGE + destinationType.getName() + " '" 
                                 + jndiName + "'";
        
        // Change some property values
        configureResource(JMS_NAV_LABEL, destinationType.getNavLabel(), jndiName, 
                          propertiesMap, propertiesMapChanges, componentType,
                          expectedMessage);
  
        assertTrue("The destination is not deployed ",
                   isDeployed(jndiName + "-service.xml"));
        
        // Clean up
        deleteDestination(destinationType, jndiName); 
    }
    
    /**
     * Test Name: testConfigureQueueUnsetPropertiesUsingExistingServiceFile
     * Assertion: Verify the ability to unset queue property values. Use a
     * queue that already exists.
     * 
     * Strategy: Unset some property values for the queue. Make sure the
     * rest of the properties for the ManagedComponent remain unchanged.
     */
    public void testConfigureQueueUnsetPropertiesUsingExistingServiceFile() throws Exception {
        String jndiName = "UnsetPropertiesExistingQueue";    
        unsetPropertiesUsingExistingServiceFile(jndiName, DestinationType.QUEUE, 
                                                QUEUE_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testConfigureTopicUnsetPropertiesUsingExistingServiceFile
     * Assertion: Verify the ability to unset topic property values. Use a
     * topic that already exists.
     * 
     * Strategy: Unset some property values for the topic. Make sure the
     * rest of the properties for the ManagedComponent remain unchanged.
     */
    public void testConfigureTopicUnsetPropertiesUsingExistingServiceFile() throws Exception {
        String jndiName = "UnsetPropertiesExistingTopic";    
        unsetPropertiesUsingExistingServiceFile(jndiName, DestinationType.TOPIC, 
                                                TOPIC_COMPONENT_TYPE);
    }
    
    /**
     * Common code for the testConfigure*UnsetPropertiesUsingExistingServiceFile 
     * tests.
     */
    private void unsetPropertiesUsingExistingServiceFile(String jndiName, 
                                                         DestinationType destinationType,
                                                         ComponentType componentType) throws Exception {
        Map<String, MetaValue> propertiesMap = getSpecificComponentProperties(jndiName, 
                                                                              componentType);
        
        String[] propertiesToUnset = new String[] { "maxDeliveryAttempts" }; 
        propertiesMap.remove("maxDeliveryAttempts");
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        unsetDestinationProperties(destinationType, jndiName, propertiesMap, 
                                   propertiesToUnset, componentType);
    }
    
    /**
     * Test Name: testConfigureQueueUnsetProperties
     * Assertion: Verify the ability to unset queue property values. 
     * 
     * Strategy: Create a new queue. Unset some property values for the 
     * queue. Make sure the rest of the properties for the ManagedComponent 
     * remain unchanged.
     */
    public void testConfigureQueueUnsetProperties() throws Exception {
        String jndiName = "UnsetPropertiesQueue";
        
        // Create the queue first
        Map<String, MetaValue> propertiesMap = createQueue(jndiName);
        
        String[] propertiesToUnset = new String[] { "maxSize" }; 
        propertiesMap.remove("maxSize"); 
        
        unsetDestinationProperties(DestinationType.QUEUE, jndiName, propertiesMap, 
                                   propertiesToUnset, QUEUE_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testConfigureTopicUnsetProperties
     * Assertion: Verify the ability to unset topic property values. 
     * 
     * Strategy: Create a new topic. Unset some property values for the 
     * topic. Make sure the rest of the properties for the ManagedComponent 
     * remain unchanged.
     */
    public void testConfigureTopicUnsetProperties() throws Exception {
        String jndiName = "UnsetPropertiesTopic";
        
        // Create the topic first
        Map<String, MetaValue> propertiesMap = createTopic(jndiName);
        
        String[] propertiesToUnset = new String[] { "maxSize" }; 
        propertiesMap.remove("maxSize"); 
        
        unsetDestinationProperties(DestinationType.TOPIC, jndiName, propertiesMap, 
                                   propertiesToUnset, TOPIC_COMPONENT_TYPE);
    }
    
    /**
     * Common code for the testConfigure*UnsetProperties* tests.
     */
    private void unsetDestinationProperties(DestinationType destinationType,
                                            String jndiName,
                                            Map<String, MetaValue> propertiesMap,
                                            String[] propertiesToUnset,
                                            ComponentType componentType) throws Exception {
        
        // The success message we should see
        String expectedMessage = UPDATE_MESSAGE + destinationType.getName() + " '" 
                                 + jndiName + "'";
        
        unsetResourceProperties(JMS_NAV_LABEL, destinationType.getNavLabel(), 
                                jndiName, propertiesMap, propertiesToUnset, 
                                componentType, expectedMessage);
        
        assertTrue("The destination is not deployed ",
                   isDeployed(jndiName + "-service.xml"));
     
        // Clean up
        deleteDestination(destinationType, jndiName);   
    }
}
