/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.jmsDestinations;

import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import junit.framework.Test;
import java.util.Map;
import java.util.HashMap;
import java.util.LinkedHashMap;
import org.jboss.jopr.jsfunit.*;
import javax.management.ObjectName;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.metatype.api.values.SimpleValueSupport;
import org.jboss.metatype.api.values.MetaValue;
import org.jboss.managed.api.ComponentType;

/**
 * This class contains creation tests for JMS topics and queues.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class JMSCreationTest extends JMSTest {
    
    /**
     * Test Name: testCreateQueue
     * Assertion: Verify the ability to create a new queue.
     * 
     * Strategy: Create a queue. Make sure the queue is deployed and that
     * the properties of the ManagedComponent are set correctly.
     */
    public void testCreateQueue() throws Exception {
        
        String jndiName = "CreateQueue";
        String expectedMessage = ADD_MESSAGE + DestinationType.QUEUE.getName();
        
        createDestinationCommon(DestinationType.QUEUE, QUEUE_DEFAULT_TEMPLATE, 
                                jndiName, expectedMessage, QUEUE_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testCreateTopic
     * Assertion: Verify the ability to create a new topic.
     * 
     * Strategy: Create a topic. Make sure the topic is deployed and that
     * the properties of the ManagedComponent are set correctly.
     */
    public void testCreateTopic() throws Exception {
        
        String jndiName = "CreateTopic";
        String expectedMessage = ADD_MESSAGE + DestinationType.TOPIC.getName();
        
        createDestinationCommon(DestinationType.TOPIC, TOPIC_DEFAULT_TEMPLATE, 
                                jndiName, expectedMessage, TOPIC_COMPONENT_TYPE);
    }
    
    /**
     * Common code for the testCreateQueue() and testCreateTopic() tests.
     */
    private void createDestinationCommon(DestinationType destinationType, 
                                         String destinationTemplate, 
                                         String jndiName, 
                                         String expectedMessage,
                                         ComponentType componentType) throws Exception {
        
        // The properties we want to configure
        Map<String, MetaValue> propertiesMap = new LinkedHashMap<String, MetaValue>();
        propertiesMap.put("name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("JNDIName", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("clustered", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("downCacheSize", SimpleValueSupport.wrap(new Integer(2500)));
        propertiesMap.put("fullSize", SimpleValueSupport.wrap(new Integer(80000)));
        propertiesMap.put("maxDeliveryAttempts", SimpleValueSupport.wrap(new Integer(15)));
        propertiesMap.put("maxSize", SimpleValueSupport.wrap(new Integer(80000)));
        /** propertiesMap.put("messageCounterHistoryDayLimit", SimpleValueSupport.wrap(new Integer(0))); **/
        propertiesMap.put("pageSize", SimpleValueSupport.wrap(new Integer(2500)));
        propertiesMap.put("redeliveryDelay", SimpleValueSupport.wrap(new Long(50000)));

        createDestination(destinationType, destinationTemplate, propertiesMap);
        client.click(SAVE_BUTTON);

        // Check for the appropriate success messages
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);

        // Make sure the ManagedComponent was created and that the properties are
        // set correctly
        assertTrue("The destination is not deployed ",
                   isDeployed(jndiName + "-service.xml"));
        checkComponentProperties(propertiesMap, jndiName, componentType);
        
        // Clean up
        deleteDestination(destinationType, jndiName);
    }
    
    /**
     * Test Name: testCreateQueueSetObjectNameManagedProperties
     * Assertion: Verify the ability to specify ObjectName ManagedProperties
     * (eg. DLQ, expiryQueue, serverPeer) when creating a new queue.
     * (See JOPR-121)
     */
    public void testCreateQueueSetObjectNameManagedProperties() throws Exception {
        String jndiName = "ObjectNameManagedPropertiesQueue";
        String expectedMessage = ADD_MESSAGE + DestinationType.QUEUE.getName();
        
        createDestinationWithObjectNameManagedProperties(DestinationType.QUEUE, QUEUE_DEFAULT_TEMPLATE, 
                                                         jndiName, expectedMessage, QUEUE_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testCreateTopicSetObjectNameManagedProperties
     * Assertion: Verify the ability to specify ObjectName ManagedProperties
     * (eg. DLQ, expiryQueue, serverPeer) when creating a new topic.
     * (See JOPR-121)
     */
    public void testCreateTopicSetObjectNameManagedProperties() throws Exception {
        
        String jndiName = "ObjectNameManagedPropertiesTopic";
        String expectedMessage = ADD_MESSAGE + DestinationType.TOPIC.getName();
        
        createDestinationWithObjectNameManagedProperties(DestinationType.TOPIC, TOPIC_DEFAULT_TEMPLATE, 
                                                         jndiName, expectedMessage, TOPIC_COMPONENT_TYPE);
    }
    
    /**
     * Common code for the testCreateQueueSetObjectNameManagedProperties() and 
     * testCreateTopicSetObjectNameManagedProperties() tests.
     */
    private void createDestinationWithObjectNameManagedProperties(DestinationType destinationType, 
                                                                  String destinationTemplate, 
                                                                  String jndiName, 
                                                                  String expectedMessage,
                                                                  ComponentType componentType) throws Exception {
        
        // The properties we want to configure
        Map<String, MetaValue> propertiesMap = new LinkedHashMap<String, MetaValue>();
        propertiesMap.put("name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("JNDIName", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("DLQ", SimpleValueSupport.wrap(DLQ));
        propertiesMap.put("expiryQueue", SimpleValueSupport.wrap(EXPIRY_QUEUE));
        propertiesMap.put("serverPeer", SimpleValueSupport.wrap(SERVER_PEER));

        createDestination(destinationType, destinationTemplate, propertiesMap);
        client.click(SAVE_BUTTON);

        // Check for the appropriate success messages
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);

        // Make sure the ManagedComponent was created and that the properties are
        // set correctly
        assertTrue("The destination is not deployed ",
                   isDeployed(jndiName + "-service.xml"));
        checkComponentProperties(propertiesMap, jndiName, componentType);
        
        // Clean up
        deleteDestination(destinationType, jndiName);
    }
    
     /**
     * Test Name: testCreateQueueMissingRequiredValue
     * Assertion: Verify the ability to handle a missing required value when
     * creating a queue. Make sure the appropriate error message occurs.
     */
    public void testCreateQueueMissingRequiredValue() throws IOException, EmbJoprTestException {
        createDestinationMissingRequiredValue(DestinationType.QUEUE, QUEUE_DEFAULT_TEMPLATE);
    }
    
    /**
     * Test Name: testCreateTopicMissingRequiredValue
     * Assertion: Verify the ability to handle a missing required value when
     * creating a topic. Make sure the appropriate error message occurs.
     */
    public void testCreateTopicMissingRequiredValue() throws IOException, EmbJoprTestException {
        createDestinationMissingRequiredValue(DestinationType.TOPIC, TOPIC_DEFAULT_TEMPLATE);
    }
    
    /**
     * Common code for the testCreate*MissingRequiredValue() tests.
     */
    private void createDestinationMissingRequiredValue(DestinationType destinationType,
                                                       String destinationTemplate) throws IOException, EmbJoprTestException {
        // Leave the JNDI name and name unset
        Map<String, MetaValue> propertiesMap = new LinkedHashMap<String, MetaValue>();
        propertiesMap.put("clustered", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("fullSize", SimpleValueSupport.wrap(new Integer(80000)));
        
        createDestination(destinationType, destinationTemplate, propertiesMap);
        client.click(SAVE_BUTTON);
        
        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE, 
                                     MISSING_VALUE_MESSAGE, true);
    }
    
    /**
     * Test Name: testCreateQueuePropertyOutOfRange
     * Assertion: Verify the ability to handle a property value beyond its
     * expected range of values when creating a queue. Make sure the appropriate 
     * error message occurs.
     */
    public void testCreateQueuePropertyOutOfRange() throws IOException, EmbJoprTestException {
        createDestinationPropertyOutOfRange(DestinationType.QUEUE, 
                                            QUEUE_DEFAULT_TEMPLATE, 
                                            "PropertyOutOfRangeQueue");
    }
    
    /**
     * Test Name: testCreateTopicPropertyOutOfRange
     * Assertion: Verify the ability to handle a property value beyond its
     * expected range of values when creating a topic. Make sure the appropriate 
     * error message occurs.
     */
    public void testCreateTopicPropertyOutOfRange() throws IOException, EmbJoprTestException {
        createDestinationPropertyOutOfRange(DestinationType.TOPIC, 
                                            TOPIC_DEFAULT_TEMPLATE, 
                                            "PropertyOutOfRangeTopic");
    }

    
    /**
     * Common code for the testCreate*PropertyOutOfRange() tests.
     */
    private void createDestinationPropertyOutOfRange(DestinationType destinationType,
                                                     String destinationTemplate,
                                                     String jndiName) throws IOException, EmbJoprTestException {
        // The properties we want to configure
        Map<String, MetaValue> propertiesMap = new LinkedHashMap<String, MetaValue>();
        propertiesMap.put("name", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("JNDIName", SimpleValueSupport.wrap(jndiName));
        propertiesMap.put("clustered", SimpleValueSupport.wrap(Boolean.TRUE));
        propertiesMap.put("downCacheSize", SimpleValueSupport.wrap(new Integer(2500)));
        propertiesMap.put("fullSize", SimpleValueSupport.wrap(new Integer(80000)));
        
        // This number is too large
        propertiesMap.put("maxDeliveryAttempts", SimpleValueSupport.wrap(new Long(Long.MAX_VALUE)));
        
        createDestination(destinationType, destinationTemplate, propertiesMap);
        client.click(SAVE_BUTTON);
   
        // Check for the appropriate error messages
        checkClientAndServerMessages(INVALID_VALUE_MESSAGE, 
                                     OUT_OF_RANGE_MESSAGE, true);
    }
    
    /**
     * Test Name: testCreateQueueInvalidPropertyType
     * Assertion: Verify the ability to handle a property value that has an
     * invalid type when creating a queue. Make sure the appropriate 
     * error message occurs.
     */
     public void testCreateQueueInvalidPropertyType() throws IOException, EmbJoprTestException {
         createDestinationInvalidPropertyType(DestinationType.QUEUE, 
                                              QUEUE_DEFAULT_TEMPLATE, 
                                              "InvalidPropertyTypeQueue");
     }
     
    /**
     * Test Name: testCreateTopicInvalidPropertyType
     * Assertion: Verify the ability to handle a property value that has an
     * invalid type when creating a topic. Make sure the appropriate 
     * error message occurs.
     */
     public void testCreateTopicInvalidPropertyType() throws IOException, EmbJoprTestException {
          createDestinationInvalidPropertyType(DestinationType.TOPIC, 
                                               TOPIC_DEFAULT_TEMPLATE, 
                                               "InvalidPropertyTypeTopic");
     }
     
     /**
      * Common code for the testCreate*InvalidPropertyType() tests.
      */ 
     private void createDestinationInvalidPropertyType(DestinationType destinationType,
                                                       String destinationTemplate,
                                                       String jndiName) throws IOException, EmbJoprTestException {
         // The properties we want to configure
         Map<String, MetaValue> propertiesMap = new LinkedHashMap<String, MetaValue>();
         propertiesMap.put("name", SimpleValueSupport.wrap(jndiName));
         propertiesMap.put("JNDIName", SimpleValueSupport.wrap(jndiName));
         propertiesMap.put("clustered", SimpleValueSupport.wrap(Boolean.TRUE));
         /** propertiesMap.put("messageCounterHistoryDayLimit", SimpleValueSupport.wrap(new Integer(0))); **/
         propertiesMap.put("redeliveryDelay", SimpleValueSupport.wrap(new Long(50000)));
         propertiesMap.put("downCacheSize", SimpleValueSupport.wrap(new Integer(1500)));
         
         // This property value is supposed to be an integer
         propertiesMap.put("maxDeliveryAttempts", SimpleValueSupport.wrap("fifteen"));
         
         createDestination(destinationType, destinationTemplate, propertiesMap);
         client.click(SAVE_BUTTON);
         
         // Check for the appropriate error messages
         checkClientAndServerMessages(INVALID_VALUE_MESSAGE,
                                      INVALID_INTEGER_MESSAGE, true);
     }
     
     /**
      * Test Name: testCreateQueueDuplicateJNDIName
      * Assertion: Verify the ability to handle a duplicate JNDI name 
      * when creating a queue. Make sure the appropriate error message 
      * occurs.
      */
     public void testCreateQueueDuplicateJNDIName() throws IOException, EmbJoprTestException {
         String jndiName = "TestQueue";
         createQueue(jndiName);
         
         checkDestinationDuplicateJNDIName(DestinationType.QUEUE, jndiName);

     }
     
     /**
      * Test Name: testCreateTopicDuplicateJNDIName
      * Assertion: Verify the ability to handle a duplicate JNDI name 
      * when creating a queue. Make sure the appropriate error message 
      * occurs.
      */
     public void testCreateTopicDuplicateJNDIName() throws IOException, EmbJoprTestException {
         String jndiName = "TestTopic";
         createTopic(jndiName);
         
         checkDestinationDuplicateJNDIName(DestinationType.TOPIC, jndiName);
     }
     
     /**
      * Common code for the testCreate*DuplicateJNDIName() tests.
      */
     public void checkDestinationDuplicateJNDIName(DestinationType destinationType,
                                                   String jndiName) throws IOException, EmbJoprTestException {

         // Check for the appropriate error messages
         String expectedMessage = "A " + destinationType.getName() + " named '" + jndiName + "' already exists";
         checkClientAndServerMessages(expectedMessage, expectedMessage, true);
     }
}
