/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.jmsDestinations;

import com.gargoylesoftware.htmlunit.html.*;
import junit.framework.Test;
import java.util.Map;
import java.util.HashMap;
import java.util.LinkedHashMap;
import org.jboss.jopr.jsfunit.*;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.metatype.api.values.MetaValue;
import org.jboss.managed.api.ComponentType;

/**
 * This class contains deletion tests for JMS topics and queues.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class JMSDeletionTest extends JMSTest {
    
    /**
     * Test Name: testDeleteQueueUsingExistingServiceFile
     * Assertion: Verify the ability to delete a queue. Use a queue
     * that already exists.
     * 
     * Strategy: Delete the queue. Make sure the ManagedComponent is removed.
     */
    public void testDeleteQueueUsingExistingServiceFile() throws Exception {
        String jndiName = "DeleteQueueExistingFile";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        deleteDestinationCommon(jndiName, DestinationType.QUEUE, QUEUE_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testDeleteQueue
     * Assertion: Verify the ability to delete a queue. 
     * 
     * Strategy: Create a queue and then delete it. Make sure the ManagedComponent 
     * is removed.
     */
    public void testDeleteQueue() throws Exception {
        String jndiName = "QueueDelete";
        
        // Create the queue first
        Map<String, MetaValue> propertiesMap = createQueue(jndiName);
        deleteDestinationCommon(jndiName, DestinationType.QUEUE, QUEUE_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testDeleteTopicUsingExistingServiceFile
     * Assertion: Verify the ability to delete a topic. Use a topic
     * that already exists.
     * 
     * Strategy: Delete the topic. Make sure the ManagedComponent is removed.
     */
    public void testDeleteTopicUsingExistingServiceFile() throws Exception {
        String jndiName = "DeleteTopicExistingFile";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        deleteDestinationCommon(jndiName, DestinationType.TOPIC, TOPIC_COMPONENT_TYPE);
    }
    
    /**
     * Test Name: testDeleteTopic
     * Assertion: Verify the ability to delete a topic. 
     * 
     * Strategy: Create a topic and then delete it. Make sure the ManagedComponent 
     * is removed.
     */
    public void testDeleteTopic() throws Exception {
        String jndiName = "TopicDelete";
        
        // Create the topic first
        Map<String, MetaValue> propertiesMap = createTopic(jndiName);
        deleteDestinationCommon(jndiName, DestinationType.TOPIC, TOPIC_COMPONENT_TYPE);
    }
    
    /**
     * Common code for the testDelete* tests.
     */
    public void deleteDestinationCommon(String jndiName, 
                                        DestinationType destinationType, 
                                        ComponentType componentType) throws Exception {
        deleteDestination(destinationType, jndiName);
        
        // Check for the appropriate success messages
        String expectedMessage = DELETE_MESSAGE + destinationType.getName() + " '" + jndiName + "'";
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);
        
        // Make sure the ManagedComponent was removed
        assertNull("The destination was not removed after deletion (the ManagedComponent is non-null)",
                   getManagedComponent(jndiName, componentType));
        assertTrue("The entry was removed from the -service.xml file after deletion",
                    containsElement(getDestinationDeploymentFile(jndiName), 
                                    SERVICE_FILE_CHILD_ELEMENT, 
                                    SERVICE_FILE_ROOT_ELEMENT));
    }
}
