/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.jmsDestinations;

import java.util.Map;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import javax.jms.Topic;
import javax.jms.TopicSession;
import javax.jms.Queue;
import javax.jms.QueueSession;
import javax.jms.QueueReceiver;
import javax.naming.InitialContext;
import javax.jms.TopicConnectionFactory;

/**
 * This class contains metrics tests for JMS topics and queues.  
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class JMSMetricsTest extends JMSTest {
    
    /**
     * Test Name: testTopicMetricsAfterCreation
     * Assertion: Verify that topic metrics are correct after topic
     * creation. 
     */
    public void testTopicMetricsAfterCreation() throws Exception {
        String jndiName = "MetricsAfterTopicCreation";
        
        // Create the topic first
        createTopic(jndiName);
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(ALL_MSG_COUNT, "0");
        expectedMetrics.put(ALL_SUB_COUNT, "0");
        expectedMetrics.put(DURABLE_MSG_COUNT, "0");
        expectedMetrics.put(DURABLE_SUB_COUNT, "0");
        expectedMetrics.put(NON_DURABLE_MSG_COUNT, "0");
        expectedMetrics.put(NON_DURABLE_SUB_COUNT, "0");
          
        checkDestinationMetrics(jndiName, expectedMetrics, getTopicSummaryMetrics(),
                                DestinationType.TOPIC);
    }
    
    /**
     * Test Name: testTopicMetricsAfterOneNonDurableSubscription
     * Assertion: Verify that topic metrics are correct after creating
     * a non-durable subscriber for the topic.
     */
    public void testTopicMetricsAfterOneNonDurableSubscription() throws Exception {
        String jndiName = "MetricsAfterOneNonDurableTopicSubscriber";
        
        // Create the topic first
        createTopic(jndiName);
        
        checkTopicMetricsAfterOneSubscription(jndiName, Boolean.FALSE);
    }
    
    /**
     * Test Name: testTopicMetricsAfterOneNonDurableSubscriptionUsingExistingServiceFile
     * Assertion: Verify that topic metrics are correct after creating
     * a non-durable subscriber for the topic. Use a topic that already exists.
     */
    public void testTopicMetricsAfterOneNonDurableSubscriptionUsingExistingServiceFile() throws Exception {
        String jndiName = "MetricsOneNonDurableTopicSubscriberExistingTopic";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkTopicMetricsAfterOneSubscription(jndiName, Boolean.FALSE);
    }
    
    /**
     * Test Name: testTopicMetricsAfterOneDurableSubscription
     * Assertion: Verify that topic metrics are correct after creating
     * a durable subscriber for the topic.
     */
    public void testTopicMetricsAfterOneDurableSubscription() throws Exception {
        String jndiName = "MetricsAfterOneDurableTopicSubscriber";
        
        // Create the topic first
        createTopic(jndiName);
        
        checkTopicMetricsAfterOneSubscription(jndiName, Boolean.TRUE);
    }
    
    /**
     * Test Name: testTopicMetricsAfterOneDurableSubscriptionUsingExistingServiceFile
     * Assertion: Verify that topic metrics are correct after creating
     * a durable subscriber for the topic. Use a topic that already exists.
     */
    public void testTopicMetricsAfterOneDurableSubscriptionUsingExistingServiceFile() throws Exception {
        String jndiName = "MetricsOneDurableTopicSubscriberExistingTopic";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkTopicMetricsAfterOneSubscription(jndiName, Boolean.TRUE);
    }
    
    /**
     * Common code for the testTopicMetricsAfterOne*Subscription* tests.
     */
    private void checkTopicMetricsAfterOneSubscription(String jndiName, 
            Boolean isDurable) throws Exception {

        // Create a subscriber
        TopicSession session = createTopicSession();
        Topic topic = getTopic(jndiName);

        if(isDurable) {
            createDurableTopicSubscriber(session, topic, jndiName + "Subscriber1");
        } else {
            createNonDurableTopicSubscriber(session, topic);
        }

        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(ALL_MSG_COUNT, "0");
        expectedMetrics.put(ALL_SUB_COUNT, "1");
        expectedMetrics.put(DURABLE_MSG_COUNT, "0");
        expectedMetrics.put(NON_DURABLE_MSG_COUNT, "0");

        if(isDurable) {
            expectedMetrics.put(DURABLE_SUB_COUNT, "1");
            expectedMetrics.put(NON_DURABLE_SUB_COUNT, "0");
        } else {
            expectedMetrics.put(DURABLE_SUB_COUNT, "0");
            expectedMetrics.put(NON_DURABLE_SUB_COUNT, "1");
        }

        checkDestinationMetrics(jndiName, expectedMetrics, getTopicSummaryMetrics(),
                                DestinationType.TOPIC);
    }
    
    /**
     * Test Name: testTopicMetricsAfterMultipleSubscriptions()
     * Assertion: Verify that topic metrics are correct after multiple subscribers
     * for the topic are created.
     */
    public void testTopicMetricsAfterMultipleSubscriptions() throws Exception {
        String jndiName = "MetricsAfterMultipleTopicSubscribers";
        
        // Create the topic first
        createTopic(jndiName);
        
        checkTopicMetricsAfterMultipleSubscriptions(jndiName);
    }
    
    /**
     * Test Name: testTopicMetricsAfterMultipleSubscriptionsUsingExistingServiceFile()
     * Assertion: Verify that topic metrics are correct after multiple subscribers
     * for the topic are created. Use a topic that already exists.
     */
    public void testTopicMetricsAfterMultipleSubscriptionsUsingExistingServiceFile() throws Exception {
        String jndiName = "MetricsMultipleTopicSubscribersExistingTopic";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkTopicMetricsAfterMultipleSubscriptions(jndiName); 
    }
    
    /**
     * Common code for the testTopicMetricsAfterMultipleSubscriptions* tests.
     */
    private void checkTopicMetricsAfterMultipleSubscriptions(String jndiName) throws Exception {
        
        TopicSession session = createTopicSession();
        Topic topic = getTopic(jndiName);

        // Create multiple subscriptions (2 durable ones, 2 non-durable ones)
        for(int i = 0; i < 2; i++) {
            createDurableTopicSubscriber(session, topic, jndiName + "Subscriber" + i);
            createNonDurableTopicSubscriber(session, topic);
        }
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(ALL_MSG_COUNT, "0");
        expectedMetrics.put(ALL_SUB_COUNT, "4");
        expectedMetrics.put(DURABLE_MSG_COUNT, "0");
        expectedMetrics.put(NON_DURABLE_MSG_COUNT, "0");
        expectedMetrics.put(DURABLE_SUB_COUNT, "2");
        expectedMetrics.put(NON_DURABLE_SUB_COUNT, "2");
         
        checkDestinationMetrics(jndiName, expectedMetrics, getTopicSummaryMetrics(),
                                DestinationType.TOPIC);
    }
    
    /**
     * Test Name: testTopicMetricsAfterOneMessage()
     * Assertion: Verify that topic metrics are correct after sending
     * one message to the topic.
     */
    public void testTopicMetricsAfterOneMessage() throws Exception {
        String jndiName = "TopicMetricsAfterOneMessage";
        
        // Create the topic first
        createTopic(jndiName);
        
        checkTopicMetricsAfterOneMessage(jndiName);
    }
    
    /**
     * Test Name: testTopicMetricsAfterOneMessageUsingExistingServiceFile()
     * Assertion: Verify that topic metrics are correct after sending
     * one message to the topic. Use a topic that already exists.
     */
    public void testTopicMetricsAfterOneMessageUsingExistingServiceFile() throws Exception {
        String jndiName = "TopicMetricsOneMessageExistingTopic";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkTopicMetricsAfterOneMessage(jndiName);
    }
    
    /**
     * Common code for the testTopicMetricsAfterOneMessage* tests.
     */
    private void checkTopicMetricsAfterOneMessage(String jndiName) throws Exception {
        
        // Create a subscriber for the topic and then publish
        // 1 message
        TopicSession session = createTopicSession();
        Topic topic = getTopic(jndiName);
        createDurableTopicSubscriber(session, topic, jndiName + "Subscriber1");
        
        publishMessages(session, topic, 1);
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(ALL_MSG_COUNT, "1");
        expectedMetrics.put(ALL_SUB_COUNT, "1");
        expectedMetrics.put(DURABLE_MSG_COUNT, "1");
        expectedMetrics.put(NON_DURABLE_MSG_COUNT, "0");
        expectedMetrics.put(DURABLE_SUB_COUNT, "1");
        expectedMetrics.put(NON_DURABLE_SUB_COUNT, "0");
         
        checkDestinationMetrics(jndiName, expectedMetrics, getTopicSummaryMetrics(),
                                DestinationType.TOPIC);
        
        
    }
    
    /**
     * Test Name: testTopicMetricsAfterMultipleMessages()
     * Assertion: Verify that topic metrics are correct after sending
     * multiple message to the topic.
     */
    public void testTopicMetricsAfterMultipleMessages() throws Exception {
        String jndiName = "TopicMetricsAfterMultipleMessages";
            
        // Create the topic first
        createTopic(jndiName);
        
        checkTopicMetricsAfterMultipleMessages(jndiName);
    }
    
    /**
     * Test Name: testTopicMetricsAfterMultipleMessagesUsingExistingServiceFile()
     * Assertion: Verify that topic metrics are correct after sending
     * multiple message to the topic. Use a topic that already exists.
     */
    public void testTopicMetricsAfterMultipleMessagesUsingExistingServiceFile() throws Exception {
        String jndiName = "TopicMetricsMultipleMessagesExistingTopic";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkTopicMetricsAfterMultipleMessages(jndiName);
    }
    
    /**
     * Common code for the testTopicMetricsAfterMultipleMessages* tests.
     */
    private void checkTopicMetricsAfterMultipleMessages(String jndiName) throws Exception {
        
        InitialContext context = new InitialContext();
        Object tmp = context.lookup(TOPIC_FACTORY);
        
        TopicConnectionFactory tcf = (TopicConnectionFactory)tmp;
        topicConnection = tcf.createTopicConnection();
        topicConnection.setClientID(CLIENT_ID);
        
        TopicSession session = topicConnection.createTopicSession(Boolean.FALSE, 
                                                                  TopicSession.AUTO_ACKNOWLEDGE);
        Topic topic = getTopic(jndiName);
        
        // Create 1 durable subscriber, 1 non-durable subscriber
        // and then publish 3 messages
        createDurableTopicSubscriber(session, topic, jndiName + "Subscriber");
        createNonDurableTopicSubscriber(session, topic);
        
        publishMessages(session, topic, 3);
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(ALL_MSG_COUNT, "6");
        expectedMetrics.put(ALL_SUB_COUNT, "2");
        expectedMetrics.put(DURABLE_MSG_COUNT, "3");
        expectedMetrics.put(NON_DURABLE_MSG_COUNT, "3");
        expectedMetrics.put(DURABLE_SUB_COUNT, "1");
        expectedMetrics.put(NON_DURABLE_SUB_COUNT, "1");
         
        checkDestinationMetrics(jndiName, expectedMetrics, getTopicSummaryMetrics(),
                                DestinationType.TOPIC);
    }
    
    /**
     * Test Name: testQueueMetricsAfterCreation
     * Assertion: Verify that queue metrics are correct after queue
     * creation. 
     */
    public void testQueueMetricsAfterCreation() throws Exception {
        String jndiName = "MetricsAfterQueueCreation";
        
        // Create the queue first
        createQueue(jndiName);
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(CONSUMER_COUNT, "0");
        expectedMetrics.put(DELIVERING_COUNT, "0");
        expectedMetrics.put(MSG_COUNT, "0");
        expectedMetrics.put(SCHEDULED_MSG_COUNT, "0");
        expectedMetrics.put(COUNT, "0");
        //expectedMetrics.put(COUNT_DELTA, "0");
        expectedMetrics.put(DEPTH, "0");
        //expectedMetrics.put(DEPTH_DELTA, "0");
        //expectedMetrics.put(TIME_LAST_UPDATE, "0");
          
        checkDestinationMetrics(jndiName, expectedMetrics, getQueueSummaryMetrics(),
                                DestinationType.QUEUE);
    }
   
    /**
     * Test Name: testQueueMetricsAfterCreatingReceiver
     * Assertion: Verify that queue metrics are correct after creating
     * a queue receiver.
     */
    public void testQueueMetricsAfterCreatingReceiver() throws Exception {
        String jndiName = "MetricsAfterQueueReceiver";
        
        // Create the queue first
        createQueue(jndiName);
        
        checkQueueMetricsAfterCreatingReceiver(jndiName);
    }
    
    /**
     * Test Name: testQueueMetricsAfterCreatingReceiverUsingExistingServiceFile
     * Assertion: Verify that queue metrics are correct after creating
     * a queue receiver. Use a queue that already exists.
     */
    public void testQueueMetricsAfterCreatingReceiverUsingExistingServiceFile() throws Exception {
        String jndiName = "MetricsOneQueueReceiverExistingQueue";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkQueueMetricsAfterCreatingReceiver(jndiName);
    }
    
    /**
     * Common code for the testQueueMetricsAfterCreatingReceiver* tests.
     */
    private void checkQueueMetricsAfterCreatingReceiver(String jndiName) throws Exception {
        
        // Create a receiver
        QueueSession session = createQueueSession();
        Queue queue = getQueue(jndiName);
        createQueueReceiver(session, queue);
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(CONSUMER_COUNT, "1");
        expectedMetrics.put(DELIVERING_COUNT, "0");
        expectedMetrics.put(MSG_COUNT, "0");
        expectedMetrics.put(SCHEDULED_MSG_COUNT, "0");
        expectedMetrics.put(COUNT, "0");
        //expectedMetrics.put(COUNT_DELTA, "0");
        expectedMetrics.put(DEPTH, "0");
        //expectedMetrics.put(DEPTH_DELTA, "0");
        //expectedMetrics.put(TIME_LAST_UPDATE, "0"); // This currently changes
				//(09:27:08) ozizka: There's a metric, Time Last Update, which has a description: The timestamp of the last message add
				//(09:27:31) ozizka: But it changes upon queue receiver creation... so, is the description wrong or the behavior?
				
        checkDestinationMetrics(jndiName, expectedMetrics, getQueueSummaryMetrics(),
                                DestinationType.QUEUE);
    }
    
    /**
     * Test Name: testQueueMetricsAfterOneMessage
     * Assertion: Verify that queue metrics are correct after sending
     * a message to the queue.
     */
    public void testQueueMetricsAfterOneMessage() throws Exception {
        String jndiName = "QueueMetricsAfterOneMessage";
        
       // Create the queue first
       createQueue(jndiName);
      
       checkQueueMetricsAfterOneMessage(jndiName);
    }
    
    /**
     * Test Name: testQueueMetricsAfterOneMessageUsingExistingServiceFile
     * Assertion: Verify that queue metrics are correct after sending
     * a message to the queue. Use a queue that already exists.
     */
    public void testQueueMetricsAfterOneMessageUsingExistingServiceFile() throws Exception {
        String jndiName = "QueueMetricsOneMessageExistingQueue";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkQueueMetricsAfterOneMessage(jndiName);
    }
    
    /**
     * Common code for the testQueueMetricsAfterOneMessage* tests.
     */
    private void checkQueueMetricsAfterOneMessage(String jndiName) throws Exception {
        
        // Send 1 message to the queue
        createQueueSessionAndSendMessages(1, jndiName);
           
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(CONSUMER_COUNT, "0");
        expectedMetrics.put(DELIVERING_COUNT, "0");
        expectedMetrics.put(MSG_COUNT, "1");
        expectedMetrics.put(SCHEDULED_MSG_COUNT, "0");
        expectedMetrics.put(COUNT, "1");
        //expectedMetrics.put(COUNT_DELTA, "1");
        expectedMetrics.put(DEPTH, "1");
        //expectedMetrics.put(DEPTH_DELTA, "1");
        
        checkDestinationMetrics(jndiName, expectedMetrics, getQueueSummaryMetrics(),
                                DestinationType.QUEUE);
    }
    
    /**
     * Test Name: testQueueMetricsAfterMulipleMessages
     * Assertion: Verify that queue metrics are correct after sending
     * multiple messages to the queue.
     */
    public void testQueueMetricsAfterMultipleMessages() throws Exception {
        String jndiName = "QueueMetricsAfterMultipleMessages";
        
        // Create the queue first
        createQueue(jndiName);
      
        checkQueueMetricsAfterMultipleMessages(jndiName);
    }
    
    /**
     * Test Name: testQueueMetricsAfterMulipleMessagesUsingExistingServiceFile
     * Assertion: Verify that queue metrics are correct after sending
     * multiple messages to the queue. Use a queue that already exists.
     */
    public void testQueueMetricsAfterMultipleMessagesUsingExistingServiceFile() throws Exception {
        String jndiName = "QueueMetricsMultipleMessagesExistingQueue";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkQueueMetricsAfterMultipleMessages(jndiName);
    }
    
    /**
     * Common code for the testQueueMetricsAfterMultipleMessages* tests.
     */
    private void checkQueueMetricsAfterMultipleMessages(String jndiName) throws Exception {
        
        // Send multiple messages to the queue
        createQueueSessionAndSendMessages(3, jndiName);
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(CONSUMER_COUNT, "0");
        expectedMetrics.put(DELIVERING_COUNT, "0");
        expectedMetrics.put(MSG_COUNT, "3");
        expectedMetrics.put(SCHEDULED_MSG_COUNT, "0");
        expectedMetrics.put(COUNT, "3");
        //expectedMetrics.put(COUNT_DELTA, "3");
        expectedMetrics.put(DEPTH, "3");
        //expectedMetrics.put(DEPTH_DELTA, "3");
        
        checkDestinationMetrics(jndiName, expectedMetrics, getQueueSummaryMetrics(),
                                DestinationType.QUEUE);
    }
    
    /**
     * Test Name: testQueueMetricsAfterReceivingMessage
     * Assertion: Verify that queue metrics are correct 
     * after a queue receiver actually receives a message.
     */
    public void testQueueMetricsAfterReceivingMessage() throws Exception {
        String jndiName = "QueueMetricsAfterReceivingMessage";
        
        // Create the queue first
        createQueue(jndiName);
      
        checkQueueMetricsAfterReceivingMessage(jndiName);
    }
    
    /**
     * Test Name: testQueueMetricsAfterReceivingMessageUsingExistingServiceFile
     * Assertion: Verify that queue metrics are correct 
     * after a queue receiver actually receives a message. Use a queue
     * that already exists.
     */
    public void testQueueMetricsAfterReceivingMessageUsingExistingServiceFile() throws Exception {
        String jndiName = "QueueMetricsReceiveMessageExistingQueue";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        checkQueueMetricsAfterReceivingMessage(jndiName);
    }
    
    /**
     * Common code for the testQueueMetricsAfterReceivingMessage* tests.
     */
    private void checkQueueMetricsAfterReceivingMessage(String jndiName) throws Exception {
        
        // Create a queue receiver and then send multiple messages to the
        // queue
        QueueSession session = createQueueSession();
        Queue queue = getQueue(jndiName);
        QueueReceiver receiver = createQueueReceiver(session, queue);
        sendMessages(session, queue, 3);
        
        // Actually receive a message
        receiver.receive();
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(CONSUMER_COUNT, "1");
        expectedMetrics.put(DELIVERING_COUNT, "2");
        expectedMetrics.put(MSG_COUNT, "2");
        expectedMetrics.put(SCHEDULED_MSG_COUNT, "0");
        expectedMetrics.put(COUNT, "3");
        //expectedMetrics.put(COUNT_DELTA, "3");
        expectedMetrics.put(DEPTH, "2");
        //expectedMetrics.put(DEPTH_DELTA, "2");
        
        checkDestinationMetrics(jndiName, expectedMetrics, getQueueSummaryMetrics(),
                                DestinationType.QUEUE);
    }
    
    /**
     * Test Name: testQueueMetricsAfterScheduledMessages
     * Assertion: Verify that queue metrics are correct after creating
     * some scheduled messages for the queue.
     */
    public void testQueueMetricsAfterScheduledMessages() throws Exception {
        String jndiName = "QueueMetricsAfterScheduledMessages";
        
        // Create the queue first
        createQueue(jndiName);
        
        checkQueueMetricsAfterScheduledMessage(jndiName);
    }
    
    /**
     * Test Name: testQueueMetricsAfterScheduledMessagesUsingExistingServiceFile
     * Assertion: Verify that queue metrics are correct after creating
     * a scheduled message for the queue. Use a queue that already exists.
     */
    public void testQueueMetricsAfterScheduledMessagesUsingExistingServiceFile() throws Exception {
        String jndiName = "QueueMetricsScheduledMessagesExistingQueue";
        
        expandNavTreeArrow(JMS_NAV_LABEL); 
        checkQueueMetricsAfterScheduledMessage(jndiName);
    }
    
    /**
     * Common code for the testQueueMetricsAfterScheduledMessage* tests.
     */
    private void checkQueueMetricsAfterScheduledMessage(String jndiName) throws Exception {
        
        // Create a queue receiver and then schedule some
        // messages
        QueueSession session = createQueueSession();
        Queue queue = getQueue(jndiName);
        createQueueReceiver(session, queue);
        sendScheduledMessages(session, queue, 2);
        
        // Set up the expected values
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(SCHEDULED_MSG_COUNT, "2");
        expectedMetrics.put(COUNT, "2");
        //expectedMetrics.put(COUNT_DELTA, "2");
        expectedMetrics.put(CONSUMER_COUNT, "1");
        expectedMetrics.put(MSG_COUNT, "2");
        expectedMetrics.put(DELIVERING_COUNT, "0");
        
        checkDestinationMetrics(jndiName, expectedMetrics, getQueueSummaryMetrics(),
                                DestinationType.QUEUE);
    }
    
    /**
     * Test Name: testQueueMetricsTimeLastUpdate
     * Assertion: Verify that the "Time Last Update" metric for queues gets
     * displayed appropriately.
     */
    public void testQueueMetricTimeLastUpdate() throws Exception {
        String jndiName = "QueueMetricsCheckTimeLastUpdate";
           
        // Create the queue first
        createQueue(jndiName);
        checkQueueMetricTimeLastUpdate(jndiName);
    }
    
    /**
     * Test Name: testQueueMetricsTimeLastUpdateUsingExistingServiceFile
     * Assertion: Verify that the "Time Last Update" metric for queues gets
     * displayed appropriately. Use a queue that already exists.
     */
    public void testQueueMetricTimeLastUpdateUsingExistingServiceFile() throws Exception {
        String jndiName = "QueueMetricsTimeLastUpdateExistingQueue";
           
        expandNavTreeArrow(JMS_NAV_LABEL); 
        checkQueueMetricTimeLastUpdate(jndiName);
    }
    
    /**
     * Common code for the testQueueMetricsTimeLastUpdate* tests.
     */
    private void checkQueueMetricTimeLastUpdate(String jndiName) throws Exception {
        
        // Send a message to the queue
        createQueueSessionAndSendMessages(1, jndiName);
        
        ArrayList<String> expectedMetric = new ArrayList<String>();
        expectedMetric.add(TIME_LAST_UPDATE);
        
        checkResourceMetricsNotNull(JMS_NAV_LABEL, DestinationType.QUEUE.getNavLabel(), 
                                    jndiName, expectedMetric.iterator(),
                                    ZERO_VALUE_MESSAGE,
                                    "0");
        // Clean up
        disconnect();
        deleteDestination(DestinationType.QUEUE, jndiName);    
    }
}
