/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.jmsDestinations;

import com.gargoylesoftware.htmlunit.html.*;
import junit.framework.Test;
import org.jboss.jopr.jsfunit.*;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import javax.jms.Topic;
import javax.jms.TopicSession;
import javax.jms.Queue;
import javax.jms.QueueSession;
import javax.jms.JMSSecurityException;
import java.util.Map;
import java.util.LinkedHashMap;

/**
 * This class tests topics and queues that have security roles configured.
 * This class should be run against JBAS 5.x.
 * 
 * @author Farah Juma
 */
public class JMSSecurityTest extends JMSTest {
    
    // A pre-defined JMS user from messaging/hsqldb-persistence-service.xml 
    private static final String JMS_USER = "dynsub";
    
    // A pre-defined security role name from messaging/hsqldb-persistence-service.xml 
    private static final String ROLE_NAME = "publisher";
    
    /**
     * Test Name: testSendMessageToSecureTopicAndSpecifyUser
     * Assertion: Verify the ability to send a message to a topic that has
     * a security role configured. 
     */
    public void testSendMessageToSecureTopicAndSpecifyUser() throws Exception {
        String jndiName = "TopicWithSecurityRoleSpecifyUser";
        String expectedMessage = ADD_MESSAGE + DestinationType.TOPIC.getName();

        createDestinationWithSecurityRole(DestinationType.TOPIC, TOPIC_DEFAULT_TEMPLATE, 
                                          jndiName, expectedMessage, TOPIC_COMPONENT_TYPE,
                                          ROLE_NAME, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE);
        sendMessageToSecureTopicAndSpecifyUser(jndiName, JMS_USER, JMS_USER);   
    }

    /**
     * Test Name: testSendMessageToSecureTopicAndSpecifyUserUsingExistingServiceFile
     * Assertion: Verify the ability to send a message to a topic that has
     * a security role defined. Use a topic that already exists.
     */
    public void testSendMessageToSecureTopicAndSpecifyUserUsingExistingServiceFile() throws Exception {
        String jndiName = "ExistingTopicWithSecurityRoleUserSpecified";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        sendMessageToSecureTopicAndSpecifyUser(jndiName, JMS_USER, JMS_USER);
    }
    
    /**
     * Test Name: testSendMessageToSecureTopicAndSpecifyInvalidUser
     * Assertion: Attempt to send a message to a secure topic, specifying an
     * invalid username. Make sure a JMSSecurityException occurs.
     */
    public void testSendMessageToSecureTopicAndSpecifyInvalidUser() throws Exception {
        String jndiName = "TopicWithSecurityRoleSpecifyInvalidUser";
        String expectedMessage = ADD_MESSAGE + DestinationType.TOPIC.getName();
        Exception jmsSecurityException = null;

        createDestinationWithSecurityRole(DestinationType.TOPIC, TOPIC_DEFAULT_TEMPLATE, 
                                          jndiName, expectedMessage, TOPIC_COMPONENT_TYPE,
                                          ROLE_NAME, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE);
        
        // Specify an invalid user when creating the topic connection
        try {
            sendMessageToSecureTopicAndSpecifyUser(jndiName, "fakeUser", "fakePassword");
        } catch (JMSSecurityException e) {
            jmsSecurityException = e;
        } finally {
            assertNotNull("Expected a JMSSecurityException to be thrown", jmsSecurityException);
            deleteDestination(DestinationType.TOPIC, jndiName);  
        }
    }

    /**
     * Test Name: testSendMessageToSecureTopicAndSpecifyInvalidUserUsingExistingServiceFile
     * Assertion: Attempt to send a message to a secure topic, specifying an
     * invalid username. Make sure a JMSSecurityException occurs. Use a topic
     * that already exists.
     */
    public void testSendMessageToSecureTopicAndSpecifyInvalidUserUsingExistingServiceFile() throws Exception {
        String jndiName = "ExistingTopicWithSecurityRoleInvalidUserSpecified";
        Exception jmsSecurityException = null;
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        
        // Specify an invalid user when creating the topic connection
        try {
            sendMessageToSecureTopicAndSpecifyUser(jndiName, "fakeUser", "fakePassword");
        } catch (JMSSecurityException e) {
            jmsSecurityException = e;
        } finally {
            assertNotNull("Expected a JMSSecurityException to be thrown", jmsSecurityException);
            deleteDestination(DestinationType.TOPIC, jndiName);  
        }
    }

    /**
     * Attempt to send a message to the topic given by jndiName,
     * specifying the given username and password when creating the
     * topic connection. If the username and password are not valid,
     * a JMSSecurityException will be thrown.
     */
    private void sendMessageToSecureTopicAndSpecifyUser(String jndiName, 
                                                        String username, 
                                                        String password) throws Exception {

        TopicSession session = createTopicSessionAndSpecifyUser(username, password);
        Topic topic = getTopic(jndiName);

        createDurableTopicSubscriber(session, topic, jndiName + "Subscriber");
        publishMessages(session, topic, 1);
        
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(ALL_MSG_COUNT, "1");
        expectedMetrics.put(ALL_SUB_COUNT, "1");
          
        checkDestinationMetrics(jndiName, expectedMetrics, getTopicSummaryMetrics(),
                                DestinationType.TOPIC);
    }

    /**
     * Test Name: testSendMessageToSecureTopicWithoutSpecifyingUser
     * Assertion: Make sure an error occurs when trying to send a message, without
     * specifying a user, to a topic that has a security role configured.  
     * 
     * This test should pass if a JMSSecurityException is thrown.
     */
    public void testSendMessageToSecureTopicWithoutSpecifyingUser() throws Exception {
        String jndiName = "TopicWithSecurityRoleNoUserSpecified";
        String expectedMessage = ADD_MESSAGE + DestinationType.TOPIC.getName();
        
        createDestinationWithSecurityRole(DestinationType.TOPIC, TOPIC_DEFAULT_TEMPLATE, 
                                          jndiName, expectedMessage, TOPIC_COMPONENT_TYPE,
                                          ROLE_NAME, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE);

        sendMessageToSecureTopicWithoutSpecifyingUser(jndiName);
    }
    
    /**
     * Test Name: testSendMessageToSecureTopicWithoutSpecifyingUserUsingExistingServiceFile
     * Assertion: Make sure an error occurs when trying to send a message, without
     * specifying a user, to a topic that has a security role configured. 
     * Use a topic that already exists.
     * 
     * This test should pass if a JMSSecurityException is thrown.
     */
    public void testSendMessageToSecureTopicWithoutSpecifyingUserUsingExistingServiceFile() throws Exception {
        String jndiName = "ExistingTopicWithSecurityRoleNoUserSpecified";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        sendMessageToSecureTopicWithoutSpecifyingUser(jndiName);
    }
    
    /**
     * Common code for the testSendMessageToSecureTopicWithoutSpecifyingUser* tests.
     */
    private void sendMessageToSecureTopicWithoutSpecifyingUser(String jndiName) throws Exception {
        
        Exception jmsSecurityException = null;
        
        // We should not be able to send a message since no user was specified when
        // creating the topic connection
        try {
            createTopicSessionAndSendMessages(1, jndiName);
        } catch (JMSSecurityException e) {
            jmsSecurityException = e;
        } finally {
            assertNotNull("Expected a JMSSecurityException to be thrown", jmsSecurityException);
            deleteDestination(DestinationType.TOPIC, jndiName);    
        }
    }
    
    /**
     * Test Name: testSendMessageToSecureQueueAndSpecifyUser
     * Assertion: Verify the ability to send a message to a queue that has
     * a security role configured. 
     */
    public void testSendMessageToSecureQueueAndSpecifyUser() throws Exception {
        String jndiName = "QueueWithSecurityRoleSpecifyUser";
        String expectedMessage = ADD_MESSAGE + DestinationType.QUEUE.getName();

        createDestinationWithSecurityRole(DestinationType.QUEUE, QUEUE_DEFAULT_TEMPLATE, 
                                          jndiName, expectedMessage, QUEUE_COMPONENT_TYPE,
                                          ROLE_NAME, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE);
        sendMessageToSecureQueueAndSpecifyUser(jndiName, JMS_USER, JMS_USER);
    }
    
    /**
     * Test Name: testSendMessageToSecureQueueAndSpecifyUserUsingExistingServiceFile
     * Assertion: Verify the ability to send a message to a queue that has
     * a security role defined. Use a queue that already exists.
     */
    public void testSendMessageToSecureQueueAndSpecifyUserUsingExistingServiceFile() throws Exception {
        String jndiName = "ExistingQueueWithSecurityRoleUserSpecified";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        sendMessageToSecureQueueAndSpecifyUser(jndiName, JMS_USER, JMS_USER);
    }
    
    /**
     * Test Name: testSendMessageToSecureQueueAndSpecifyInvalidUser
     * Assertion: Attempt to send a message to a secure queue, specifying an
     * invalid username. Make sure a JMSSecurityException occurs. 
     */
    public void testSendMessageToSecureQueueAndSpecifyInvalidUser() throws Exception {
        String jndiName = "QueueWithSecurityRoleSpecifyInvalidUser";
        String expectedMessage = ADD_MESSAGE + DestinationType.QUEUE.getName();
        Exception jmsSecurityException = null;

        createDestinationWithSecurityRole(DestinationType.QUEUE, QUEUE_DEFAULT_TEMPLATE, 
                                          jndiName, expectedMessage, QUEUE_COMPONENT_TYPE,
                                          ROLE_NAME, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE);
        
        // Specify an invalid username when creating the queue connection
        try {
            sendMessageToSecureQueueAndSpecifyUser(jndiName, "fakeUser", "fakePassword");
        } catch (JMSSecurityException e) {
            jmsSecurityException = e;
        } finally {
            assertNotNull("Expected a JMSSecurityException to be thrown", jmsSecurityException);
            deleteDestination(DestinationType.QUEUE, jndiName);    
        }
    }
    
    /**
     * Test Name: testSendMessageToSecureQueueAndSpecifyInvalidUserUsingExistingServiceFile
     * Assertion: Attempt to send a message to a secure queue, specifying an
     * invalid username. Make sure a JMSSecurityException occurs. Use a queue
     * that already exists. 
     */
    public void testSendMessageToSecureQueueAndSpecifyInvalidUserUsingExistingServiceFile() throws Exception {
        String jndiName = "ExistingQueueWithSecurityRoleInvalidUserSpecified";
        Exception jmsSecurityException = null;
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        
        // Specify an invalid username when creating the queue connection
        try {
            sendMessageToSecureQueueAndSpecifyUser(jndiName, "fakeUser", "fakePassword");
        } catch (JMSSecurityException e) {
            jmsSecurityException = e;
        } finally {
            assertNotNull("Expected a JMSSecurityException to be thrown", jmsSecurityException);
            deleteDestination(DestinationType.QUEUE, jndiName);    
        }
    }

    /**
     * Attempt to send a message to the queue given by jndiName,
     * specifying the given username and password when creating the
     * queue connection. If the username and password are not valid,
     * a JMSSecurityException will be thrown.
     */
    private void sendMessageToSecureQueueAndSpecifyUser(String jndiName, 
                                                        String username, 
                                                        String password) throws Exception {

        QueueSession session = createQueueSessionAndSpecifyUser(username, password);
        Queue queue = getQueue(jndiName);

        sendMessages(session, queue, 1);
        
        Map<String, String> expectedMetrics = new LinkedHashMap<String, String>();
        expectedMetrics.put(MSG_COUNT, "1");
        expectedMetrics.put(CONSUMER_COUNT, "0");
           
        checkDestinationMetrics(jndiName, expectedMetrics, getQueueSummaryMetrics(),
                                DestinationType.QUEUE);
    }
    
    /**
     * Test Name: testSendMessageToSecureQueueWithoutSpecifyingUser
     * Assertion: Make sure an error occurs when trying to send a message, without
     * specifying a user, to a queue that has a security role configured.
     * 
     * This test should pass if a JMSSecurityException is thrown.
     */
    public void testSendMessageToSecureQueueWithoutSpecifyingUser() throws Exception {
        String jndiName = "QueueWithSecurityRoleNoUserSpecified";
        String expectedMessage = ADD_MESSAGE + DestinationType.QUEUE.getName();
        
        createDestinationWithSecurityRole(DestinationType.QUEUE, QUEUE_DEFAULT_TEMPLATE, 
                                          jndiName, expectedMessage, QUEUE_COMPONENT_TYPE,
                                          ROLE_NAME, Boolean.TRUE, Boolean.TRUE, Boolean.TRUE);

        sendMessageToSecureQueueWithoutSpecifyingUser(jndiName);
    }
    
    /**
     * Test Name: testSendMessageToSecureQueueWithoutSpecifyingUserUsingExistingServiceFile
     * Assertion: Make sure an error occurs when trying to send a message, without
     * specifying a user, to a queue that has a security role configured.
     * Use a queue that already exists.
     * 
     * This test should pass if a JMSSecurityException is thrown.
     */
    public void testSendMessageToSecureQueueWithoutSpecifyingUserUsingExistingServiceFile() throws Exception {
        String jndiName = "ExistingQueueWithSecurityRoleNoUserSpecified";
        
        expandNavTreeArrow(JMS_NAV_LABEL);
        sendMessageToSecureQueueWithoutSpecifyingUser(jndiName);
    }
    
    /**
     * Common code for the testSendMessageToSecureQueueWithoutSpecifyingUser* tests.
     */
    private void sendMessageToSecureQueueWithoutSpecifyingUser(String jndiName) throws Exception {
        
        Exception jmsSecurityException = null;
        
        // We should not be able to send a message since no user is specified when
        // creating the queue connection
        try {
            createQueueSessionAndSendMessages(1, jndiName);
        } catch (JMSSecurityException e) {
            jmsSecurityException = e;
        } finally {
            assertNotNull("Expected a JMSSecurityException to be thrown", jmsSecurityException);
            deleteDestination(DestinationType.QUEUE, jndiName);    
        }
    }
}
