package org.jboss.jopr.jsfunit.util;

import org.jboss.jopr.jsfunit.*;
import org.apache.commons.lang.StringUtils;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jsfunit.jsfsession.JSFClientSession;
import org.jboss.logging.*;


/**
 * Actively waits for a given condition.
 * @see Condition
 */
public class ActiveConditionChecker {

  protected final Logger log = Logger.getLogger(this.getClass().getName());

	Condition condition;
	private boolean dumpPageOnTimout = false;
	private boolean throwExceptionOnTimout = false;

	// Used when dumping the page in case of exceeded timeout.
	private String dumpFileName;
	private JSFClientSession client;


	public ActiveConditionChecker(Condition condition) {
		this.condition = condition;
	}

	
	@SuppressWarnings("empty-statement")
	public boolean waitWithTimeout( int msInterval, int retries ) throws Exception {

		log.info("Waiting for condition: "+condition.getDescription());

		while( retries-- > 0 ){
			if( this.condition.isTrue() ){
				log.info("  Condition '"+condition.getDescription()+"' satisfied.");
				return true;
			}

			if( retries <= 0){
				log.info("  Condition '"+condition.getDescription()+"' not satisfied, "
								+retries+" retries left. Leaving the wait loop.");
				break;
			}


			log.info("  Condition '"+condition.getDescription()+"' not satisfied, "
							+retries+" retries left. Sleeping for "+msInterval+" ms.");
			try {
				Thread.sleep(msInterval);
			} catch (InterruptedException ex){
				;
			}
		}

		// While ended, thus we reached the "timeout".

		if( this.dumpPageOnTimout ){
			try {
				// Since this is still inner class, we could use 'this'.
				log.info("Dumping page text to: "+this.dumpFileName);
				DebugUtils.writeFile("target/"+this.dumpFileName, this.client.getPageAsText());///
			} catch (Exception ex) { log.error("Can't dump page.", ex); }
		}

		if( this.throwExceptionOnTimout ){
			String message = "Timeout expired while waiting for condition: "+condition.getDescription();
			throw new EmbJoprTestException(message);
		}

		return false;
	}

	/** If called, exception is thrown if the timeout expires. */
	public ActiveConditionChecker throwOnTimeout( /*String conditionDesc*/ ){
		this.throwExceptionOnTimout = true;
		//this.conditionDesc = conditionDesc;
		return this;
	}

	// This has nothing to do with condition checking, but is convenient shorthand...
	public ActiveConditionChecker dumpPageOnTimeout(EmbjoprTestCase test) {
		if( null == test ) return this;
		return dumpPageOnTimeout( test.getName()+"-timeout.html", test.getClient() );
	}

	// This has nothing to do with condition checking, but is convenient shorthand...
	public ActiveConditionChecker dumpPageOnTimeout(String dumpFileName, JSFClientSession client) {
		this.dumpPageOnTimout = true;
		this.dumpFileName = dumpFileName;
		this.client = client;
		return this;
	}


}// class


