package org.jboss.jopr.jsfunit.util.jaas;

import java.lang.reflect.Proxy;

import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.deployers.spi.management.ManagementView;
import org.jboss.deployers.spi.management.deploy.DeploymentManager;
import org.jboss.logging.Logger;
import org.jboss.profileservice.spi.ProfileService;



/**
 * Singleton class to keep JAAS secured ProfileService, ManagementView and DeploymentManager.
 * Partially copied from Jopr's test.EjbClient.
 * @author Ondrej Zizka
 *
 */
public class SecuredProfileServiceAccess {

	private static final Logger log = Logger.getLogger(SecuredProfileServiceAccess.class);

	// Singleton.
	private static SecuredProfileServiceAccess INSTANCE_SECURED = null;
	private static SecuredProfileServiceAccess INSTANCE_UNSECURED = null;

	public static SecuredProfileServiceAccess getInstance() throws NamingException { return getSecuredInstance(); }

	public static SecuredProfileServiceAccess getSecuredInstance() throws NamingException {
		if( null == INSTANCE_SECURED ){
			INSTANCE_SECURED = new SecuredProfileServiceAccess();
			INSTANCE_SECURED.init( true );
		}
		return INSTANCE_SECURED;
	}

	public static SecuredProfileServiceAccess getUnsecuredInstance() throws NamingException {
		if( null == INSTANCE_UNSECURED ){
			INSTANCE_UNSECURED = new SecuredProfileServiceAccess();
			INSTANCE_UNSECURED.init( false );
		}
		return INSTANCE_UNSECURED;
	}

	private static final String JNDI_LOGIN_INITIAL_CONTEXT_FACTORY = "org.jboss.security.jndi.JndiLoginInitialContextFactory";
	private static final String NAMING_CONTEXT_FACTORY = "org.jnp.interfaces.NamingContextFactory";
	private static final String SECURE_PROFILE_SERVICE_JNDI_NAME = "SecureProfileService/remote";
	private static final String SECURE_MANAGEMENT_VIEW_JNDI_NAME = "SecureManagementView/remote";
	private static final String SECURE_DEPLOYMENT_MANAGER_JNDI_NAME = "SecureDeploymentManager/remote";
	private static final String PROFILE_SERVICE_JNDI_NAME = "ProfileService";
	private static final String MANAGEMENT_VIEW_JNDI_NAME = "ManagementView";
	private static final String DEPLOYMENT_MANAGER_JNDI_NAME = "DeploymentManager";
	private static final String PROFILE_SERVICE_PRINCIPAL = "admin";
	private static final String PROFILE_SERVICE_CREDENTIALS = "admin";

	
	ProfileService profileService;
	ManagementView managementView;
	DeploymentManager deploymentManager;

	public DeploymentManager getDeploymentManager() {		return deploymentManager;	}
	public ManagementView getManagementView() {		return managementView;	}
	public ProfileService getProfileService() {		return profileService;	}


	/**
	 * Obtains ProfileService, ManagementView and DeploymentManager.
	 * @param useJaas  If true, all those three are secured with JAAS.
	 */
	private void init(boolean useJaas) throws NamingException {

		Properties env = new Properties();
		env.setProperty(Context.PROVIDER_URL, "jnp://127.0.0.1:1099/");
		env.setProperty(Context.INITIAL_CONTEXT_FACTORY, NAMING_CONTEXT_FACTORY);

		InitialContext initialContext = createInitialContext(env);


		if (!useJaas) {
			this.profileService = (ProfileService) lookup(initialContext, PROFILE_SERVICE_JNDI_NAME);
			this.managementView = this.profileService.getViewManager();
			this.deploymentManager = this.profileService.getDeploymentManager();
		}
		else {
			this.profileService = (ProfileService) lookup(initialContext, PROFILE_SERVICE_JNDI_NAME);
			this.managementView = (ManagementView) lookup(initialContext, MANAGEMENT_VIEW_JNDI_NAME);
			this.deploymentManager = (DeploymentManager) lookup(initialContext, DEPLOYMENT_MANAGER_JNDI_NAME);

			//apply the jaas security
			ClassLoader classLoader = SecuredProfileServiceAccess.class.getClassLoader();
			this.profileService = (ProfileService) Proxy.newProxyInstance(classLoader,
							new Class<?>[]{ProfileService.class},
							new JaasAuthenticationInvocationHandler( this.profileService, PROFILE_SERVICE_PRINCIPAL, PROFILE_SERVICE_CREDENTIALS ));

			this.managementView = (ManagementView) Proxy.newProxyInstance(classLoader,
							new Class<?>[]{ManagementView.class},
							new JaasAuthenticationInvocationHandler( this.managementView, PROFILE_SERVICE_PRINCIPAL, PROFILE_SERVICE_CREDENTIALS ));

			this.deploymentManager = (DeploymentManager) Proxy.newProxyInstance(classLoader,
							new Class<?>[]{DeploymentManager.class},
							new JaasAuthenticationInvocationHandler( this.deploymentManager, PROFILE_SERVICE_PRINCIPAL, PROFILE_SERVICE_CREDENTIALS ));
		}

		//  This must be called before the other ManagementView operations may be used.
		this.managementView.load();
	}


	private static InitialContext createInitialContext(Properties env) throws NamingException {
		log.info("Creating JNDI InitialContext with env [" + env + "]...");
		InitialContext initialContext = new InitialContext(env);
		log.info("Created JNDI InitialContext [" + initialContext + "].");
		return initialContext;
	}

	private static Object lookup(InitialContext initialContext, String name) throws NamingException {
		log.info("Looking up name '" + name + "' from InitialContext...");
		Object obj = initialContext.lookup(name);
		log.info("Found Object: " + obj);
		return obj;
	}
}// class
